/*--------------------------------------------------------------------------------- *
 * DVX : Dart Vxworks eXtensions						    *
 *--------------------------------------------------------------------------------- *
 * FILE:
 *       dvx.h     DVX : Dart Vxworks eXtensions Task Variable facility.
 *                           Include file used by dvx_var.c
 *
 * -------------------------------------------------------------------------
 * AUTHORS:
 * 
 *      Margherita Vittone
 *
 * Mods: 
 *	M.Vittone: adding a field in the private data structure to store  
 *		   size of malloced space information.
 * -------------------------------------------------------------------------------- */

#include "vxWorks.h"
#include "stdio.h"
#include "stdioLib.h"
#include "taskLib.h"
#include "taskVarLib.h"
#include "taskHookLib.h"
#include "logLib.h"
#include "ioLib.h"
#include "errno.h"

/* -------------------------------------------------------------------------------- */

#define  DVX_VAR_START	 1		    /* Used to check on DVX variable package to be
					       initialized in the system			 */

#define  DVX_MAX_VAR	 32		    /* Max number of dvx task variables for 
					       DVX task variables			 	 */

#define  DVX_INVAR_CNT    2		    /* Number of INternal task variables automatically   
					       created by dvx when the task is spawned, not 
					       explicitly created by the user.			 */

#define  DVX_MAX_XHAND	 32		    /* Max number of exit handlers	    		 */
#define  DVX_MAX_SOCKET	 10		    /* Max number of SOCKETS per task	    	         */

/* Error status codes */

#define  DVX_TABLE_FULL	 -1		    /* Table to register function is full  		 */

#define  DVX_VAR_FULL	 -1		    /* Max number of dvx task var is reached 		 */
#define  DVX_VAR_INVALID -2		    /* Invalid dvx handle for task var 	     		 */
#define	 DVX_NO_TASKVAR  -3		    /* Handle ok  but no task variable associatet to it  */
#define	 DVX_NO_HANDLE	 -4		    /* Handle does not exist				 */
#define  DVX_INFO_ERROR  -5		    /* Error status from dvx_var_info_get		 */
#define  DVX_INFO_NOVAR  -6		    /* No task var found, from dvx_var_info_get		 */

/* Task variable type */

#define  DVX_SCALAR_TYPE 0x01		    /* To be used to identify a scalar variable  */

#define  DVX_PTR_TYPE	 0x10		    /* To be used to identify a pointer variable  */

#define  DVX_MAL_PTR	 0x100		    /* To be used to identify malloced spaces    */

#define  DVX_XHAND_PTR   0x1000		    /* Used to 	specify exit handler data type  */

#define  DVX_SOCKET_PTR  0x10000	    /* Used to 	specify socket data type  */

#define  DVX_TASK_VAR	 0x100000	    /* To identify a variable as a task variable  */

#define  DVX_TASK_INTERNAL_VAR 0x1000000    /* To identify a variable as a task variable  */
					    /* internally created for the task		  */


#define	 DVX_CREATE_HANDLE	 0		    /* Used to check on valid handle		  */
#define	 DVX_VERIFY_HANDLE	 1		    /* Used to check on valid handle		  */
#define  DVX_OLD_HANDLE		 1
#define  DVX_NEW_HANDLE		 0

#define  SCALAR_TYPE		 0
#define	 PTR_TYPE		 1

/*----------------------------------------------------------------------------------------------- */
typedef  void	DVX_PTR;	    

typedef unsigned int	DVX_SOCK_PTR;

extern char    *dvxVarType[];			/* Definded in dvx_var.c; used to 
						   distinguish between a scalar and pointer.      */
/* ---------------------------------------------------------------------------------------------- */

/* Structure to hold exit handlers registered with atexit call: 
   next is the index that is updated at every new atexit call. 
   The array holds the function pointers of the registered functions 		     */

struct tskfun_atexit {
		int next;	    	     	     /* To point to next element in array */
 		void (*funArrPtr[DVX_MAX_XHAND])(void);  /* Buffer to hold function addresses */
	      };

/* Structure to hold exit handlers registered with dvx_atexit call: 
   next is the index that is updated at every new atexit call;
   xhand_dat is the data passed to the exit handler (typically the task id).
   The array holds the function pointers of the registered functions 		     */


struct tskfun_dvx_atexit {
		int next;	    	     	     /* To point to next element in array */
		int xhand_dat[DVX_MAX_XHAND];	     /* Data passed to exit handler	  */
 		int (*funArrPtr[DVX_MAX_XHAND])(int);  /* Buffer to hold function addresses */
	      };

/* Structure definition for task private data			    			*/

struct prvData{
		 int	pdpVal;		    /* To store variable value			*/
		 int    pdpType;	    /* To distinguish type of variable       	*/ 
		 int	pdpSize;	    /* To store size of malloced space		*/
	       };


struct tsksock {
                int nextSock;               /* To point to next element in array 	*/
		DVX_PTR *sockPtr[DVX_MAX_SOCKET];    /* Socket address			*/
		};

/* --------------------------------------------------------------------------------- */

/* Function prototypes declarations	*/
 

STATUS   	dvx_init_verify(void);

STATUS 	 	dvx_setup(void);

STATUS		dvx_create_scalar (int tId, int *dvxHandle);
STATUS		dvx_set_scalar (int tId, int dvxHandle, int varVal);
STATUS		dvx_get_scalar (int tId, int dvxHandle, int *rdval);

STATUS		dvx_create_ptr (int tId, int *dvxHandle, int malsiz, DVX_PTR **varAdd );
STATUS		dvx_get_ptr (int tId, int dvxHandle, DVX_PTR **rdptr);

/* STATUS   	dvx_add_socket ( int tId, DVX_SOCK_PTR *sockAddr); */

STATUS		dvx_verify_taskvar (int tId, int dvxHandle, int varType);

STATUS		dvx_delete_taskvar (int tId, int dvxHandle);

STATUS		dvx_var_info (int tId );

STATUS		dvx_atexit (int (*regFunc)(int), int );
/* --------------------------------------------------------------------------------- */
/* --------------------------------------------------------------------------------- */
