/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1991 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/

/* ==========================================================================
**
** DESCRIPTION
**      header files - has macros for simple timing functions on MVMEXXX
**	boards (mvme167 and mvme162 etc.).
**
** DEVELOPERS
**      Ron Rechenmcher
**
** MODIFICATIONS - modification history (if any) should be kept in source
**                 code management system. (don't clutter this file)
**
**		   Nov 13th,1995 : Generalized the file to support
**				   different mvme boards.
**				   M.Vittone
*/

/* include from the ./config/mvNNN directory under the vxWorks root */
#include "types.h"

#include "vxt.h"

#if defined(mv167)

#include "mv167.h"		/* include vmechip2, pccchip2, etc */

#elif defined(mv162)
#include "mv162.h"		/* include vmechip2, pccchip2, etc */

#endif

struct dma_chain
{   long		Cont;		/* Control word			*/
    caddr_t		Local_A;	/* Local address		*/
    caddr_t		VME_A;		/* Remote address		*/
    unsigned int	Nbytes;		/* Transfer size in bytes	*/
    struct dma_chain	*Next;		/* Pointer to next in chain	*/
};
#define DMACHAIN_END (struct dma_chain *)1	/* End of DMA chain	*/


#define GPIO_INT_HIGH		0
#define GPIO_INT_RISING		1
#define GPIO_INT_LOW		2
#define GPIO_INT_FALLING	3

#define GPIO_MODE_INPUT		0
#define GPIO_MODE_OUTPUT	1
#define GPIO_SET_LOW		0
#define GPIO_SET_HIGH		1

/* constants for vmeMasterMapnAM calls */

#define SUP_BLK         0x0F
#define SUP_PRG         0x0E
#define SUP_DAT         0x0D
#define SUP_D64         0x0C

#define USR_BLK         0x0B
#define USR_PRG         0x0A
#define USR_DAT         0x09
#define USR_D64         0x08

/* dma stuff */

#define FROMVME         (0<<0)
#define TOVME           (1<<0)
#define NO_ADDR_INC_VME	(1<<1)
#define NO_ADDR_INC_LB	(1<<2)
#define D16             0
#define D32             1
#define D32BLK		2
#define D64BLK		3

#define DMA_NOT_DONE	0
#define DMA_DONE_OK	1
#define DMA_DONE_ERROR(x)  (   (x==ICR_VME)  || (x==ICR_TBL)  || (x==ICR_DLTO)\
			    || (x==ICR_DLOB) || (x==ICR_DLPE) || (x==ICR_DLBE) )
#define DMA_DONE(x)  ( (x==ICR_DONE)||(x==ICR_VME)||(x==ICR_TBL)||(x==ICR_DLTO)\
		    || (x==ICR_DLOB)||(x==ICR_DLPE) || (x==ICR_DLBE) )

				/* THIS WILL WORK ACROSS REGISTER ROLL OVER */
#define clk1Diff(first_rd)	((unsigned)*VMECHIP2_TTCOUNT1 - (unsigned)first_rd)
#define	clk1Read		*VMECHIP2_TTCOUNT1

#define clk2Diff(first_rd)	((unsigned)*VMECHIP2_TTCOUNT2 - (unsigned)first_rd)
#define	clk2Read		*VMECHIP2_TTCOUNT2

/* could be used to see if a tick is about to happen - could be helpful in
   determining wether to delay 0,1,or 2 ticks (for example) */
#define sysClkRead		*PCC2_TIMER1_CNT
#define sysAuxClkRead		*PCC2_TIMER2_CNT

/* These macros should be removed because they would only make sense of the
   clock periods were 0xffffffff microseconds - I'll "if 0" them out and 
   remove them next time */
#if 0
#define sysClkDiff(first_rd)	((unsigned)*PCC2_TIMER1_CNT - (unsigned)first_rd)
#define sysAuxClkDiff(first_rd)	((unsigned)*PCC2_TIMER2_CNT - (unsigned)first_rd)
#endif

/* provides approx 16 seconds of difference - always running a full 24 bit 
   cycle.  (This may not be true of the other clocks if they are used to 
   generate interrupts) */
#define clkuDiff(frst)		((  ((unsigned)*VMECHIP2_PRESCALE_CNT>>8) \
				  -  (unsigned)frst               )&0x00ffffff)
#define clkuRead		((unsigned)*VMECHIP2_PRESCALE_CNT >> 8)

#define LEVEL_SENSITIVE_HIGH	0
#define EDGE_SENSITIVE_RISING	1
#define LEVEL_SENSITIVE_LOW	2
#define EDGE_SENSITIVE_FALLING	3


