/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1992 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/*
 
File:           sysTools.h
=====

Author:         Jim Pangburn
=======         CDF on-line computing
                Fermilab MS 318
                P.O. Box 500
                Batavia, IL  60510
                (708) 840-2207

        e-mail: "pangburn@fnal.fnal.gov"

Creation date:  4-Feb-1994
==============

Revision history:-
=================

4-Feb-1994      created

3-Aug-1994	M.Vittone: adding conditional fro MC68k family for prototyping
			   under VxWorks v5_1_1.
			   efsShow prototypes for R3k AND MC68k
Description:-
============

	Header file for users to include to use sysTools.


Include files:-
==============
*/


#ifndef systools_h
#define systools_h


#include "sysToolsConfig.h"

#if CPU_FAMILY==MIPS_R3K
#include "drv/lr33000.h"
#define LH_INSTR	0x84000000
#define SH_INSTR	0xa4000000
#endif


#include "types.h"		/* To have time_t defined M. Vittone Mar 20, 95 */

/*
Macros:-
=======
*/

#define SYSTOOLS_TASK_MONITOR	1	/* include task monitor */
#define SYSTOOLS_EXC_HANDLER	2	/* include exception handling */
#define SYSTOOLS_TRACE		4	/* include trace facility */

#define SYSTOOLS_ALL		0xff	/* include everything */



/*
Global declarations:-
====================
*/


extern int	shellTaskId;		/* secret */

time_t		UPTIME;			/* call to time() at system startup */
char		UPTIME_ASCII[26];	/* ASCII string of startup time */

#if CPU_FAMILY==MIPS_R3K
unsigned long	sysToolsSearchSym;	/* global to search for */
unsigned long	sysToolsFoundSym;	/* global for what we found */
unsigned long	sysToolsMinDiff;	/* minimize difference */
#endif

typedef struct install_table
  {
  int vector_number;		/* exception vector number */
  int install_flag;		/* whether or not to install us here */
  FUNCPTR old_handler;		/* (presumably) VxWorks default handler */
  } INSTALL_TABLE;

typedef struct signal_params
  {
  int signal_number;
  int signal_code;
  } SIGNAL_PARAMS;



/*
Prototypes:-
============
*/

/* user-callable routines */

#if CPU_FAMILY==MC680X0
VOID	uptime(void);		/* analog to Unix uptime() */
VOID	crashShow(void);		/* show last crash info */
STATUS	sysToolsInit(void);		/* initialization routine */
#else
VOID	uptime();		/* analog to Unix uptime() */
VOID	crashShow();		/* show last crash info */
STATUS	sysToolsInit();		/* initialization routine */
#endif

int	search( char *start, int len, int inc, unsigned long pattern,
	int bytemask );

/* "internal" routines */
VOID	sysToolsTaskSwitchHook( WIND_TCB *oldTCB, WIND_TCB *newTCB );

VOID	esfShow( volatile unsigned long *esf );

/*
#if CPU_FAMILY==MIPS_R3K
VOID	esfShow( volatile unsigned long *esf );
#endif
*/

#if CPU_FAMILY==MIPS_R3K
VOID	sysToolsR3kExcHandle( int vector, unsigned long esf_par );
STATUS	symFindByValueR3k( SYMTAB_ID symTblId, int value, char *name,
	int *pValue, SYM_TYPE *pType );
BOOL	sysToolsR3kSymEach( char *name, int val, SYM_TYPE type, int arg );
#endif

#if CPU_FAMILY==MC680X0
VOID	sysTools68kExcHandle( unsigned long esf_par );
VOID    sysTools68kTrcPrint(INSTR *callAdrs, int rtnAdrs, int nargs, int *args);
#endif


#endif		/* file already included */



