/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1992 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/*
 
File:           sysTools68k.h
=====

Author:         Jim Pangburn
=======         CDF on-line computing
                Fermilab MS 318
                P.O. Box 500
                Batavia, IL  60510
                (708) 840-2207

        e-mail: "pangburn@fnal.fnal.gov"

Creation date:  17-Feb-1994
==============

Revision history:-
=================

17-Feb-1994      created
02-Aug-1994      M.Vittone: Need to add cast to vector numbers for vxworks

Description:-
============

	680X0-specific data structures.  Users should edit this file and
	compile systools for first-time installation or any time they wish
	to customize which exception conditions the systools exception handler
	should be connected to.  Normally, this should be all exceptions
	to which the user's applications do not connect, except for the
	breakpoint exception, which the VxWorks built-in debugging uses.

*/

#ifndef systools68k_h
#define systools68k_h

/*
Macros:-
=======
*/

#define ESTKSIZE		0x80

#define SYSTEM_CRASH_MESSAGE	EXC_MSG_ADRS
#define SYSTEM_CRASH_MESSAGE_SIZE	0x100
#define SYSTEM_CRASH_REGS	0x900
#define SYSTEM_CRASH_REGS_SIZE	ESTKSIZE
#define SYSTEM_TASK_INFO	(SYSTEM_CRASH_REGS+SYSTEM_CRASH_REGS_SIZE)
#define SYSTEM_TASK_INFO_SIZE	32
#define SYSTOOLS_NVRAM_OFFSET	(TOD_NVRAM_OFFSET+128)
#define SYSTOOLS_NVRAM_CRASH	SYSTOOLS_NVRAM_OFFSET
#define SYSTOOLS_NVRAM_CRASH_SIZE	SYSTEM_CRASH_MESSAGE_SIZE*4
#define SYSTOOLS_NVRAM_REGS     (SYSTOOLS_NVRAM_CRASH+SYSTOOLS_NVRAM_CRASH_SIZE)
#define SYSTOOLS_NVRAM_REGS_SIZE        ESTKSIZE
#define SYSTOOLS_NVRAM_DOWN	(SYSTOOLS_NVRAM_REGS+SYSTOOLS_NVRAM_REGS_SIZE)
#define SYSTOOLS_NVRAM_DOWN_SIZE	4*4
#define SYSTOOLS_NVRAM_TASK	(SYSTOOLS_NVRAM_DOWN+SYSTOOLS_NVRAM_DOWN_SIZE)
#define SYSTOOLS_NVRAM_TASK_SIZE	32*4


#define INSTALL_TABLE_SIZE	15
#define SYSTOOLS_BASE_INT_NUM	2

/* The vector numbers IV_.... are defined in the include file
   ivMc68k.h in the mc68k architecture subdirectory under vxworks. */

#if CPU_FAMILY==MC680X0

INSTALL_TABLE	systoolsExcInstallTable[INSTALL_TABLE_SIZE] =
	{
		{ (int)IV_BUS_ERROR,		TRUE,	NULL },
		{ (int)IV_ADDRESS_ERROR,	TRUE,	NULL },
		{ (int)IV_ILLEGAL_INSTRUCTION,	TRUE,	NULL },
		{ (int)IV_ZERO_DIVIDE,		TRUE,	NULL },
		{ (int)IV_CHK_INSTRUCTION,	TRUE,	NULL },
		{ (int)IV_TRAP_INSTRUCTION,	FALSE,	NULL },
		{ (int)IV_PRIVILEGE_VIOLATION,	TRUE,	NULL },
		{ (int)IV_TRACE,		FALSE,	NULL },
		{ (int)IV_LINE_1010_EMULATOR,	FALSE,	NULL },
		{ (int)IV_LINE_1111_EMULATOR,	FALSE,	NULL },
		{ (int)IV_RESERVED_12,		TRUE,	NULL },
		{ (int)IV_CP_PROTOCOL_VIOLATION,TRUE,	NULL },
		{ (int)IV_FORMAT_ERROR,		TRUE,	NULL },
		{ (int)IV_UNITIALIZED_INTERRUPT,TRUE,	NULL },
		{ (int)IV_SPURIOUS_INTERRUPT,	TRUE,	NULL }
	};

/* The value of signal_code are defined in the include file
   sigMc68kCodes.h in the mc68k architecture subdirectory under vxworks. */

SIGNAL_PARAMS systoolsExcToSigMap[INSTALL_TABLE_SIZE] = 
	{
		{ SIGBUS, (int)SEGV_BUS_ERROR },
		{ SIGBUS, (int)BUS_ADDERR },
		{ SIGILL, (int)ILL_ILLINSTR_FAULT },
		{ 0, 0 },
		{ SIGFPE, (int)FPE_CHKINST_TRAP },
		{ SIGILL, 0 },
		{ SIGILL, (int)ILL_PRIVVIO_FAULT },
		{ SIGTRAP, 0 },
		{ SIGEMT, (int)EMT_EMU1010 },
		{ SIGEMT, (int)EMT_EMU1111 },
		{ SIGILL, (int)ILL_ILLINSTR_FAULT },
		{ SIGILL, (int)ILL_COPROCERR_FAULT },
		{ SIGFMT, 0 },
		{ 0, 0 },
		{ 0, 0 }
	};

#else
INSTALL_TABLE	systoolsExcInstallTable[INSTALL_TABLE_SIZE] =
	{
		{ IV_BUS_ERROR,			TRUE,	NULL },
		{ IV_ADDRESS_ERROR,		TRUE,	NULL },
		{ IV_ILLEGAL_INSTRUCTION,	TRUE,	NULL },
		{ IV_ZERO_DIVIDE,		TRUE,	NULL },
		{ IV_CHK_INSTRUCTION,		TRUE,	NULL },
		{ IV_TRAP_INSTRUCTION,		FALSE,	NULL },
		{ IV_PRIVILEGE_VIOLATION,	TRUE,	NULL },
		{ IV_TRACE,			FALSE,	NULL },
		{ IV_LINE_1010_EMULATOR,	FALSE,	NULL },
		{ IV_LINE_1111_EMULATOR,	FALSE,	NULL },
		{ IV_RESERVED_12,		TRUE,	NULL },
		{ IV_CP_PROTOCOL_VIOLATION,	TRUE,	NULL },
		{ IV_FORMAT_ERROR,		TRUE,	NULL },
		{ IV_UNITIALIZED_INTERRUPT,	TRUE,	NULL },
		{ IV_SPURIOUS_INTERRUPT,	TRUE,	NULL }
	};

SIGNAL_PARAMS systoolsExcToSigMap[INSTALL_TABLE_SIZE] = 
	{
		{ SIGBUS, BUS_BUSERR },
		{ SIGBUS, BUS_ADDERR },
		{ SIGILL, ILL_ILLINSTR_FAULT },
		{ 0, 0 },
		{ SIGFPE, FPE_CHKINST_TRAP },
		{ SIGILL, 0 },
		{ SIGILL, ILL_PRIVVIO_FAULT },
		{ SIGTRAP, 0 },
		{ SIGEMT, EMT_EMU1010 },
		{ SIGEMT, EMT_EMU1111 },
		{ SIGILL, ILL_ILLINSTR_FAULT },
		{ SIGILL, ILL_COPROCERR_FAULT },
		{ SIGFMT, 0 },
		{ 0, 0 },
		{ 0, 0 }
	};

#endif

char		*EXC68K_TXT[INSTALL_TABLE_SIZE] =
		{
		"Bus Error Exception",
		"Address Error Exception",
		"Illegal Instruction Exception",
		"CPU Zero Divide Exception",
		"CHK Instruction Exception",
		"Trap Exception",
		"Privilege Violation Exception",
		"Trace Exception",
		"Line 1010 Emulator Exception",
		"Line 1111 Emulator Exception",
		"Reserved 12 Exception",
		"Coprocessor Protocol Violation Exception",
		"Format Error Exception",
		"Unitialized Interrupt",
		"Spurious Interrupt"
		};


#endif
