/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1992 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/

/* @+Public+@
 * PROJECT:	vx_tools
 *		timerTask.h
 *
 * ABSTRACT:	header file for timerTask
 *
 * ENVIRONMENT:	ANSI C.
 *
 * AUTHOR:      ron,  initial creation: 12-Apr-1995
 *
 * @-Public-@
 ******************************************************************************/
#ifndef TIMER_TASK_INC
#define TIMER_TASK_INC
/*--------------------------------- INCLUDES ---------------------------------*/
#include "vxWorks.h"	/* inc. appropriate "types.h" - STATUS, VOIDFUNCPTR */
#include "msgQLib.h"	/* MSG_Q_ID */

/*-------------------------- GLOBAL DEFINES/DECLARES -------------------------*/
enum e_tmr		/* ALSO USED AS STATUS - internal and return */
{   tmr_e_add
  , tmr_e_abort
  , tmr_e_status
  , tmr_e_add_func	/* timerTask checks additional fields in the msg */
  , tmr_e_func_started	/* internal status only! Not to be sent to timerTask */
  , tmr_e_mid_abort	/* abort based on mid only - risky */
  , tmr_e_tid_abort	/* abort based on tid only - all msg from a task */
};

enum e_tmr_func
{   tmr_func_e_start
  , tmr_func_e_during
  , tmr_func_e_expire
  , tmr_func_e_abort
};

typedef void	(TMRFUNC)( enum e_tmr_func time, int arg);

/* It is intended that struct s_tmr_msg be inserted into a user message
   structure similar to the one below it.  The timerTask will handle receiving
   either a s_tmr_msg_fmt structure (which may actually be larger than the size
   of the structure below as vxworks will ignore the extra size) or a pointer
   to an s_tmr_msg_fmt structure. 

   NOTE: if an pointer system in used, it must be noted that the "status" msg
   may or may not be returned and the abort msg_ptr will never be returned.
   For status, the behavior could be changed to alway return the msg_ptr - in
   the case of an already expired msg, the tick count could be set to 0 or
   negative 1???

   Note: the timerTask will ignore the msg_type in the example s_tmr_msg_fmt
   struct.  Although it is not expected, the user may use the s_tmr_msg_fmt if
   other messages in the system are not already following this structure. It
   is intended though, that the structure will be used and that the system
   will define a msg_type of e_tmr in a larger message type enum, for
   example. */

struct s_tmr_msg
{   enum e_tmr	cmd;		/* either add (to q) or abort */
    int		tick_cnt;	/* filled in by sending task (ignored if cmd
				   is abort or status*/
    int		mid;		/* msg id assigned by sender */
    MSG_Q_ID	return_q;	/* users q (ignored if cmd is abort) */
    int		sts;		/* filled in by timer task - will normally be
				   cmd, but when aborting, it will be either
				   abort or the orignal add cmd */
    int		tid;		/* task id of sending task (used when abort) -
				   gaurd against two different tasks
				   coinicidently assigning the same msg id */
    /* NOTE: these functions can only abort timer messages if they know both
       the mid and tid in the timer msg to be aborted */
    void	(*func_p)( enum e_tmr_func time, int arg);
    int		func_arg;	/* the user could use arg as an index into a
				   global structure of more args */
			/* at expire time, the function could be used to
			   restart same timer msg if true infinite (or at
			   least potentially infinite) timing is desired. For
			   really infinite, set the return mailbox to the
			   timerTask and the expire func could set the tick
			   count to -1. 
			   Because the timerTask does mbox processing, then
			   timing processing, then Delays 1 clock, if the tick
			   count is 0 and the expire func just repeats the
			   message, the function will be executed each clock
			   tick - there will not be an infinite loop was one
			   *may* expect */
};


/* mock message stucture - the user is expect to define his own structure that
   may contain many more message type structures in the union.  The definition
   for msg_type for add and abort are defined by the user and passed as
   parameters to the timerTask. */

struct	s_tmr_msg_fmt
{   int				msg_type;
    union
    {   int		 dmy;
	struct s_tmr_msg tmr;
    }				u;
};

extern	MSG_Q_ID        timer_msg_q;

/*--------------------------------- PROTOTYPES -------------------------------*/
STATUS	timerStart(  MSG_Q_ID	mQ	/* Applications can pass 0  - forcing */
		   , int	q_sz	/*   the use of timer_msg_q, otherwise*/
		   , int	prio );	/*   a user defined message q can be
					     passed  */

STATUS	timerSend(  int		msg_type
		  , int		cmd
		  , int		tick_cnt
		  , int		mid
		  , MSG_Q_ID	return_q );

STATUS	timerSendP(  struct s_tmr_msg_fmt	*msg
		   , int			 cmd
		   , int			 tick_cnt
		   , int			 mid
		   , MSG_Q_ID			 return_q );

STATUS	timerSendF(  int	 msg_type
		   , int	 cmd
		   , int	 tick_cnt
		   , int	 mid
		   , MSG_Q_ID	 return_q
		   , TMRFUNC	*func_p
		   , int	 func_arg );

STATUS	timerSendFP(  struct s_tmr_msg_fmt	*msg
		    , int			 cmd
		    , int			 tick_cnt
		    , int			 mid
		    , MSG_Q_ID			 return_q
		    , TMRFUNC			*func_p
		    , int			 func_arg );

#endif /* TIMER_TASK_INC */
