/************************************************************************/
/* File: tod_ds1386.h							*/
/*									*/
/* This is a Time-Of-Day device driver for the DS1286 and DS1386.	*/
/* It is implemented for a PC3/PC4/PC4a FSCC.				*/
/*									*/
/* Generic MACRO Identifiers:						*/
/*									*/
/*	TOD_DEVICE_ID		Device String Identifier		*/
/*	TOD_DEVICE_ON		Enable Time-Of-Day Clk DS1386		*/
/*	TOD_DEVICE_OFF		Disable Time-Of-Day Clk (save Battery)	*/
/*	TOD_READ_DEVICE		Read the Time-Of-Day DS1386		*/
/*	TOD_WRITE_DEVICE	Write the Time-Of-Day DS1386		*/
/*	TOD_DEVICE_NVRAM	Size of NV RAM in bytes			*/
/*									*/
/* history:								*/
/* 07/06/94	swk	sysNvRamWrite now required for pc4a		*/
/*			CLK_BASE changed to RTC_BASE in vx_fscc		*/
/* 06jan94	das	added sysNvRamWrite prototype			*/
/* 07jan94	das	added type casts willy nilly to satisify IRIX	*/
/*
SCCS release:
@(#) tod_ds1386.h 1.5 Delta: 94/07/06 14:40:17 Extraction: 94/10/24 10:03:23 @(#)
*/

#ifndef INCtod_ds1386
#define INCtod_ds1386

/* from VxWorks sysLib.c        */
extern void    sysNvRamWrite (int we);

/************************************************************************/
/*									*/
/* MACROS BCD Conversions						*/
/*						1			*/
/*			 8421	       BCD	2631			*/
/*			 0000 8421 	vs	84268421		*/
/*			|nnnn|nnnn|   Binary   |nnnnnnnn|		*/
/*									*/
/*	The conversion macros operate on byte values only where the	*/
/*	Binary to BCD conversion is available for bytes values under	*/
/*	100.								*/

#define TOD_BCDtoBIN(n) ((((int)(n) & 0xf0) >> 4) * 10 + ((int)(n) & 0x0f))
#define TOD_BintoBCD(n) ((((int)(n) / 10) << 4) | ((int)(n) % 10))

/************************************************************************/
/*									*/
/* MACRO Device Driver							*/
/*									*/
/* Device:	DS1386 Time Of Day Clock				*/
/*									*/
/* Clock Setup:		24:00 Hour Format				*/
/*			Alarms Not Used					*/
/*			WDG Alarms Not Used				*/
/*			Interrupts Disabled				*/
/*									*/
/* DS1386 Registers: 	(See Dallas Semiconductor Specification)	*/
/*									*/
/*	Address		Bits	|7|6|5|4|3|2|1|0|			*/
/*		0		|0.1Sec |0.01Sec|	not used	*/
/*		1		|0|  Seconds    |	Seconds		*/
/*		2		|0|  Minutes    |	Minutes		*/
/*		3		|M| Alarm Min   |	not used	*/
/*		4	     	|0|F| Hours	|	Format 24 Hours	*/
/*		5		|M| Alarm Hour  |	not used	*/
/*		6		|0|0|0|0|0| Day |	Day of week	*/
/*		7		|M| Alarm Day   |	not used	*/
/*		8		|0|0| Day Date  |	Day of Month	*/
/*		9		|0|1|0| Month   |	OscOn/Month	*/
/*		10		|   Year        |	Year 2 digit	*/
/*		11		|   Cmd Reg     |	Command Reg	*/
/*		12		|WDG alarm 0.01s|	not used	*/
/*		13		|WDG alarm Sec  |	not used	*/
/*		14		|   Usr Reg     |	not used	*/
/*		15		|   Usr Reg     |	not used	*/
/*									*/
/*    _______________________________________________________________	*/
/*   | Bit 7 | Bit 6 | Bit 5 | Bit 4 | Bit 3 | Bit 2 | Bit 1 | Bit 0 |	*/
/*									*/
/*   Hours Register 4							*/
/*    _______________________________________________________________	*/
/*   |   0   | 12/24 |ampm/10|            H O U R S                  |	*/
/*   |   0   |   0   |        H       O      U       R       S       |	*/
/*									*/
/*   Months Register 9							*/
/*    _______________________________________________________________	*/
/*   |EOSC   | ESQW  |   0   |            M O N T H S                |	*/
/*   |   0   |   1   |   0   |            M O N T H S                |	*/
/*									*/
/*   Command Register 10 (FSCC	PC4)					*/
/*    _______________________________________________________________	*/
/*   |TE     |IPSW   |IBH/LO |PU/LVL |WAM    |TDM    |WAF    |TDF    |	*/
/*   |   X   |   0   |   0   |   1   |   1   |   1   |   X   |   X   |	*/
/*									*/
/*									*/

#ifdef DS1286
#define TOD_DEVICE_ID "DS1286\0"	/* Give it a string name */
#define TOD_DEVICE_NVRAM 50		/* Size of NVRAM */
#else
#define TOD_DEVICE_ID "DS1386\0"	/* Give it a string name */
#define TOD_DEVICE_NVRAM 8192		/* Size of NVRAM */
#endif

#ifndef TOD_CLOCK		/* MVME147, MVME167 definition for MK48T08 */
#ifdef RTC_BASE			/* FSCC definition for the DS3186 */
#define TOD_CLOCK RTC_BASE	/* if RTC_BASE, it's an FSCC */
#endif
#endif

/* Got rid here of conditional on ols vxworks */

#define TOD_WRUNLOCK sysNvRamWrite(TRUE);
#define TOD_WRLOCK sysNvRamWrite(FALSE);

#if defined (frc)
#ifndef TOD_REGOFF
#define TOD_REGOFF 4		/* displacement longwords, long=4bytes */
#endif
#endif

#ifndef TOD_REGOFF
#define TOD_REGOFF 1		/* displacement byte=1, long=4 */
#endif

/* on IRIX this macro evaluates to a value with type unsigned int */
#define TOD_BASEOFFSET(n) 	*( (char *) (TOD_CLOCK+(n)) )

#define TOD_DS1386_CNTRL	TOD_BASEOFFSET(0x0b*TOD_REGOFF)	/* Control registerister */
#define TOD_DS1386_01SEC	TOD_BASEOFFSET(0x00*TOD_REGOFF)	/* BCD 0.01 - 0.1 Sec */
#define TOD_DS1386_SEC		TOD_BASEOFFSET(0x01*TOD_REGOFF)	/* BCD Seconds */
#define TOD_DS1386_MIN		TOD_BASEOFFSET(0x02*TOD_REGOFF)	/* BCD Minutes */
#define TOD_DS1386_HRS		TOD_BASEOFFSET(0x04*TOD_REGOFF)	/* BCD Hours */
#define TOD_DS1386_DAY		TOD_BASEOFFSET(0x06*TOD_REGOFF)	/* BCD Day of Week */
#define TOD_DS1386_DAYDATE	TOD_BASEOFFSET(0x08*TOD_REGOFF)	/* BCD Date Day */
#define TOD_DS1386_MODATE	TOD_BASEOFFSET(0x09*TOD_REGOFF)	/* BCD Date Month */
#define TOD_DS1386_YRDATE	TOD_BASEOFFSET(0x0a*TOD_REGOFF)	/* BCD Date Year */

/* DS1386 Command register bits */

#define TOD_DS1386_TE		(unsigned int)0x80	/* TE - Transfer Enable 0 = Stop */
#define TOD_DS1386_IPSW		(unsigned int)0x40	/* Interrupt Switch */
#define TOD_DS1386_IBHLO	(unsigned int)0x20	/* Interrupt B */
#define TOD_DS1386_PULVU	(unsigned int)0x10	/* Interrupt Pulse/Level mode */
#define TOD_DS1386_WAM		(unsigned int)0x08	/* Watchdog Alarm Mask */
#define TOD_DS1386_TDM		(unsigned int)0x04	/* TOD Alarm mask */
#define TOD_DS1386_WAF		(unsigned int)0x02	/* Watchdog Alarm Flag */
#define TOD_DS1386_TDF		(unsigned int)0x01	/* TOD Alarm Flag */

#define TOD_DS1386_EOSC  (unsigned int)0x80	/* 0 = ON  Real Time Clock Oscillator */
#define TOD_DS1386_ESQW  (unsigned int)0x40	/* 1 = OFF 1024Hz Square Wave (pin31) */
#define	TOD_DS1386_12HRS (unsigned int)0x40	/* 1 = 12; 0 = 24 hour select */

/* Safe State Initial Setup */

#define TOD_DS1386_SAFESTATE {		/* FSCC |x|0|x|1|1|1|x|x| */ \
	TOD_WRUNLOCK \
	TOD_DS1386_CNTRL = ~TOD_DS1386_TE & (TOD_DS1386_PULVU | \
			 TOD_DS1386_WAM | TOD_DS1386_TDM ); \
	TOD_WRLOCK \
} /* set CMD = |0|0|0|1|1|1|0|0| */


	/****************************************************************/
	/*								*/
	/* Device ON/OFF MACROs						*/
	/*								*/

#define TOD_DS1386_ON {	/* Is the clock off ? If so, turn it on */ \
	if (TOD_DS1386_MODATE & TOD_DS1386_EOSC) \
	{ \
		TOD_DS1386_SAFESTATE /* Safe State Initial Setup */ \
		TOD_WRUNLOCK \
		TOD_DS1386_HRS &= ~TOD_DS1386_12HRS; \
		TOD_DS1386_MODATE |= TOD_DS1386_ESQW; \
		TOD_DS1386_MODATE &= ~TOD_DS1386_EOSC; \
		TOD_DS1386_CNTRL |= TOD_DS1386_TE; /* start update */ \
		TOD_WRLOCK \
	} \
}

#define TOD_DS1386_OFF {	/* Is the clock on ? If so, turn it off */ \
	if ( !(TOD_DS1386_MODATE & TOD_DS1386_EOSC)) \
	{ \
		TOD_WRUNLOCK \
		TOD_DS1386_MODATE |= (TOD_DS1386_EOSC | TOD_DS1386_ESQW); \
		TOD_WRLOCK \
	} \
}

	/****************************************************************/
	/*								*/
	/* Device Start/Stop External Register Update MACROs		*/
	/*								*/
	/* Stop the update prior to reading/writing the Time Registers	*/
	/* This has no effect on the internal Time registers		*/
	/*								*/

#define TOD_DS1386_STOPUPDATE {	/* Is Transfer Disabled ? If not, Stop */ \
	if ( TOD_DS1386_CNTRL & TOD_DS1386_TE ) \
	{ 			/* Stop updating of External Register(read) */ \
		TOD_WRUNLOCK \
		TOD_DS1386_CNTRL &= (char) ~TOD_DS1386_TE; \
		TOD_WRLOCK \
	} \
}

#define TOD_DS1386_STARTUPDATE { /* Is Transfer Enabled ? If not, Start */ \
	if ( !(TOD_DS1386_CNTRL & TOD_DS1386_TE) ) \
	{ 			/* Resume updating of the External Register */ \
		TOD_WRUNLOCK \
		TOD_DS1386_CNTRL |= TOD_DS1386_TE; \
		TOD_WRLOCK \
	} \
}

	/****************************************************************/
	/*								*/
	/* Device Read/Write MACROs					*/
	/* Assuming the External registers are stopped for Rd/Wr;	*/
	/* otherwise, updated continuously.				*/
	/*								*/
	/* Offsets:				Arrays [0..6], [0..11]	*/
	/*								*/
	/* The Time-Of-Day devices see the DayOfWeek as 1-7 (Sun-Sat)	*/
	/* The program wants DayOfWeek as 0-6 (Sun-Sat)			*/
	/*								*/
	/* The Time-Of-Day devices see the Months as 1-12 (Jan-Dec)	*/
	/* The program wants Months as 0-11 (Jan-Dec)			*/
	/*								*/

#define TOD_DS1386_RDTIME(data) { \
	TOD_DS1386_ON \
	TOD_DS1386_STOPUPDATE \
	(data)->tm_sec = TOD_BCDtoBIN(TOD_DS1386_SEC); \
	(data)->tm_min = TOD_BCDtoBIN(TOD_DS1386_MIN); \
	(data)->tm_hour = TOD_BCDtoBIN((int)TOD_DS1386_HRS & 0x3f); \
	(data)->tm_wday = (TOD_BCDtoBIN((int)TOD_DS1386_DAY) - 1) & 0x07; \
	(data)->tm_mday = TOD_BCDtoBIN(TOD_DS1386_DAYDATE); \
	(data)->tm_mon = (TOD_BCDtoBIN((int)TOD_DS1386_MODATE & 0x1f) - 1) & 0x1f; \
	(data)->tm_year = TOD_BCDtoBIN(TOD_DS1386_YRDATE); \
	TOD_DS1386_STARTUPDATE /* Do an update */ \
}

#define TOD_DS1386_WRTIME(data) { \
	TOD_DS1386_ON \
	TOD_DS1386_STOPUPDATE \
	TOD_WRUNLOCK \
	TOD_DS1386_01SEC = 0; /* Not Used */ \
	TOD_DS1386_SEC = TOD_BintoBCD((data)->tm_sec); \
	TOD_DS1386_MIN = TOD_BintoBCD((data)->tm_min); \
	TOD_DS1386_HRS = TOD_BintoBCD((data)->tm_hour) & ~TOD_DS1386_12HRS; \
	TOD_DS1386_DAY = TOD_BintoBCD((data)->tm_wday + 1) & 0x07; \
	TOD_DS1386_DAYDATE = TOD_BintoBCD((data)->tm_mday); \
	TOD_DS1386_MODATE = ((TOD_BintoBCD((data)->tm_mon + 1) & 0x1f) | \
				TOD_DS1386_ESQW) & ~TOD_DS1386_EOSC; /* RTC Osc Stays ON! */ \
	TOD_DS1386_YRDATE = TOD_BintoBCD((data)->tm_year % 100); \
	TOD_WRLOCK \
	TOD_DS1386_STARTUPDATE /* Do an update */ \
}

/************************************************************************/
/*									*/
/* Generic Macro Names							*/
/*									*/

#define TOD_DEVICE_ON		TOD_DS1386_ON
#define TOD_DEVICE_OFF		TOD_DS1386_OFF
#define TOD_READ_DEVICE		TOD_DS1386_RDTIME
#define TOD_WRITE_DEVICE	TOD_DS1386_WRTIME

#endif /* INCtod_ds1386 */

/* file: tod_ds1386.h */
