#ifndef INC_TRACE
#define INC_TRACE

/* at rockwell, we didn't use printf! */

#include "vxWorks.h"	/* basics i.e. STATUS and other types */
#include "msgQLib.h"	/* for the traceTask's traceMsgQ */
#include "logLib.h"	/* for logMsg(...) - NOTE: The prototype should, but
			   doesn't, specify `const' for arg 1. This may cause
			   compiler warning which the user will have to deal
			   with. */

#define ITEMS_PER_ENTRY         6     /*timestamp,tracetask,lvl,msg_str,p1,p2*/
#define DEFAULT_MAX_TASKS       40
#define DEFAULT_NUM_ENTRIES     1024
#define TRACE_TASK_MSG_Q_SIZE	1024*ITEMS_PER_ENTRY
#define TRACE_TASK_PRIO		151

#ifdef WHAT
# undef WHAT
#else

typedef struct user_p { unsigned int p[ITEMS_PER_ENTRY-4]; } USER_P_T;

typedef struct entry {  unsigned int    time;
                        int             tnum;
                        unsigned int    lvl;
                        char            *msg;
                        USER_P_T        params;
                        } ENTRY_T;

	struct trace_key_variables
		{
                char             traceBlockMarker[25]; /* room for TFS+TSS */
                unsigned int    *traceTid;
                unsigned int    *traceLevel;
		char		**traceName;
                int              traceNumTasks;
                int              traceMaxTasks;
                int              traceNotTasks;		/* trace procedures */
                ENTRY_T         *traceCircQueBegin;
                ENTRY_T         *traceCircQueEnd;
                ENTRY_T         *traceCircQueHead;
                ENTRY_T         *traceCircQueTail;
                int              traceCircQueEmpty;
                int              traceCircQueFull;
		int              traceMode_;
		int		 traceInitialLvl;
		unsigned int	 traceTtyLvl;		/* in TRACE macro */
		unsigned int    *traceClk;	/* ptr to clock to read -
					not con- sidered an integral part of
					trace */
                } ;

#endif

#ifdef TRACE_DEFINE_GLOBALS
# define WHAT
	unsigned int    traceInited = 0;	/* this is how we don't have
						   to rely on zero-ed memory
						   (for non-prom system) */
#else
# define WHAT   extern
extern	unsigned int    traceInited;
#endif


WHAT	unsigned int	traceTask;              /* task var */
WHAT	MSG_Q_ID	traceMsgQ;

#define TFS             "77777777"
#define TSS		"TraceBlockMarker"
                                                /*do 8 byte so no align probs*/

WHAT	struct trace_key_variables trace_;
WHAT	char            *traceLvlStr[32];



/* b/c of static decl, this macro must be first thing in procedure */
#define TRACEPROC(name)	static unsigned int	traceProc;\
			if (traceProc==0) traceProcCreate(name,&traceProc)


/* msg_str MUST BE CONSTANT */

#ifndef TRACE_OLD_TRACE

  /* TRACE_OLD_TRACE not defined, so using the new TRACE where
     TRACE_USR_FUNC(tlvl,msg_str,p1,p2) \
     can be defined, for example, as:
     {char b[200];\
      sprintf(b,"utime:%10u %12s %s"msg_str"\n"\
              ,traceTS,taskName(0),traceLvlStr[tlvl],p1,p2);\
      mur_route_send_text(b);\
     }
     NOTE: traceTS is a automatic stack variable defined in the macro below.
   */

# ifndef TRACE_USR_FUNC
#  define TRACE_USR_FUNC(tlvl,msg_str,p1,p2) \
        logMsg( "utime: %10u %s"msg_str"\n" \
               ,(int)traceTS,(int)traceLvlStr[tlvl],(int)p1,(int)p2,0,0)
# endif

# define TRACE(tlvl,msg_str,p1,p2)  TRACE_TSK_TRC(traceTask,tlvl,msg_str,p1,p2)
# define TRACE0(tlvl,msg_str,p1,p2) TRACE_TSK_TRC(0        ,tlvl,msg_str,p1,p2)
# define TRACEP(tlvl,msg_str,p1,p2) TRACE_TSK_TRC(traceProc,tlvl,msg_str,p1,p2)

# define TRACE_TSK_TRC(tTsk,tlvl,msg_str,p1,p2) \
   {if(trace_.traceLevel[tTsk]&(1<<tlvl)) \
    {unsigned int traceTS=*trace_.traceClk;\
     switch(trace_.traceMode_)\
     {\
     case 0: break;\
     case 1: traceCircQuePut(traceTS,tTsk,tlvl,msg_str,(int)p1,(int)p2);\
             break;\
     case 2: if (trace_.traceTtyLvl&(1<<tlvl))\
              logMsg( "utime: %10u %s"msg_str"\n" \
                     ,(int)traceTS,(int)traceLvlStr[tlvl],(int)p1,(int)p2,0,0);\
             break;\
     case 3: traceCircQuePut(traceTS,tTsk,tlvl,msg_str,(int)p1,(int)p2);\
             if (trace_.traceTtyLvl&(1<<tlvl))\
              logMsg( "utime: %10u %s"msg_str"\n" \
                     ,(int)traceTS,(int)traceLvlStr[tlvl],(int)p1,(int)p2,0,0);\
             break;\
     case 4: if (trace_.traceTtyLvl&(1<<tlvl))\
              TRACE_USR_FUNC(tlvl,msg_str,p1,p2);\
             break;\
     case 5: traceCircQuePut(traceTS,tTsk,tlvl,msg_str,(int)p1,(int)p2);\
             if (trace_.traceTtyLvl&(1<<tlvl))\
              TRACE_USR_FUNC(tlvl,msg_str,p1,p2);\
             break;\
     }\
    }\
   }

#else /* TRACE_OLD_TRACE */
# ifndef TRACE_CODE_LIMIT
#  define TRACE_CODE_LIMIT 0
# endif

# if TRACE_CODE_LIMIT == 0
#  define TRACE(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceTask]&(1<<tlvl))\
     switch(trace_.traceMode_)\
     {\
     case 0: break;\
     case 1: if(getSR()&0x1000){printf("%10u %7x%s"msg_str"\n"\
	,*trace_.traceClk,trace_.traceTid[traceTask],traceLvlStr[tlvl],p1,p2);break;}\
     case 2: logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2);break;\
     case 3: traceCircQuePut(*trace_.traceClk,traceTask,tlvl,msg_str,p1,p2);break;\
     case 4: {ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=(int)trace_.traceTid[traceTask];b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}\
	break;\
     default: break;\
     }\
    }
#  define TRACE0(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[0]&(1<<tlvl)) \
     switch(trace_.traceMode_)\
     {\
     case 0: break;\
     case 1: if(getSR()&0x1000){printf("%10u %7x%s"msg_str"\n"\
	,*trace_.traceClk,0,traceLvlStr[tlvl],p1,p2);break;}\
     case 2: logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2);break;\
     case 3: traceCircQuePut(*trace_.traceClk,0,tlvl,msg_str,p1,p2);break;\
     case 4: {ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=0;b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}\
	break;\
     default: break;\
     }\
    }
#  define TRACEP(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceProc]&(1<<tlvl)) \
     switch(trace_.traceMode_)\
     {\
     case 0: break;\
     case 1: if(getSR()&0x1000){printf("%10u %7x%s"msg_str"\n"\
	,*trace_.traceClk,trace_.traceTid[traceProc],traceLvlStr[tlvl],p1,p2);break;}\
     case 2: logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2);break;\
     case 3: traceCircQuePut(*trace_.traceClk,traceProc,tlvl,msg_str,p1,p2);break;\
     case 4: {ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=(int)trace_.traceTid[traceProc];b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}\
	break;\
     default: break;\
     }\
    }
# elif TRACE_CODE_LIMIT == 1
#  define TRACE(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceTask]&(1<<tlvl))\
    if(trace_.traceMode_)\
	if(getSR()&0x1000)printf("%10u %7x%s"msg_str"\n",*trace_.traceClk,trace_.traceTid[traceTask],traceLvlStr[tlvl],p1,p2);\
	else logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
#  define TRACE0(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[0]&(1<<tlvl))\
    if(trace_.traceMode_)\
	if(getSR()&0x1000)printf("%10u %7x%s"msg_str"\n",*trace_.traceClk,0,traceLvlStr[tlvl],p1,p2);\
	else logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
#  define TRACEP(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceProc]&(1<<tlvl)) \
    if(trace_.traceMode_)\
	if(getSR()&0x1000)printf("%10u %7x%s"msg_str"\n",*trace_.traceClk,trace_.traceTid[traceProc],traceLvlStr[tlvl],p1,p2);\
	else logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
# elif TRACE_CODE_LIMIT == 2
#  define TRACE(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceTask]&(1<<tlvl))\
    if(trace_.traceMode_) logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
#  define TRACE0(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[0]&(1<<tlvl))\
    if(trace_.traceMode_) logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
#  define TRACEP(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceProc]&(1<<tlvl)) \
    if(trace_.traceMode_) logMsg("%10u%s"msg_str"\n",*trace_.traceClk,traceLvlStr[tlvl],p1,p2)}
# elif TRACE_CODE_LIMIT == 3
#  define TRACE(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceTask]&(1<<tlvl))\
    if(trace_.traceMode_) traceCircQuePut(*trace_.traceClk,traceTask,tlvl,msg_str, p1,p2)}
#  define TRACE0(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[0]&(1<<tlvl))\
    if(trace_.traceMode_) traceCircQuePut(*trace_.traceClk,0,tlvl,msg_str, p1,p2)}
#  define TRACEP(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceProc]&(1<<tlvl)) \
    if(trace_.traceMode_) traceCircQuePut(*trace_.traceClk,traceProc,tlvl,msg_str,p1,p2)}
# elif TRACE_CODE_LIMIT == 4
#  define TRACE(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceTask]&(1<<tlvl))\
    if(trace_.traceMode_)\
	{ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=(int)trace_.traceTid[traceTask];b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}}
#  define TRACE0(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[0]&(1<<tlvl))\
    if(trace_.traceMode_)\
	{ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=0;b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}}
#  define TRACEP(tlvl,msg_str,p1,p2) {if(trace_.traceLevel[traceProc]&(1<<tlvl)) \
    if(trace_.traceMode_)\
	{ENTRY_T b;\
	b.msg=msg_str;b.time=(unsigned int)*trace_.traceClk;\
	b.tnum=(int)trace_.traceTid[traceProc];b.lvl=(unsigned int)tlvl;\
	b.params.p[0]=(unsigned int)p1;b.params.p[1]=(unsigned int)p2;\
	msgQSend(traceMsgQ,(char *)&b,ITEMS_PER_ENTRY*4,NO_WAIT,MSG_PRI_NORMAL);}}
# endif /* TRACE_CODE_LIMIT */
#endif /* TRACE_OLD_TRACE */

/* prototypes */
#ifndef	TRACE_DEFINE_GLOBALS
void	traceCircQuePut(  unsigned int	 timestamp
			, unsigned int	 tracetask
			, unsigned int	 lvl
			, const char	*msg_str
# if 0
			, USER_P_T		 params
# endif
			, int		 p1
			, int		 p2
		                                 );
void	traceProcCreate(  const char	*name
			, unsigned int	*procVar );

void	traceMode( int x );
int	traceModeGet( void );
#endif

#endif /* INC_TRACE */
