#ifndef ROOT_daq
#define ROOT_daq

/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/

class TDAQRecvBuffer : public TObject 
{
public:
  TDAQRecvBuffer()  {};
  TDAQRecvBuffer(Int_t size) {fArray.Set(size);}
  ~TDAQRecvBuffer() {};
  Int_t* GetArray() {return fArray.GetArray();}
  Int_t  GetSize()  {return fArray.GetSize();}
  TArray* GetTArray() {return &fArray;}
private:
  TArrayI fArray;
};
// TDAQClient.
class TClientArray;
class TDAQClient : public TObject 
{
public:
   friend class TClientArray;
   TDAQClient(struct sockaddr_in *sockaddr, UChar_t type)
     { 
       ftype  = type;
       memcpy(&fSockaddr,sockaddr,sizeof(struct sockaddr_in));
       cout<<"constructing fqueue\n";
       fqueue         = new TList();
       frecvbuf       = 0;
       //fevstart     = 0;
       fevend         = 0;
       fsuspend       = 0;
       fresponse.bytes = 0;
       fmsg_confirmed = 0;
       fmsg_received =  0;
       fsomething_suspended = 0;
     }
   ~TDAQClient() { delete fqueue;}
   ULong_t Addr()                {return fSockaddr.sin_addr.s_addr;}
   Char_t *TextAddr()            {return inet_ntoa(fSockaddr.sin_addr);}
   struct sockaddr *Sockaddr()   {return  (struct sockaddr *)&fSockaddr;}
   void    Print(Option_t *) 
                                 {cout<<"TDAQClient = "<< 
                                 inet_ntoa(fSockaddr.sin_addr); }
   ULong_t  Hash()               { return fSockaddr.sin_addr.s_addr; }
   Bool_t  IsEqual(TObject *obj) { return fSockaddr.sin_addr.s_addr == 
                                ((TDAQClient*)obj)->fSockaddr.sin_addr.s_addr; }
   Bool_t  IsSortable() const    { return kTRUE; }
   Int_t   Compare(TObject *obj) { if (fSockaddr.sin_addr.s_addr == 
                                 ((TDAQClient*)obj)->fSockaddr.sin_addr.s_addr)
                                      return 0;
                                 else if (fSockaddr.sin_addr.s_addr < 
                                 ((TDAQClient*)obj)->fSockaddr.sin_addr.s_addr)
                                      return -1;
                                 else
                                      return 1; }
   Int_t EventN() {return fevstart->seqn;}
   void  AddBuffer(TObject* ob) {fqueue->Add(ob);}
   TObject* RemoveBuffer() {return fqueue->Remove(fqueue->LastLink());}
   Int_t NBuffers() {return fqueue->GetSize();}
   UChar_t Type() {return ftype;}
   void  MoveAllBuffersTo(TList* queue) 
     {
       TObject* ob;
       while(ob = fqueue->Remove(fqueue->LastLink()))
	 queue->Add(ob);
     }
   Int_t   NextEvent(struct DAQEVENT** ev, Int_t* bytes, Int_t* next);
   void    Suspend() 
     {
       fsuspend = 1; 
       fsomething_suspended = 1;
       cout<<"Suspended "<<(int)ftype<<endl;
     }
   Int_t   Suspended() {return fsuspend;}
   Int_t   Handshake(struct DAQMSG *msg);
   void    Resume()  
     {
       if(fsuspend)
	 {
	   if(fresponse.bytes)
	     {
	       cout<<"Send stored handshake"<<endl;
	       Handshake(&fresponse);
	     }
	   cout<<"Resumed "<<(int)ftype<<endl;
	 }
       fsuspend = 0;
       fresponse.bytes = 0;
     }
   void    SuspendIfNonEmpty()
     {
       if (!((fevend==0) && (fqueue->GetSize()==0))) //non-empty
	 Suspend();
     }
   void    StoreMessage(struct  DAQMSG *response) 
     {
       cout<<"StoreMessage "<<this<<endl;
       memcpy((char*)&fresponse, (char*)response, sizeof(struct sockaddr_in));
     }
 
private:
  static Int_t        fsomething_suspended;
  Int_t               fsuspend;
  struct  sockaddr_in fSockaddr;
  struct  DAQMSG      fresponse;
  UChar_t             ftype;
  TList*              fqueue;
  struct DAQEVENT*    fevstart;
  Long_t*             fevend;
  Long_t*             fbufstart;
  Long_t*             fbufend;
  TDAQRecvBuffer*     frecvbuf;
  ULong_t             fmsg_confirmed;
  ULong_t             fmsg_received;

  //TDAQRecvBuffer*     frecvbuf;
};
class TClientArray : public TObjArray
{
public: 
  TClientArray(int n) : TObjArray(n) {}
  Int_t Send_to_all(struct DAQMSG *msg, int len);
  Int_t Items() 
    {
      int n=0;
      for(int ii=0; ii <= GetLast(); ii++)
	if(At(ii) != 0) n++;
      return n;
    }
  void  MoveAllBuffersTo(TList* queue) 
    {
      cout<<"MoveAllBuffersTo\n";
      for(int ii = 0; ii <= GetLast(); ii++)
	((TDAQClient*)At(ii))->MoveAllBuffersTo(queue);
    }
  void  SuspendAllNonEmpty()
    {
      //cout<<"SuspendAllNonEmpty\n";
      for(int ii = 0; ii <= GetLast(); ii++)
	((TDAQClient*)At(ii))->SuspendIfNonEmpty();
    }
  void  ResumeAll()
    {
      if(!TDAQClient::fsomething_suspended)
	return;
      cout<<"Resume all of "<<GetLast()<<endl;
      for(int ii = 0; ii <= GetLast(); ii++)
	{
	  cout << "Resuming " << ii << endl;
	((TDAQClient*)At(ii))->Resume();
	}
      TDAQClient::fsomething_suspended = 0;
    }
    Int_t SomethingSuspended() {return TDAQClient::fsomething_suspended;}
  void PrintNBuffers()
    {
      for(int ii = 0; ii <= GetLast(); ii++)
	      cout<<"client#"<<ii<<" #buffers "
                  <<((TDAQClient*)At(ii))->NBuffers()
                  <<endl;
    }
};

struct DAQSTATUS
{
  UInt_t runnum;           //0
  UInt_t bytes;            //1
  UInt_t recv_bytes;       //2
  UInt_t clients;          //3
  UInt_t transfers;        //4
  UInt_t messages;         //5
  UInt_t events;           //6
  UInt_t errors;           //7
  UInt_t lost_events;      //8
  UInt_t control;          //9
  UInt_t state;            //10
  UInt_t analized_events;  //11
  UInt_t started;          //12
  UInt_t stopped;          //13
  UInt_t reserved[10];     //14-23
};

// bits in the DAQSTATUS.control
#define CONTROL_EVENT_FORMAT_CHECK  1
#define CONTROL_EVENT_SPECIAL_CHECK 2
#define CONTROL_EVENT_WRITING       4
#define CONTROL_EVENT_DISTRIBUTION  8
#define CONTROL_NOTPHEVENT          16
#define CONTROL_PRINTOUT            32
#define CONTROL_WRITING_TREE        64

#define STATE_STOPPED   0
#define STATE_RUNNING   10
#define STATE_SUSPENDED 11
#define STATE_RESUMED   12

#endif

