/* qaqmessages.h - header file for phatdaq.cxx and ROC programs
 * it should not contain C++ stuff as it will be compiled on VxWorks,
 * possibly, without C++ support.
 *
 * DESCRIPTION
 *
 *     Definition of all messages in the phatdaq-ROC communication.
 */

struct DAQMSG
{
  unsigned long  bytes;       /* size of the message in bytes */
  unsigned long  seqn;        /* sequential number of the message */
  unsigned short command;     /* see DAQMSG_x below */
  unsigned char  source;      /* see SRC_x below */
  unsigned char  source_type; /* see TYPE_x below */
  long data[1];  /* here comes array of events, see DAQEVENT below */
};
struct DAQEVENT
{
  unsigned long  bytes;       /* size of the event in bytes */
  unsigned long  headersize;  /* &data-&bytes */
  unsigned long  seqn;        /* software sequential number of the event,
                               * set to 0 at the beginning of run and 
                               * incremented by ROC at every trigger */
  unsigned long  run;
  unsigned long  testword;    /* checked by EB first of all */
  long data[1];  /* subsystem specific data */
};
#define DAQEVENT_TESTWORD 0x12345678

/* Commands, response command should have the code|RESPONSE */
#define RESPONSE 1
#define DAQMSG_STOP          0
#define DAQMSG_START         2*1
#define DAQMSG_EXIT          2*2
#define DAQMSG_1ST_EV_PACKET 2*3
#define DAQMSG_EV_PACKET     2*4
#define DAQMSG_REPEAT        2*5
#define DAQMSG_IDLE          2*6
#define DAQMSG_WAITING_FOR   2*7
#define DAQMSG_STATUS        2*8
#define DAQMSG_SUSPEND       2*9
#define DAQMSG_RESUME        2*10
/* Sources */

#define SRC_ROC0 0
#define SRC_ROC1 1
#define SRC_ROC2 2
#define SRC_ROCMAX 9
/* SRC_SERVER should be bigger than ROC */
#define SRC_SERVER  9
#define SRC_CMD 10
#define SRC_MAX 10

/* Source types */

#define TYPE_TEST    0
#define TYPE_TRIGGER 1
#define TYPE_SILICON 2
#define TYPE_TOF     3
#define TYPE_MAX     3

