/* uxudpc.c - blasts another system with the given message size */

/*
modification history
--------------------
*/

/*****************************************************************************
 * uxudpc - client program for UNIX host
 *
 * DESCRIPTION
 *
 *     This is a client program which connects to the server via UDP socket.
 *     It allows to configure the maximum  size  of  the  socket-level
 *     send buffer. It repeatedly sends a given size message to the given port
 *     at destination target. 
 *
 * EXAMPLE:
 *
 *     To run this uxudpc program from your UNIX host do the following: 
 *     % uxudpc  <target name>  7000  1000  16000 &
 *
 *
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>

#include "../daqmessages.h"

/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
int    blastNum = 0;
long    *pool;
long    *poolptr;
long    *poolend;
int    sock;

int    runnum;
unsigned long runnbytes = 0;
unsigned char source = 0;
unsigned char source_type = 0;
unsigned long seqn = 0;
unsigned int sleep_usec = 0;
int ds_po = 0;

struct sockaddr_in	sin;
struct sockaddr_in  serverAddr;
int sockaddrsize = sizeof(struct sockaddr_in);
struct DAQMSG server_msg;
struct DAQMSG *pevpacket;

/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
void dumpbuf(int* buf, int nwords)
{
  int i;
  for(i=0; i<nwords; i++)
    {
      if((i % 8) == 0)
	printf("\n%4.4x:",i*4);
      printf(" %8.8x",*buf++);
    }
  printf("\n");
}
/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
int waitformsg(unsigned short command)
{
  int bytes;
  struct fd_set readFds;
  struct fd_set saveFds;
  int width;
  int numFds;
  struct timeval timeout;
  struct DAQMSG  waitingfor;

  timeout.tv_sec = 0;
  timeout.tv_usec = 500000;
  waitingfor.bytes = sizeof(struct DAQMSG);
  waitingfor.seqn = seqn;
  waitingfor.command = DAQMSG_WAITING_FOR;
  waitingfor.source = source;
  waitingfor.data[1] = command;

  width = 0;
  FD_ZERO (&saveFds);    /* initialize the set - all bits off */
  FD_SET (sock, &saveFds); /* Turn on bit for sock */
  width = sock + 1;

  while(1)
    {
      readFds = saveFds;
      if ((numFds = select (width, &readFds, NULL, NULL, &timeout)) == -1)
	{
	  perror (" ERROR in select");   
	  return (-1);
	};
      /*printf ("select: Number of file descriptors ready = %d\n", numFds);*/
      if ((numFds == 0) || !(FD_ISSET (sock, &readFds)))
	{
	  /* Timeout. Send waitingfor message to server */
	  /*printf("timeout\n");*/
          waitingfor.data[0] = command;
          waitingfor.source = source;
          waitingfor.source_type = source_type;
	  sendto(sock, (char*)&waitingfor, waitingfor.bytes, 0,
		 (struct sockaddr *) &sin, sizeof (sin));
	  continue;
	}

      if ((bytes = recvfrom (sock, (char *) &server_msg, sizeof(server_msg), 0, 
		   (struct sockaddr *) &serverAddr, &sockaddrsize)) < 0)
	{
	  perror ("uxudpc handshake receiving error");
	  return -1;
	}
      if(ds_po) printf("received %d bytes",bytes);
      if(ds_po) dumpbuf((int*)&server_msg, (bytes<64) ? bytes/sizeof(int) : 8);
      if(server_msg.command != command)
	{
	  /*printf("unexpected command %8.8x, should be %8.8x\n", 
                  server_msg.command, command);*/
	  return -3;      
	}
      if(serverAddr.sin_addr.s_addr == sin.sin_addr.s_addr)
	break;
      else
        printf("foreign message from %s, skipped\n",inet_ntoa(serverAddr.sin_addr));
    }
  return 0;
}
/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
void poolinit(int nbytes)
{
  if ((pool = (long *) malloc(nbytes + sizeof(struct DAQMSG))) == NULL)
    {
      perror ("cannot allocate buffer of size ");
      exit (1);
    }
  pevpacket = (struct DAQMSG*) pool;
  pevpacket->bytes = nbytes;
  poolptr = pevpacket->data;
  poolend = pool + nbytes/sizeof(int);
  /*printf("pool=%8.8x : %8.8x %8.8x\n",pool, poolend, nbytes);*/
}
/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
int poolget(int size, long **ptr)
{
  int nbytes;
  char string[80];
  int ecode;

  if (size <= (poolend - poolptr))
    {
      *ptr = poolptr;
      return poolend - poolptr;
    }
  else
    {
      if (size > (poolend - pool))
	printf("Event too big %8.8x > %8.8x\n",size,(poolend - pool));

      /*Uncomment for variable length transfer.
        caution should be taken in that case on receiving part 
        to keep up buffer splitting
      *pool = (poolptr - pool)*sizeof(int);
      */
      /* uncomment for fixed length transfer*/
      if (poolend > poolptr)
	*poolptr++ = 0;
      *pool = (poolend - pool)*sizeof(int);
      /*
      printf("handshake %d, press CR>",pevpacket->seqn);
      gets(string);
      */
      if(sleep_usec)
	usleep(sleep_usec);

      /* Handshake. Receive confirmation */
      if(runnbytes)
	if ((ecode =waitformsg(DAQMSG_1ST_EV_PACKET | RESPONSE)))
          return ecode;
      else
	{
	  if(server_msg.seqn != ++seqn)
	    {
	      printf("lost handshake, have: %8.8x, asked: %8.8x, send it\n", 
		     seqn-1, server_msg.seqn);
	      /*return -2;*/
	    }
	  /*pevpacket->seqn = ++seqn;*/
	  seqn = server_msg.seqn;
	}
      pevpacket->seqn = seqn;
      pevpacket->command = DAQMSG_1ST_EV_PACKET;
      pevpacket->source = source;
      pevpacket->source_type = source_type;
      if ((nbytes = sendto(sock, (char*)pool, *pool, 0,
        (struct sockaddr *) &sin, sizeof (sin))) < 0)
	return nbytes;

      runnbytes += nbytes;
      if(ds_po) printf("sent packet %8.8x, size %8.8x, total %8.8x, event %8.8x\n",
        pevpacket->seqn,nbytes,runnbytes,pevpacket->data[2]);
      blastNum += nbytes;
      *pool = 0;
      pevpacket = (struct DAQMSG*) pool;
      poolptr = pevpacket->data;
      size = poolend - poolptr;
      *ptr = poolptr;
      return size;
    }
}
int poolput(int size)
{
  poolptr += size;
  return poolend - poolptr;
}
int poolflush()
{
  int nbytes;
  if ((nbytes = write(sock, (char*)pool, (poolptr - pool)*sizeof(int))) < 0)
    return nbytes;
  blastNum += nbytes;
  return nbytes;
}
void poolfree()
{
  free (pool);
}
/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
main (argc, argv)
    int		argc;
    char	*argv [];
{
  int    ix = 0;
  int	   blen; /* maximum size of socket-level send buffer */
  int    size; /* size of the message to be sent */
  struct hostent  *hp;
  struct hostent  *gethostbyname ();
  int    eventn=0;
  int    uxudpcEventSize;
  int    len;
  int    goon;
  int    ecode;
  int    ii;

  /* Our Socket options */
  int                     opt_linger=0;
  int                     opt_tcp=0;
  int                     opt_reuse=1;
  /*int                     opt_sndbufsiz=1024*128;*/
  int getval;
  int lenval = sizeof(int);
  FILE *filein;
  char simulated_event[10000];
  int  simev_size = 0;
  char *simev_ptr = simulated_event;
  long *simev_iptr;
  long *simev_end;
  int  padding_needed = 0;
  struct DAQEVENT *ev;
  

if (argc < 6)
    {
      printf ("usage: %s targetname port source type print evsize [socksize] [sleep_usec] [filename]\n", 
               argv [0]);
      exit (1);
    }

  bzero (&sin, sizeof (sin));

  sock = socket (AF_INET, SOCK_DGRAM, 0);

  if (sock < 0)
    {
      perror ("cannot open socket");
      exit (1);
    }

  hp = gethostbyname (argv[1]);
  if (hp == 0 && (sin.sin_addr.s_addr = inet_addr (argv [1])) == -1)
    {
      fprintf (stderr, "%s: unknown host\n", argv [1]);
      exit (2);
    }

  if (hp != 0)
    bcopy (hp->h_addr, &sin.sin_addr, hp->h_length);

  sin.sin_family 	= AF_INET;
  sin.sin_port 	= htons (atoi (argv [2]));
  if (argc > 3) source        = atoi (argv [3]);
  if (argc > 4) source_type   = atoi (argv [4]);
  if (argc > 5) ds_po         = atoi (argv [5]);
  if (argc > 6) {size	      = atoi (argv [6]); blen = size;}
  if (argc > 7) blen          = atoi (argv [7]);
  if (argc > 8) sleep_usec    = atoi (argv [8]);

  poolinit(size + sizeof(struct DAQMSG));
  uxudpcEventSize = size/sizeof(long);
  if(argc > 9)
    {
      filein = fopen(argv[9], "r");
      if(filein)
	{
	  printf("File opened %s\n",argv[9]);
	  for( ii = 0; ii < 10000; ii++)
	    {
	      if (fread(simev_ptr++, sizeof(char), 1, filein) == 0)
                break;
	    }
	  simev_size = simev_ptr - simulated_event - 1;
	  padding_needed = simev_size % sizeof(long);
	  if (padding_needed)
	    {
	      padding_needed = sizeof(long) - padding_needed;
	      for(;padding_needed; padding_needed--)
		*simev_ptr++ = 0;
	    }
	  simev_size = simev_ptr - simulated_event - 1;
          simev_end = (long*)(simev_ptr);
	  printf("Event filled for %d bytes from %s\n",simev_size,argv[9]);
          uxudpcEventSize = simev_size / sizeof(long);
	}
      else
	printf("Error: file %s not opened\n",argv[9]);
    }

  if (setsockopt (sock, SOL_SOCKET, SO_SNDBUF, (char*) &blen, sizeof (blen)) < 0)
    {
      perror ("setsockopt SO_SNDBUF failed");
      exit (1);
    }
  setsockopt(sock,SOL_SOCKET,SO_LINGER,(char*)&opt_linger,sizeof(opt_linger));
  /*setsockopt(sock,IPPROTO_TCP,TCP_NODELAY,(char*)&opt_tcp,sizeof(opt_tcp));*/
  setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(char*)&opt_reuse,sizeof(opt_reuse));

  getsockopt (sock, SOL_SOCKET, SO_SNDBUF, (char *)&getval,&lenval);
  printf("socket size %d\n",getval);

/*///////1/////////2/////////3/////////4/////////5/////////6/////////7////////*/
  /* Main loop */

  while((server_msg.command != DAQMSG_EXIT))
    {
      /* Waiting for DAQMSG_START command*/
      for(ii=1;ii;)
	{
	  printf("waiting for start command...\n");
          ecode = waitformsg(DAQMSG_START);
          printf("waitformsg %d\n",ecode);
          switch (ecode)
	    {
	    case 0:
              ii = 0;
              eventn = 0;
              break;
	    case -3:
              if(server_msg.command == DAQMSG_EXIT)
		goto finish;
              break;
	    default:
	      return -1;
	    }
	}

      runnum = server_msg.data[0];
      runnbytes = 0;
      seqn = 0;
      printf("run %d started\n",runnum);
      
      while( (server_msg.command != DAQMSG_STOP) &&
             (server_msg.command != DAQMSG_EXIT) )
	{
	  int nbytes,nwords,i;
	  long *iptr, *iptrend;

	  /* form event*/

	  /*printf("iptr old= %8.8x\n",iptr);*/
	  if ( (nwords = poolget(uxudpcEventSize, &iptr)) < 0)
	    {
              if(server_msg.command == DAQMSG_STOP)
                printf("run %d stopped\n",runnum);
              else
	        printf ("uxudpc write error %d",nwords);
	      continue;
	    }
	  /*printf("iptr new= %8.8x, free=%8.8x\n",iptr,nwords);*/
	  nwords = (nwords < uxudpcEventSize) ? nwords: uxudpcEventSize;
          iptrend = iptr + nwords;
	  ev = (struct DAQEVENT*)iptr;
	  ev->headersize = sizeof(struct DAQEVENT) - sizeof(long);
	  ev->seqn  = eventn;
	  ev->run   = runnum;
	  ev->testword = DAQEVENT_TESTWORD;
          iptr = ev->data;
          if(simev_size)
	    {
	      for(simev_iptr = (long*)simulated_event; simev_iptr < simev_end;)
		{
		  *iptr++ = *simev_iptr++;
		}
	    }
	  else
	    {
	      int tmp = eventn;
	      for(;iptr < iptrend-1;)
		{
		  *iptr++ = tmp;
		  *iptr++ = ~tmp++;
		}
	    }
	  ev->bytes = (iptr-(long*)ev);
	  poolput(ev->bytes);
          ev->bytes *= sizeof(long);
	  eventn++;
	}
    }
finish:
  close (sock);
  poolfree();
  printf ("uxudpc exit.\n");
}

