/* vxudpc.c - blasts another system with the variable message size */

/*
modification history
--------------------
02a,25mar99,as   UDP, 
01f,06nov97,mm   added copyright.
01e,10oct97,mm   deleted line int ix = 0
01d,10oct97,mm   cast arg 4 of setsockopt and cast arg 2 of connect
01c,10oct97,mm   added include <sockLib.h> and "arpa/inet.h"
01b,29sep97,mm   cast arg 1 of 'bzero'
01a,29jan94,ms   cleaned up for VxDemo.
*/

/*****************************************************************************
 * vxudpc - client task for VxWorks target
 *
 * DESCRIPTION
 *
 *     This is a client task which connects to the server via UDP socket.
 *     It allows to configure the maximum  size  of  the  socket-level
 *     send buffer. It repeatedly sends a given size message to the given port
 *     at destination target. It stops sending the message when the global
 *     variable vxudpcStop is set to 1. 
 *
 * EXAMPLE:
 *
 *     To run this vxudpc task from the VxWorks shell do as follows: 
 *     -> sp (vxudpc, "192.2.200.42", 7000, 1000, 16000)
 *
 *     To stop vxudpc task from the VxWorks shell do as follows: 
 *     -> vxudpcStop = 1 
 *
 */

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include "vxWorks.h"
#include "socket.h"
#include <sockLib.h>
#include "arpa/inet.h"
#include "in.h"
#include "taskLib.h"
#include "sysLib.h"
#include "netinet/tcp.h"
#include <logLib.h>
#include "wdLib.h"
#include "selectLib.h"


#include "daqmessages.h"

/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
LOCAL int	blastNum = 0;
LOCAL long       *pool;
LOCAL long       *poolptr;
LOCAL long       *poolend;
LOCAL int       sock;

int vxudpcStop;
int vxudpcDelayMsec;
int vxudpcEventSize;

int    runnum;
unsigned long runnbytes = 0;
short  source;
unsigned long seqn = 0;

LOCAL int 	wdIntvl = 5;
LOCAL WDOG_ID	blastWd = NULL;

struct sockaddr_in	sin;
struct sockaddr_in  serverAddr;
int sockaddrsize = sizeof(struct sockaddr_in);
struct DAQMSG server_msg;
struct DAQMSG *pevpacket;

/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
LOCAL void dumpbuf(char* buf, int bytes)
{
  int i;
  for(i=0; i < bytes; i++)
    {
      if((i % 32) == 0)
	printf("\n%4.4x:",i);
      printf("%2.2x",(unsigned char)(*buf++));
      if((i % 4) == 3)
        printf(" ");
    }
  printf("\n");
}
/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
LOCAL int waitformsg(unsigned short command)
{
  int bytes;
  struct fd_set readFds;
  struct fd_set saveFds;
  int width;
  int numFds;
  struct timeval timeout;
  struct DAQMSG  waitingfor;

  timeout.tv_sec = 0;
  timeout.tv_usec = 500000;
  waitingfor.bytes = sizeof(struct DAQMSG);
  waitingfor.seqn = seqn;
  waitingfor.command = DAQMSG_WAITING_FOR;
  waitingfor.source = source;
  waitingfor.data[1] = command;

  width = 0;
  FD_ZERO (&saveFds);    /* initialize the set - all bits off */
  FD_SET (sock, &saveFds); /* Turn on bit for sock */
  width = sock + 1;

  while(1)
    {
      readFds = saveFds;
      if ((numFds = select (width, &readFds, NULL, NULL, &timeout)) == -1)
	{
	  perror (" ERROR in select");   
	  return (-1);
	};
      /*printf ("selectOnReadFds: Number of file descriptors ready for reading = %d\n", numFds);*/
      if ((numFds == 0) || !(FD_ISSET (sock, &readFds)))
	{
	  /* Timeout. Send waitingfor message to server */
	  /*printf("timeout\n");*/
          waitingfor.data[0] = command;
          waitingfor.source = source;
	  sendto(sock, (char*)&waitingfor, waitingfor.bytes, 0,
		 (struct sockaddr *) &sin, sizeof (sin));
	  continue;
	}

      if ((bytes = recvfrom (sock, (char *) &server_msg, sizeof(server_msg), 0, 
			     (struct sockaddr *) &serverAddr, &sockaddrsize)) < 0)
	{
	  perror ("vxudpc handshake receiving error");
	  return -1;
	}
      /*printf("received %d bytes",bytes);*/
      /*dumpbuf((int*)&server_msg, (bytes < 64) ? bytes/sizeof(int) : 8);*/
      if(server_msg.command != command)
	{
	  /*printf("unexpected command %8.8x, should be %8.8x\n", server_msg.command, command);*/
	  return -3;      
	}
      if(serverAddr.sin_addr.s_addr == sin.sin_addr.s_addr)
	break;
      else
        printf("foreign message from %s, skipped\n",inet_ntoa(serverAddr.sin_addr));
    }
  return 0;
}
/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
LOCAL void poolinit(int nbytes)
{
  if ((pool = (long *) malloc(nbytes + sizeof(struct DAQMSG))) == NULL)
    {
      perror ("cannot allocate buffer of size ");
      exit (1);
    }
  pevpacket = (struct DAQMSG*) pool;
  pevpacket->bytes = nbytes;
  poolptr = pevpacket->data;
  poolend = pool + nbytes/sizeof(int);
  /*printf("pool=%8.8x : %8.8x %8.8x\n",pool, poolend, nbytes);*/
}
/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
LOCAL int poolget(int size, long **ptr)
{
  int nbytes;
  char string[80];
  int ecode;

  if (size <= (poolend - poolptr))
    {
      *ptr = poolptr;
      return poolend - poolptr;
    }
  else
    {
      if (size > (poolend - pool))
	printf("Event too big %8.8x > %8.8x\n",size,(poolend - pool));

      /*Uncomment for variable length transfer.
        caution should be taken in that case on receiving part 
        to keep up buffer splitting
      *pool = (poolptr - pool)*sizeof(int);
      */

      /* uncomment for fixed length transfer*/
	 if (poolend > poolptr)
	 *poolptr++ = 0;
	 *pool = (poolend - pool)*sizeof(int);
	 
      /*printf("Pool[2]=%8.8x,%8.8x,%8.8x,%8.8x,%8.8x,%8.8x\n"
	pool[0],pool[1],pool[2],pool[3],pool[4],pool[5]);
      */
	 /*
      printf("handshake %d, press CR>",pevpacket->seqn);
      gets(string);
      */
	 /*sleep(1);*/

      /* Handshake. Receive confirmation */
      if(runnbytes)
	if ((ecode =waitformsg(DAQMSG_1ST_EV_PACKET | RESPONSE)))
          return ecode;

      if(server_msg.seqn != seqn)
	{
	  printf("lost handshake %8.8x, should be %8.8x\n", server_msg.seqn, seqn);
	  /*return -2;*/
	}
      pevpacket->seqn = ++seqn;
      pevpacket->command = DAQMSG_1ST_EV_PACKET;
      pevpacket->source = source;
      if ((nbytes = sendto(sock, (char*)pool, *pool, 0,
        (struct sockaddr *) &sin, sizeof (sin))) < 0)
	return nbytes;

      runnbytes += nbytes;
      /*printf("sent %8.8x, total %8.8x\n",nbytes,runnbytes);*/
      blastNum += nbytes;
      *pool = 0;
      pevpacket = (struct DAQMSG*) pool;
      poolptr = pevpacket->data;
      size = poolend - poolptr;
      *ptr = poolptr;
      return size;
    }
}
LOCAL int poolput(int size)
{
  poolptr += size;
  return poolend - poolptr;
}
LOCAL int poolflush()
{
    int nbytes;
    if ((nbytes = write(sock, (char*)pool, (poolptr - pool)*sizeof(int))) < 0)
      return nbytes;
    blastNum += nbytes;
    return nbytes;
}
LOCAL void poolfree()
{
  free (pool);
}

/*****************************************************************************
 * blastRate - watchdog routine executed every one minute to report number of
 *             bytes read   
 *
 */

LOCAL void blastRate ()
{
  if (blastNum > 0)
    {
      logMsg ("Sent %d bytes/sec\n", blastNum / wdIntvl,0, 0, 0, 0, 0);
      blastNum = 0;
    }
  else
    {
      logMsg ("No bytes sent in the last %d seconds.\n",wdIntvl, 0, 0, 0, 0, 0);
    }
  if (!vxudpcStop)
    wdStart (blastWd, sysClkRateGet () * wdIntvl, (FUNCPTR) blastRate, 0);
}
/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
void vxudpc 
    (
    char    *targetAddr, /* targetAddress is the IP address of destination 
                            target that needs to be blasted */
    int	     port,       /* port number to send to */
    int      source,     /* source id */
    int      size,       /* size of the message to be sent */
    int      blen        /* maximum size of socket-level send buffer */
    )
{
  int    ix = 0;
  struct hostent  *hp;
  struct hostent  *gethostbyname ();
  int    eventn=0;
  int    uxudpcEventSize = 1000;
  int    len;
  int    goon;
  int    ecode;
  int    ii;

  /* Our Socket options */
  int                     opt_linger=0;
  int                     opt_tcp=1;
  int                     opt_reuse=1;
  /*int                     opt_sndbufsiz=1024*128;*/
  int getval;
  int lenval = sizeof(int);
  FILE *filein;
  char simulated_event[10000];
  int  simev_size = 0;
  char *simev_ptr = simulated_event;
  long *simev_iptr;
  long *simev_end;
  int  padding_needed = 0;

  printf("vxudpc - UDP socket client\n");
  printf("To change cycle delay to 1000 Msec: vxudpcDelayMsec = 1000\n");
  printf("To event size to 1000 words: vxudpcEventSize = 1000\n");

  bzero ((char *) &sin, sizeof (sin));

  sock = socket (AF_INET, SOCK_DGRAM, 0);

  if (sock < 0)
    {
      perror ("cannot open socket");
      exit (1);
    }

  sin.sin_family 	= AF_INET;
  sin.sin_addr.s_addr	= inet_addr (targetAddr);
  sin.sin_port	= htons (port);

  if (setsockopt (sock, SOL_SOCKET, SO_SNDBUF, (char *)&blen, sizeof (blen)) < 0)
    {
      perror ("setsockopt SO_SNDBUF failed");
      exit (1);
    }
  /*
  setsockopt(sock,IPPROTO_TCP,TCP_NODELAY,(char*)&opt_tcp,sizeof(opt_tcp));
  setsockopt(sock,SOL_SOCKET,SO_LINGER,(char*)&opt_linger,sizeof(opt_linger));
  setsockopt(sock,SOL_SOCKET,SO_REUSEADDR,(char*)&opt_reuse,sizeof(opt_reuse));
  */
  getsockopt (sock, SOL_SOCKET, SO_SNDBUF, (char *)&getval,&lenval);
  printf("socket size %d\n",getval);

  vxudpcStop = FALSE;
  vxudpcDelayMsec = 0;
  vxudpcEventSize = 1000;
     
  /* Create watchdog timer */
  if (blastWd == NULL && (blastWd = wdCreate ()) == NULL)
    {
      perror ("cannot create blast watchdog");
      exit (1);
    }

  poolinit(size);

  /* Start watchdog after a minute*/
  /*wdStart (blastWd, sysClkRateGet () * wdIntvl, (FUNCPTR) blastRate, 0);*/

/* //////1/////////2/////////3/////////4/////////5/////////6/////////7/////// */
  /* Main loop */

  while((server_msg.command != DAQMSG_EXIT))
    {
      /* Waiting for DAQMSG_START command*/
      printf("waiting for server command...\n");
      for(ii=1;ii;)
	{
          ecode = waitformsg(DAQMSG_START);
          /*printf("waitformsg %d\n",ecode);*/
          switch (ecode)
	    {
	    case 0:
              ii = 0;
              break;
	    case -3:
              if(server_msg.command == DAQMSG_EXIT)
		goto finish;
              break;
	    default:
	      return;
	    }
	}

      runnum = server_msg.data[0];
      runnbytes = 0;
      printf("run %d started\n",runnum);

      while( (server_msg.command != DAQMSG_STOP) &&
             (server_msg.command != DAQMSG_EXIT) )
	{
	  int nbytes,nwords,i,data=eventn;
	  long *iptr;
	  long *evstart;

	  /* form event*/

	  /*printf("iptr old= %8.8x\n",iptr);*/
	  if ( (nwords = poolget(uxudpcEventSize, &iptr)) < 0)
	    {
              if(server_msg.command == DAQMSG_STOP)
                printf("run %d stopped\n",runnum);
	      perror ("vxudpc write error");
	      continue;
	    }
	  /*printf("iptr new= %8.8x, free=%8.8x\n",iptr,nwords);*/
	  nwords = (nwords < uxudpcEventSize) ? nwords: uxudpcEventSize;
	  evstart = iptr++;
	  *iptr++ = 5*sizeof(int);
	  *iptr++ = eventn;

	  *iptr++ = 1;
	  *iptr++ = 0x12345678;
          if(simev_size)
	    {
	      for(simev_iptr = (long*)simulated_event; simev_iptr < simev_end;)
		{
		  *iptr++ = *simev_iptr++;
		}
	    }
	  else
	    {
	  for(;iptr < (evstart + nwords)-1;)
	    {
	      *iptr++ = data;
	      *iptr++ = ~data++;
	    }
	    }
	  *evstart = (iptr-evstart);
	  poolput(*evstart);
          *evstart *= sizeof(long);
	  eventn++;
	}
    }
finish:
  close (sock);
  poolfree();
  printf ("vxudpc exit.\n");

  wdCancel (blastWd);
}
