/***************************** Copyright Notice ********************************
 *                                                                             *
 * Copyright (c) 1992 Universities Research Association, Inc.                  *
 *               All Rights Reserved.                                          *
 *                                                                             *
 ******************************************************************************/

/*++
 * FACILITY:	Utilities
 *
 * ABSTRACT:	Support for display of longs, words and bytes under VxWorks.
 *
 * ENVIRONMENT:	GNU C.
 *		VxWorks.
 *
 * AUTHOR:	David M. Berg, Creation date: 10-Mar-1992
 *
 * MODIFIED BY:
 *
 *	V01-000	DMB		David M. Berg			 10-Mar-1992
 *		Initial entry.
 *	V01-001 DAS		David A. Slimmer		 20-Dec-1993
 *		Added printf casts, changed to prototype function declaration.
 *	V01-002 DAS		David A. Slimmer		 10-Jan-1994
 *		Conditionally added header fioLib.h
 *--
 ******************************************************************************/

/*++
 * FUNCTIONAL DESCRIPTION:
 *
 *	Display a location by byte, word, long, or string, as specified.
 *
 * FORMAL PARAMETERS:
 *			byteShow(ptr, cnt, inc)
 *			wordShow(ptr, cnt, inc)
 *			longShow(ptr, cnt, inc)
 *			stringShow(ptr)
 *
 *	ptr		Required. Input by value. Unsigned longword.
 *			Pointer to location to display.
 *
 *	cnt		Optional. Input by value. Longword.
 *			Count of elements to display (default 1).
 *
 *	inc		Optional. Input by value. Longword.
 *			Increment for pointer (default 1).
 *
 * IMPLICIT INPUTS:	None
 *
 * IMPLICIT OUTPUTS:	None
 *
 * SIDE EFFECTS:	None
 *
 *--
 ******************************************************************************/

#include <vxWorks.h>
#ifdef MIPS_R3000
#include <fioLib.h>
#else
#include <stdioLib.h>
#endif

void byteShow(unsigned char *ptr, 
	      long cnt,
	      long inc)
   {
   if (cnt == 0)
      cnt = 1;
   if (inc == 0)
      inc = 1;
   printf("\n");
   for (; cnt  > 0; cnt--)
      {
      printf("(0x%x).b = 0x%x\n", (unsigned int)ptr, *ptr);
      ptr += inc;
      }
   }

void wordShow(unsigned short *ptr, 
		long cnt,
		long inc)
   {
   if (cnt == 0)
      cnt = 1;
   if (inc == 0)
      inc = 1;
   printf("\n");
   for (; cnt  > 0; cnt--)
      {
      printf("(0x%x).w = 0x%x\n", (unsigned int)ptr, (unsigned int)*ptr);
      ptr += inc;
      }
   }

void longShow(unsigned long *ptr, 
		long cnt,
		long inc)
   {
   if (cnt == 0)
      cnt = 1;
   if (inc == 0)
      inc = 1;
   printf("\n");
   for (; cnt  > 0; cnt--)
      {
      printf("(0x%x).l = 0x%x\n", (unsigned int)ptr, *ptr);
      ptr += inc;
      }
   }

void stringShow(char *ptr)
   {
   printf("(0x%x).s = %s\n", (unsigned int)ptr, ptr);
   }
