/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1991 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/* ==========================================================================
**
** DESCRIPTION
**      routines specific to MVME167 board
**
** DEVELOPERS
**      Ron Rechenmcher
**
** MODIFICATIONS - modification history (if any) should be kept in source
**		   code management system. (don't clutter this file)
**
**		   Modified function name date to Date    M.Vittone
**							21-Mar-1995
**
** NOTE: This file, because it deals with the hardware, must be compiled
**       without optimization. (Or some work will have to be done to get
**       things to work with optimization.)  This file also uses the gnu
**       "asm" function and therefore can not be compiled with the -ansi
**       option.
*/

/* include from the ./config/mv167 directory under the vxWorks root */

#include "vxWorks.h"

#ifdef mv167
#include "mv167.h"		/* include vmechip2, pccchip2, etc */
#endif

#ifdef mv162
#include "mv162.h"
#endif

/* includes from ./h directory under the vxWorks root */
#include "types.h"			/* for caddr_t */
#include "iv.h"
#include "ioLib.h"
#include "stdioLib.h"

#include "mvsup.h"
#include "mvsup_prototypes.h"

int getKeyPress (void)
{
        int     c, opts;

    opts = ioctl(0,FIOGETOPTIONS,0);
    ioctl(0, FIOSETOPTIONS, (int)(opts & ~OPT_LINE));
    c=getchar();
    ioctl(0, FIOSETOPTIONS, (int)(opts | OPT_LINE));
    if (c != '\n') putchar('\n');
    return(c);
}


#ifdef mv167
/*******************************************************************************
* gpioConnect - connect a routine to the PCCchip2 gpio pin
*
* This routine specifies the interrupt handler to be called at each PCCchip2 
* GPIO interrupt.  It does not enable the interrupts.
* The GPIO pin is pin 19 on the J3 connector (accessible from the front
* of the 167).
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect()
*/

LOCAL	VOIDFUNCPTR	gpioRoutine	= NULL;
LOCAL	int		gpioArg		= NULL;
LOCAL	BOOL		gpioConnected	= FALSE;


STATUS	gpioConnect(  VOIDFUNCPTR	routine	/* routine called at each timer interrupt */
		    , int		arg )	/* argument with which to call routine           */
{
	void	gpioInt();	/* routine follows */

    if (!gpioConnected
	&& intConnect( INUM_TO_IVEC(((*PCC2_VBR)&0xf0)+PCC2_INT_GPIO)
		       ,gpioInt,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    gpioConnected = TRUE;

    gpioRoutine   = routine;
    gpioArg       = arg;

    return (OK);
}


/*
* gpioInt - handle GPIO interrupts
*/

void gpioInt( void )
{

    if (gpioRoutine != NULL) (*gpioRoutine)(gpioArg);

    *PCC2_GPIICR |= GPIICR_ICLR;
}


/*******************************************************************************
* gpioDisable - turn off PCCchip2 gpio interrupts
*
* RETURNS: N/A
*
* SEE ALSO: gpioEnable()
*/

LOCAL BOOL gpioRunning         = FALSE;


void gpioDisable (void)
{

    if (gpioRunning)
    {
	/* disable interrupts */
	*PCC2_GPIICR &= ~GPIICR_IEN;
	gpioRunning = FALSE;
    }
}


/********************************************************************************
* gpioEnable - turn on system clock interrupts
*
* This routine enables PCCChip2 gpio interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: gpioConnect(), gpioDisable(), gpioRateSet()
*/

LOCAL int gpioInterruptLevel = 5;


void gpioEnable (unsigned char mode)	/*0=high lvl; 1=rising edge; 
					  2=low lvl; 3=falling edge */
{

    if (!gpioRunning)
    {
	/* make sure pcc doesn't drive pin */
	*PCC2_GPIOCR &= ~GPIOCR_GPOE;

	*PCC2_GPIICR &= ~7;
	*PCC2_GPIICR |= gpioInterruptLevel;

	*PCC2_GPIICR &= ~(3<<6);	/* clear mode first */
	*PCC2_GPIICR |= (mode<<6) | GPIICR_ICLR;

	*PCC2_GPIICR |= GPIICR_IEN;

	gpioRunning = TRUE;
    }
}


/*******************************************************************************
* gpioLevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next gpioEnable call.
*/

void gpioLevelSet (int level)
{
    switch(level)
    {
    case 0:	gpioInterruptLevel = 0;			break;
    case 1:	gpioInterruptLevel = 1; break;
    case 2:	gpioInterruptLevel = 2; break;
    case 3:	gpioInterruptLevel = 3; break;
    case 4:	gpioInterruptLevel = 4; break;
    case 5:	gpioInterruptLevel = 5; break;
    case 6:	gpioInterruptLevel = 6; break;
    case 7:	gpioInterruptLevel = 7; break;
    default:	gpioInterruptLevel = 5; break;
    }
}



/******************************************************************************
* gpioStatus - returns the current level on the PCCChip2 giop pin
*
*/

int
gpioStatus(void)
{
    return ( (*PCC2_GPIOCR&GPIOCR_GPI) ? 1 : 0  );
}



/******************************************************************************
* gpioMode - sets the mode of the PCCChip2 gpio pins (which is available as
* J3 pin 19 (connector near front panel)
* This routine can conflict with gpioEnable
* gpioEnable overrides the action of this routine setting the mode to output.
* This routine can mess up gpioEnable, if it sets the mode to output.
*/

int
gpioMode(int mode)	/* 0 = input, 1 = ouput */
{
	int	old_mode = (*PCC2_GPIOCR&GPIOCR_GPOE)?1:0;

    if ( mode == 0 )
    {	*PCC2_GPIOCR &= ~GPIOCR_GPOE;
    }
    else
    {	*PCC2_GPIOCR |= GPIOCR_GPOE;
    }
    return ( old_mode );
}


/******************************************************************************
* gpioSet - sets the mode of the PCCChip2 gpio ping, but only if it has been
* setup as output  (returns -1 if not set to output else returns 1 or 0 )
*/

int
gpioSet(int val)	/* 1 = 1 and 0 = 0 */
{

    if ( *PCC2_GPIOCR & GPIOCR_GPOE )
    {
	val &= 1;	/* only 0 or 1 are valid */
	if ( val )
	    *PCC2_GPIOCR |= GPIOCR_GPO_HIGH;
	else
	    *PCC2_GPIOCR &= ~GPIOCR_GPO_HIGH;
	return ( val );
    }
    else
    {
	return ( -1 );
    }
}

#endif


#ifdef mv167
/******************************************************************************
* gpioV{,3,1}Status - returns the current level on the VMEChip2 giop pin2
* In the routines that combine VMEChip2 gpio pins 3 and 1, pin 3 is concidered
* the most significant bit.
*/
int
gpioVStatus(void)
{
	register int	temp = *VMECHIP2_IOCR;


    return ( ((temp&IOCR_GPIOI3)?2:0)|((temp&IOCR_GPIOI1)?1:0)  );
}
#endif
#ifdef mv162
/******************************************************************************
* gpioV{,3,1}Status - returns the current level on the VMEChip2 giop pin2
* for the MVME162LX there are 3 VMEChip2 I/O pins: 
*		pin 16	GPIO1
*		pin 17	GPIO2
*		pin 18	GPIO3
* In the routines that combine VMEChip2 gpio pins 3,2,and 1, pin 3 is 
* considered the most significant bit.
*/
int
gpioVStatus(void)
{
	register int	temp = *VMECHIP2_IOCR;


    return ( 
   ((temp&IOCR_GPIOI3)?4:0)|((temp&IOCR_GPIOI2)?2:0)|((temp&IOCR_GPIOI1)?1:0) 
    );
}

#endif
int
gpioV3Status(void)
{
	register int	temp = *VMECHIP2_IOCR;

    return ( (temp&IOCR_GPIOI3) ? 1 : 0 );
}

int
gpioV1Status(void)
{
	register int	temp = *VMECHIP2_IOCR;

    return ( (temp&IOCR_GPIOI1) ? 1 : 0 );
}
#ifdef mv162
/* for our extra 162 pin... */
int
gpioV2Status(void)
{
        register int    temp = *VMECHIP2_IOCR;

    return ( (temp&IOCR_GPIOI2) ? 1 : 0 );
}
#endif


/******************************************************************************
* gpioV{3,1}Mode - sets the mode of the VMEChip2 gpio pins (which are
* available as J3 pins 18 and 16 respectively (connector near front panel)
*/

int
gpioV3Mode(int mode)	/* 0 = input, 1 = ouput */
{
	int	temp = *VMECHIP2_IOCR;
	int	old_mode;


    old_mode = (temp&IOCR_GPOEN3)?1:0;
    if ( mode == 0 )
    {	temp &= ~IOCR_GPOEN3;
    }
    else
    {	temp |= IOCR_GPOEN3;
    }
    *VMECHIP2_IOCR = temp;
    return ( old_mode );
}

int
gpioV1Mode(int mode)	/* 0 = input, 1 = ouput */
{
	int	temp = *VMECHIP2_IOCR;
	int	old_mode;


    old_mode = (temp&IOCR_GPOEN1)?1:0;
    if ( mode == 0 )
    {	temp &= ~IOCR_GPOEN1;
    }
    else
    {	temp |= IOCR_GPOEN1;
    }
    *VMECHIP2_IOCR = temp;
    return ( old_mode );
}
#ifdef mv162
int
gpioV2Mode(int mode)    /* 0 = input, 1 = ouput */
{
        int     temp = *VMECHIP2_IOCR;
        int     old_mode;


    old_mode = (temp&IOCR_GPOEN2)?1:0;
    if ( mode == 0 )
    {   temp &= ~IOCR_GPOEN2;
    }
    else
    {   temp |= IOCR_GPOEN2;
    }
    *VMECHIP2_IOCR = temp;
    return ( old_mode );
}
#endif

/******************************************************************************
* gpioV{,3,1}Set - sets the mode of the VMEChip2 gpio ping, but only if it has been
* setup as output  (returns -1 if not set to output else returns 1 or 0 )
*/
#ifdef mv167
int
gpioVSet(int val)	/*  values 0 - 3 are applicable */
{
	int	temp = *VMECHIP2_IOCR;


    if ( (temp & (IOCR_GPOEN3 | IOCR_GPOEN1)) == (IOCR_GPOEN3 | IOCR_GPOEN1) )
    {
	val &= 3;	/* values 0 through 3 (inclusive) are valid */

	temp &= ~(IOCR_GPIOO3_HIGH | IOCR_GPIOO1_HIGH);
	if ( val&1 ) temp |= IOCR_GPIOO1_HIGH;
	if ( val&2 ) temp |= IOCR_GPIOO3_HIGH;

	*VMECHIP2_IOCR = temp;
	return ( val );
    }
    else
    {
	return ( -1 );
    }
}
#endif
#ifdef mv162
int
gpioVSet(int val)       /*  values 0 - 7 are applicable */
{
        int     temp = *VMECHIP2_IOCR;


    if ( (temp & (IOCR_GPOEN3 | IOCR_GPOEN2 | IOCR_GPOEN1)) 
			== (IOCR_GPOEN3 | IOCR_GPOEN2 | IOCR_GPOEN1) )
    {
        val &= 7;       /* values 0 through 3 (inclusive) are valid */

        temp &= ~(IOCR_GPIOO3_HIGH | IOCR_GPIOO2_HIGH | IOCR_GPIOO1_HIGH);
        if ( val&1 ) temp |= IOCR_GPIOO1_HIGH;
        if ( val&2 ) temp |= IOCR_GPIOO2_HIGH;
        if ( val&3 ) temp |= IOCR_GPIOO3_HIGH;

        *VMECHIP2_IOCR = temp;
        return ( val );
    }
    else
    {
        return ( -1 );
    }
}
#endif
int
gpioV3Set(int val)	/* 1 = 1 and 0 = 0 */
{
	int	temp = *VMECHIP2_IOCR;


    if ( temp & IOCR_GPOEN1 )
    {
	val &= 1;	/* only 0 or 1 are valid */
	if ( val )
	    temp |= IOCR_GPIOO3_HIGH;
	else
	    temp &= ~IOCR_GPIOO3_HIGH;
	*VMECHIP2_IOCR = temp;
	return ( val );
    }
    else
    {
	return ( -1 );
    }
}

int
gpioV1Set(int val)	/* 1 = 1 and 0 = 0 */
{
	int	temp = *VMECHIP2_IOCR;


    if ( temp & IOCR_GPOEN1 )
    {
	val &= 1;	/* only 0 or 1 are valid */
	if ( val )
	    temp |= IOCR_GPIOO1_HIGH;
	else
	    temp &= ~IOCR_GPIOO1_HIGH;
	*VMECHIP2_IOCR = temp;
	return ( val );
    }
    else
    {
	return ( -1 );
    }
}
#ifdef mv162
int
gpioV2Set(int val)      /* 1 = 1 and 0 = 0 */
{
        int     temp = *VMECHIP2_IOCR;


    if ( temp & IOCR_GPOEN2 )
    {
        val &= 1;       /* only 0 or 1 are valid */
        if ( val )
            temp |= IOCR_GPIOO2_HIGH;
        else
            temp &= ~IOCR_GPIOO2_HIGH;
        *VMECHIP2_IOCR = temp;
        return ( val );
    }
    else
    {
        return ( -1 );
    }
}
#endif


/*******************************************************************************
* clk1Connect - connect a routine to the vmechip2 timer 1 interrupt
*
* This routine specifies the interrupt handler to be called at each vmechip2 
* timer 1 clock interrupt.  It does not enable the interrupts.
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect(), usrClock(), sysClkEnable()
*/

LOCAL	VOIDFUNCPTR	clk1Routine	= NULL;
LOCAL	int		clk1Arg		= NULL;
LOCAL	BOOL		clk1Connected	= FALSE;


STATUS	clk1Connect(  VOIDFUNCPTR	routine	/* routine called at each timer interrupt */
		    , int		arg )	/* argument with which to call routine           */
{
	void	clk1Int();	/* routine follows */

    if (!clk1Connected
	&& intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_TT1))
		       ,clk1Int,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    clk1Connected = TRUE;

    clk1Routine   = routine;
    clk1Arg       = arg;

    return (OK);
}


/*
* clk1Int - handle auxiliary clock interrupts
*/

void clk1Int (void)
{
	int	temp;	/* needed to force lword access */

    if (clk1Routine != NULL) (*clk1Routine)(clk1Arg);

    /* reset clock interrupt */
    temp=~ICLR_CTIC1;
    *VMECHIP2_ICLR |= ~temp;
#ifdef NOTDEF
produces a bset #0 (byte operation)??????????????
    *VMECHIP2_ICLR |=  (int)ICLR_CTIC1;
#endif
}


/*******************************************************************************
* clk1Disable - turn off timer interrupts
*
* This routine disables the vmechip2 timer1 interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: clk1Enable()
*/

LOCAL BOOL clk1Running         = FALSE;


void clk1Disable (void)
{

    if (clk1Running)
    {
	/* disable interrupts */
	*VMECHIP2_LBIER &= ~LBIER_ETIC1;	/* also clears status (p.2-69)*/
	*VMECHIP2_TIMERCR &= ~TIMERCR_TT1_EN;	/* and disable counter */

	clk1Running = FALSE;
    }
}


/********************************************************************************
* clk1Enable - turn on system clock interrupts
*
* This routine enables system clock interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: clk1Connect(), clk1Disable(), clk1RateSet()
*/

LOCAL int clk1Period         = 1000000/100;		/* default val */
LOCAL int clk1InterruptLevel = ILR1_TICK1_LEVEL5;


void clk1Enable (void)
{
    int	temp;	/* needed to force lword access */

    if (!clk1Running)
    {
	/* load compare register */
	/* this is done by entering the number of micro seconds */

	*VMECHIP2_TTCOMP1 = clk1Period;

	*VMECHIP2_TTCOUNT1 = 0;		/* clear counter */

	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR1 &= ~ILR1_TICK1_LEVEL7;	/* clear bits before ORing */
	*VMECHIP2_ILR1 |= clk1InterruptLevel;

	temp = ~LBIER_ETIC1;			/* enable the interrupt */
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset #0,@#fff4006c?????????
	*VMECHIP2_LBIER |= LBIER_ETIC1;
#endif

	/* now enable timer with clear-on-compare and clear the overflow count*/

	*VMECHIP2_TIMERCR |= (  TIMERCR_TT1_EN 
			      | TIMERCR_TT1_COC 
			      | TIMERCR_TT1_COVF);

	clk1Running = TRUE;
    }
}


/*******************************************************************************
* clk1LevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next clkxEnable call.
*/

void clk1LevelSet(int level)
{
    switch(level)
    {
    case 0:	clk1InterruptLevel = 0;			break;
    case 1:	clk1InterruptLevel = ILR1_TICK1_LEVEL1; break;
    case 2:	clk1InterruptLevel = ILR1_TICK1_LEVEL2; break;
    case 3:	clk1InterruptLevel = ILR1_TICK1_LEVEL3; break;
    case 4:	clk1InterruptLevel = ILR1_TICK1_LEVEL4; break;
    case 5:	clk1InterruptLevel = ILR1_TICK1_LEVEL5; break;
    case 6:	clk1InterruptLevel = ILR1_TICK1_LEVEL6; break;
    case 7:	clk1InterruptLevel = ILR1_TICK1_LEVEL7; break;
    default:	clk1InterruptLevel = ILR1_TICK1_LEVEL5; break;
    }
}


/*******************************************************************************
* clk1RateGet - get the system clock rate
*
* This routine returns the interrupt rate of the system clock.
*
* RETURNS: The number of ticks per second of the system clock.
*
* SEE ALSO: clk1Enable(), clk1RateSet()
*/

int clk1RateGet (void)
{

    return (1000000/clk1Period);
}


/*******************************************************************************
* clk1uPeriodGet - get the clock period
*
* This routine returns the period of the clock, in microseconds.
*
* RETURNS: The number of microsecond in the period of the clock cycle.
*
* SEE ALSO: clk1Enable(), clk1RateSet()
*/

int clk1uPeriodGet (void)
{

    return (clk1Period);
}


/*******************************************************************************
* clk1RateSet - set the system clock rate
*
* This routine sets the interrupt rate of the clock.  It does not
* enable clock interrupts.
*
* RETURNS: OK, or ERROR if the tick rate is invalid or the timer cannot be
* set.
*
* SEE ALSO: clk1Enable(), clk1GetRate()
*/

STATUS clk1RateSet (int ticksPerSecond) /* number of clock interrupts per second */
{

    clk1Period = 1000000/ticksPerSecond;
	     
    if (clk1Running)
    {
	clk1Disable ();
	clk1Enable ();
    }

    return (OK);
}


/*******************************************************************************
* clk1uPeriodSet - set the clock rate
*
* This routine sets the interrupt period of the appropriate clock.  It does
* not enable clock interrupts.  Normally, it is called by usrRoot() in
* usrConfig.c. The parameter value is store in a global location that will be
* used to set the micro second counter compare register; effectively setting
* the period of the counter restart to the exact number of microseconds
* entered.
*
* RETURNS: OK.
*
* SEE ALSO: clk1Enable(), clk1GetRate() */

STATUS clk1uPeriodSet (int uSeconds)
{

    clk1Period = uSeconds;
	     
    if (clk1Running)
    {
	clk1Disable ();
	clk1Enable ();
    }

    return (OK);
}


/*******************************************************************************
* clk1Start
*
* This routine is used to time things to micro second accuracy.
*
* SEE ALSO: clk1Stop()
*/

void clk1Start (void)
{
    int	temp;	/* needed to force lword access */

    if (!clk1Running)
    {
	/* load compare register to max value (71.6 minutes w/usec resolution)*/

	*VMECHIP2_TTCOMP1 = 0xffffffff;

	*VMECHIP2_TTCOUNT1 = 0;		/* clear counter */

	/* enable polling interrupts by setting IRQ level = 0 */

	temp = *VMECHIP2_ILR1;
	temp  &= ~ILR1_TICK1_LEVEL7;/* THIS ENABLES OVERFLOW COUNTER */
	*VMECHIP2_ILR1 = temp;

	temp = ~LBIER_ETIC1;	/* enable interrupt (and overflow counter) */
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset #0,@#fff4006c?????????
	*VMECHIP2_LBIER |= LBIER_ETIC1;
#endif

	/* now enable timer with clear-on-compare and clear the overflow count*/

	*VMECHIP2_TIMERCR |= (  TIMERCR_TT1_EN 
			      | TIMERCR_TT1_COC 
			      | TIMERCR_TT1_COVF);

	clk1Running = TRUE;
    }
}


/*******************************************************************************
* clk1Stop
*/

unsigned int clk1Stop (void)
{
    unsigned long int temp;

    if (clk1Running)
    {
	temp = *VMECHIP2_TIMERCR;
	temp  &= ~TIMERCR_TT1_EN;
	*VMECHIP2_TIMERCR = temp;

#ifdef NOTDEF
	int	overflow_flag;

	/* see if we overflowed */
	if ( *VMECHIP2_LBISR & LBISR_TIC1 )
	    overflow_flag = 1;
	else
	    overflow_flag = 0;
#endif

	/* disable interrupts */
	temp = *VMECHIP2_LBIER; 
	temp &= ~LBIER_ETIC1;	/* also clears status (p.2-69)*/
	*VMECHIP2_LBIER = temp;

	clk1Running = FALSE;
    }
    return (*VMECHIP2_TTCOUNT1);
}

/*---------------------------------------------------------------------------*/




/*******************************************************************************
* clk2Connect - connect a routine to the vmechip2 timer 2 interrupt
*
* This routine specifies the interrupt handler to be called at each vmechip2 
* timer 2 clock interrupt.  It does not enable the interrupts.
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect(), usrClock(), sysClkEnable()
*/

LOCAL	VOIDFUNCPTR	clk2Routine	= NULL;
LOCAL	int		clk2Arg		= NULL;
LOCAL	BOOL		clk2Connected	= FALSE;


STATUS	clk2Connect(  VOIDFUNCPTR	routine	/* routine called at each timer interrupt */
		    , int		arg )	/* argument with which to call routine           */
{
	void	clk2Int();	/* routine follows */

    if (!clk2Connected
	&& intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_TT2))
		       ,clk2Int,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    clk2Connected = TRUE;

    clk2Routine   = routine;
    clk2Arg       = arg;

    return (OK);
}


/*
* clk2Int - handle auxiliary clock interrupts
*/

void clk2Int (void)
{
	int	temp;	/* needed to force lword access */

    if (clk2Routine != NULL) (*clk2Routine)(clk2Arg);

    /* reset clock interrupt */
    temp=~ICLR_CTIC2;
    *VMECHIP2_ICLR |= ~temp;
#ifdef NOTDEF
produces a bset #0 (byte operation)??????????????
    *VMECHIP2_ICLR |=  (int)ICLR_CTIC2;
#endif
}


/*******************************************************************************
* clk2Disable - turn off timer interrupts
*
* This routine disables the vmechip2 timer2 interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: clk2Enable()
*/

LOCAL BOOL clk2Running         = FALSE;


void clk2Disable (void)
{
    unsigned long int temp;

    if (clk2Running)
    {
	/* disable interrupts */

	temp = *VMECHIP2_LBIER;
	temp &= ~LBIER_ETIC2;	/* also clears status (p.2-69)*/
	*VMECHIP2_LBIER = temp;

	temp = *VMECHIP2_TIMERCR;
	temp  &= ~TIMERCR_TT2_EN;	/* and disable counter */
	*VMECHIP2_TIMERCR = temp;

	clk2Running = FALSE;
    }
}


/********************************************************************************
* clk2Enable - turn on system clock interrupts
*
* This routine enables system clock interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: clk2Connect(), clk2Disable(), clk2RateSet()
*/

LOCAL int clk2Period   = 1000000/100;		/* default val */
LOCAL int clk2InterruptLevel = ILR1_TICK2_LEVEL5;


void clk2Enable (void)
{
    int	temp;	/* needed to force lword access */

    if (!clk2Running)
    {
	/* load compare register */
	/* this is done by entering the number of micro seconds */

	*VMECHIP2_TTCOMP2 = clk2Period;

	*VMECHIP2_TTCOUNT2 = 0;		/* clear counter */

	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR1 &= ~ILR1_TICK2_LEVEL7;	/* clear bits before ORing */
	*VMECHIP2_ILR1 |= clk2InterruptLevel;

	temp = ~LBIER_ETIC2;
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset #0,@#fff4006c?????????
	*VMECHIP2_LBIER |= LBIER_ETIC2;
#endif

	/* now enable timer with clear-on-compare and clear the overflow count*/

	temp = *VMECHIP2_TIMERCR;
	temp |= (int)(  (int)TIMERCR_TT2_EN | (int)TIMERCR_TT2_COC | (int)TIMERCR_TT2_COVF);
	(int)*VMECHIP2_TIMERCR = temp;
#ifdef NOTDEF
for some reason this assembles to a oriw instead of a oril
	*VMECHIP2_TIMERCR |= (  TIMERCR_TT2_EN 
			      | TIMERCR_TT2_COC 
			      | TIMERCR_TT2_COVF);
#endif

	clk2Running = TRUE;
    }
}


/*******************************************************************************
* clk2LevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next clkxEnable call.
*/

void clk2LevelSet(int level)
{
    switch(level)
    {
    case 0:	clk2InterruptLevel = 0;			break;
    case 1:	clk2InterruptLevel = ILR1_TICK2_LEVEL1; break;
    case 2:	clk2InterruptLevel = ILR1_TICK2_LEVEL2; break;
    case 3:	clk2InterruptLevel = ILR1_TICK2_LEVEL3; break;
    case 4:	clk2InterruptLevel = ILR1_TICK2_LEVEL4; break;
    case 5:	clk2InterruptLevel = ILR1_TICK2_LEVEL5; break;
    case 6:	clk2InterruptLevel = ILR1_TICK2_LEVEL6; break;
    case 7:	clk2InterruptLevel = ILR1_TICK2_LEVEL7; break;
    default:	clk2InterruptLevel = ILR1_TICK2_LEVEL5; break;
    }
}


/*******************************************************************************
* clk2RateGet - get the system clock rate
*
* This routine returns the interrupt rate of the system clock.
*
* RETURNS: The number of ticks per second of the system clock.
*
* SEE ALSO: clk2Enable(), clk2RateSet()
*/

int clk2RateGet (void)
{

    return (1000000/clk2Period);
}


/*******************************************************************************
* clk2uPeriodGet - get the clock period
*
* This routine returns the period of the clock, in microseconds.
*
* RETURNS: The number of microsecond in the period of the clock cycle.
*
* SEE ALSO: clk2Enable(), clk2RateSet()
*/

int clk2uPeriodGet (void)
{

    return (clk2Period);
}


/*******************************************************************************
* clk2RateSet - set the clock rate
*
* This routine sets the interrupt rate of the system clock.  It does not
* enable system clock interrupts.  Normally, it is called by usrRoot() in
* usrConfig.c.
*
* RETURNS: OK.
*
* SEE ALSO: clk2Enable(), clk2GetRate()
*/

STATUS clk2RateSet (int ticksPerSecond) /* number of clock interrupts per second */
{

    clk2Period = 1000000/ticksPerSecond;
	     
    if (clk2Running)
    {
	clk2Disable ();
	clk2Enable ();
    }

    return (OK);
}


/*******************************************************************************
* clk2uPeriodSet - set the clock rate
*
* This routine sets the interrupt period of the appropriate clock.  It does
* not enable clock interrupts.  Normally, it is called by usrRoot() in
* usrConfig.c. The parameter value is store in a global location that will be
* used to set the micro second counter compare register; effectively setting
* the period of the counter restart to the exact number of microseconds
* entered.
*
* RETURNS: OK.
*
* SEE ALSO: clk2Enable(), clk2GetRate()
*/

STATUS clk2uPeriodSet (int uSeconds)
{

    clk2Period = uSeconds;
	     
    if (clk2Running)
    {
	clk2Disable ();
	clk2Enable ();
    }

    return (OK);
}


/*******************************************************************************
* clk2Start
*
* This routine is used to time things to micro second accuracy.
*
* SEE ALSO: clk2Stop()
*/

void clk2Start (void)
{
    int	temp;	/* needed to force lword access */

    if (!clk2Running)
    {
	/* load compare register */

	*VMECHIP2_TTCOMP2 = 0xffffffff;

	*VMECHIP2_TTCOUNT2 = 0;		/* clear counter */

	/* enable interrupt, and set IRQ level */

	temp = *VMECHIP2_ILR1;
	temp  &= ~ILR1_TICK2_LEVEL7; /* THIS ENABLE OVERFLOW COUNTER */
	*VMECHIP2_ILR1 = temp;

	temp = ~LBIER_ETIC2;
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset #0,@#fff4006c?????????
	*VMECHIP2_LBIER |= LBIER_ETIC2;
#endif

	/* now enable timer with clear-on-compare and clear the overflow count*/

	temp = *VMECHIP2_TIMERCR;
	temp |= (int)(  (int)TIMERCR_TT2_EN | (int)TIMERCR_TT2_COC | (int)TIMERCR_TT2_COVF);
	(int)*VMECHIP2_TIMERCR = temp;
#ifdef NOTDEF
for some reason this assembles to a oriw instead of a oril
	*VMECHIP2_TIMERCR |= (  TIMERCR_TT2_EN 
			      | TIMERCR_TT2_COC 
			      | TIMERCR_TT2_COVF);
#endif

	clk2Running = TRUE;
    }
}


/*******************************************************************************
* clk2Stop
*/

unsigned int clk2Stop (void)
{
	int	temp;	/* needed to force lword access */


    if (clk2Running)
    {
	temp = *VMECHIP2_TIMERCR;
	temp &= (int)~TIMERCR_TT2_EN;
	*VMECHIP2_TIMERCR = temp;		/* and disable counter */
#ifdef NOTDEF
this does word access
	*VMECHIP2_TIMERCR &= ~TIMERCR_TT2_EN;		/* and disable counter */
#endif

#ifdef NOTDEF
	int	overflow_flag;

	/* see if we overflowed */
	if ( *VMECHIP2_LBISR & LBISR_TIC2 )
	    overflow_flag = 1;
	else
	    overflow_flag = 0;
#endif

	/* disable interrupts */

	temp = *VMECHIP2_LBIER;
	temp &= ~LBIER_ETIC2;	/* also clears status (p.2-69)*/
	*VMECHIP2_LBIER = temp;

	clk2Running = FALSE;
    }
    return (*VMECHIP2_TTCOUNT2);
}


/*******************************************************************************
* spinDelay
* although this is a fairly simple routine, it allows for flexible command
line (vxworks prompt) testing
*/


	unsigned	*spinDelayClock = VMECHIP2_TTCOUNT1;


int	spinDelay(  unsigned	usec
		  , int		intLock_flag )
{
	unsigned	t1;
	int		key;

    if (intLock_flag) key = intLock();

    t1 = *spinDelayClock;

    while ((*spinDelayClock-t1) < usec) ;

    if (intLock_flag) intUnlock( key );

    return (OK);
}


/*******************************************************************************
* LM0Connect - connect a routine to the vmechip2 Location Monitor x interrupt
*
* This routine specifies the interrupt handler to be called at each vmechip2 
* LM0 interrupt.  It does not enable the interrupts.
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect(), usrClock(), sysClkEnable()
*/

LOCAL	VOIDFUNCPTR	LM0Routine	= NULL;
LOCAL	int		LM0Arg		= NULL;
LOCAL	BOOL		LM0Connected	= FALSE;


STATUS	LM0Connect(  VOIDFUNCPTR	routine	/* routine called at each LM interrupt */
		   , int		arg )	/* argument with which to call routine           */
{
	void	LM0Int();	/* routine follows */

    if (!LM0Connected
	&& intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_GCSR_LM0))
		       ,LM0Int,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    LM0Connected = TRUE;

    LM0Routine   = routine;
    LM0Arg       = arg;

    return (OK);
}


/*
* LM0Int - handle location monitor x interrupts
*/

void LM0Int (void)
{
	int	temp;	/* needed to force lword access */

    if (LM0Routine != NULL) (*LM0Routine)(LM0Arg);

    /* clear interrupt */
    temp=~ICLR_CLM0;
    *VMECHIP2_ICLR |= ~temp;
#ifdef NOTDEF
produces a bset #0 (byte operation)??????????????
    *VMECHIP2_ICLR |=  (int)ICLR_CLM0;
#endif
}


/*******************************************************************************
* LM0Disable - turn off location monitor x interrupts
*
* This routine disables the vmechip2 location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: LM0Enable()
*/

LOCAL BOOL LM0Running         = FALSE;


void LM0Disable (void)
{

    if (LM0Running)
    {
	/* disable interrupts */
	*VMECHIP2_LBIER &= ~LBIER_ELM0;	/* also clears status (p.2-69)*/

	LM0Running = FALSE;
    }
}


/********************************************************************************
* LM0Enable - turn on location monitor x interrupts
*
* This routine enables location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: LM0Connect(), LM0Disable(), LM0RateSet()
*/

LOCAL int LM0InterruptLevel = ILR2_LM0_LEVEL5;


void LM0Enable (void)
{
	int	temp;	/* needed to force lword access */


    if (!LM0Running)
    {
	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR2 &= ~ILR2_LM0_LEVEL7;	/* clear bits before ORing */
	*VMECHIP2_ILR2 |= LM0InterruptLevel;

	temp = ~LBIER_ELM0;			/* enable the interrupt */
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset ?????????
	*VMECHIP2_LBIER |= LBIER_ELM0;
#endif

	LM0Running = TRUE;
    }
}


/*******************************************************************************
* LM0LevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next LM0Enable call.
*/

void LM0LevelSet(int level)
{
    switch(level)
    {
    case 0:	LM0InterruptLevel = 0;			break;
    case 1:	LM0InterruptLevel = ILR2_LM0_LEVEL1; break;
    case 2:	LM0InterruptLevel = ILR2_LM0_LEVEL2; break;
    case 3:	LM0InterruptLevel = ILR2_LM0_LEVEL3; break;
    case 4:	LM0InterruptLevel = ILR2_LM0_LEVEL4; break;
    case 5:	LM0InterruptLevel = ILR2_LM0_LEVEL5; break;
    case 6:	LM0InterruptLevel = ILR2_LM0_LEVEL6; break;
    case 7:	LM0InterruptLevel = ILR2_LM0_LEVEL7; break;
    default:	LM0InterruptLevel = ILR2_LM0_LEVEL5; break;
    }
}


/*******************************************************************************
* LM1Connect - connect a routine to the vmechip2 Location Monitor x interrupt
*
* This routine specifies the interrupt handler to be called at each vmechip2 
* LM1 interrupt.  It does not enable the interrupts.
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect(), usrClock(), sysClkEnable()
*/

LOCAL	VOIDFUNCPTR	LM1Routine	= NULL;
LOCAL	int		LM1Arg		= NULL;
LOCAL	BOOL		LM1Connected	= FALSE;


STATUS	LM1Connect(  VOIDFUNCPTR	routine	/* routine called at each LM interrupt */
		   , int		arg )	/* argument with which to call routine           */
{
	void	LM1Int();	/* routine follows */

    if (!LM1Connected
	&& intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_GCSR_LM1))
		       ,LM1Int,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    LM1Connected = TRUE;

    LM1Routine   = routine;
    LM1Arg       = arg;

    return (OK);
}


/*
* LM1Int - handle location monitor x interrupts
*/

void LM1Int (void)
{
	int	temp;	/* needed to force lword access */

    if (LM1Routine != NULL) (*LM1Routine)(LM1Arg);

    /* clear interrupt */
    temp=~ICLR_CLM1;
    *VMECHIP2_ICLR |= ~temp;
#ifdef NOTDEF
produces a bset #0 (byte operation)??????????????
    *VMECHIP2_ICLR |=  (int)ICLR_CLM1;
#endif
}


/*******************************************************************************
* LM1Disable - turn off location monitor x interrupts
*
* This routine disables the vmechip2 location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: LM1Enable()
*/

LOCAL BOOL LM1Running         = FALSE;


void LM1Disable (void)
{

    if (LM1Running)
    {
	/* disable interrupts */
	*VMECHIP2_LBIER &= ~LBIER_ELM1;	/* also clears status (p.2-69)*/

	LM1Running = FALSE;
    }
}


/********************************************************************************
* LM1Enable - turn on location monitor x interrupts
*
* This routine enables location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: LM1Connect(), LM1Disable(), LM1RateSet()
*/

LOCAL int LM1InterruptLevel = ILR2_LM1_LEVEL5;


void LM1Enable (void)
{
	int	temp;	/* needed to force lword access */


    if (!LM1Running)
    {
	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR2 &= ~ILR2_LM1_LEVEL7;	/* clear bits before ORing */
	*VMECHIP2_ILR2 |= LM1InterruptLevel;

	temp = ~LBIER_ELM1;			/* enable the interrupt */
	*VMECHIP2_LBIER |= ~temp;

#ifdef NOTDEF
for some reason, the assemble is a bset ?????????
	*VMECHIP2_LBIER |= LBIER_ELM1;
#endif

	LM1Running = TRUE;
    }
}


/*******************************************************************************
* LM1LevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next LM1Enable call.
*/

void LM1LevelSet(int level)
{
    switch(level)
    {
    case 0:	LM1InterruptLevel = 0;			break;
    case 1:	LM1InterruptLevel = ILR2_LM1_LEVEL1; break;
    case 2:	LM1InterruptLevel = ILR2_LM1_LEVEL2; break;
    case 3:	LM1InterruptLevel = ILR2_LM1_LEVEL3; break;
    case 4:	LM1InterruptLevel = ILR2_LM1_LEVEL4; break;
    case 5:	LM1InterruptLevel = ILR2_LM1_LEVEL5; break;
    case 6:	LM1InterruptLevel = ILR2_LM1_LEVEL6; break;
    case 7:	LM1InterruptLevel = ILR2_LM1_LEVEL7; break;
    default:	LM1InterruptLevel = ILR2_LM1_LEVEL5; break;
    }
}


/*******************************************************************************
* LMCheck
*
*/

STATUS LMCheck (int lm)
{

    if (lm==0) return (*VC2GCSR_LM_SIG & LM_SIG_LM0 ? 1 :0);
    else if (lm==1) return (*VC2GCSR_LM_SIG & LM_SIG_LM1 ? 1 :0);
    else if (lm==2) return (*VC2GCSR_LM_SIG & LM_SIG_LM2 ? 1 :0);
    else if (lm==3) return (*VC2GCSR_LM_SIG & LM_SIG_LM3 ? 1 :0);
    else return (1);

}


/*******************************************************************************
* LMAddressSet
*
*/

void LMAddressSet (unsigned short addr)
{
	int	temp;	/* needed to force lword access */
	int	temp2;


    addr &= 0x0000ff00;		/* only group part is applicable */
    temp2 = addr;
    temp2 <<= 16;		/* move to upper word (board is 00 from above)*/
				/*  (ref p.2-41) */
    temp = *VMECHIP2_LBTVCR;
    temp &= 0x00ffffff;		/* clear bits */
    temp |= temp2;
    *VMECHIP2_LBTVCR = temp;
}


/*******************************************************************************
* LMSIGAddressShow
*
*/

STATUS LMSIGAddressShow (void)
{
	int	temp;	/* needed to force lword access */
	int	group, board;

    temp = *VMECHIP2_LBTVCR;

    group = (temp>>24) & 0xff;
    board = (temp>>20) & 0xf;

    printf("Location monitor and signal interrupt address locations\n");
    printf("in SHORT VME address space - \n");
    printf("  LM0 address: 0x%xf1     LM1 address: 0x%xf3\n",group,group);
    printf("  LM2 address: 0x%xf5     LM3 address: 0x%xf7\n",group,group);
    printf("  SIGnal register address: 0x%x%x2 (bits 3-0 for SIG3-0).\n",group
								   ,board);
    printf("The GROUP address is 0x%x and the BOARD address is 0x%x.\n",group
								   ,board);
    if (board == 0xf) printf("The map decoder is disabled.\n");
}


/*******************************************************************************
* SIGConnect - connect a routine to the vmechip2 Location Monitor x interrupt
*
* This routine specifies the interrupt handler to be called at each vmechip2 
* SIG interrupt.  It does not enable the interrupts.
*
* RETURN: OK, or ERROR if the routine cannot be connected to the interrupt.
*
* SEE ALSO: intConnect(), usrClock(), sysClkEnable()
*/

LOCAL	VOIDFUNCPTR	SIGRoutine[4]	= {NULL,NULL,NULL,NULL};
LOCAL	int		SIGArg[4]	= {NULL,NULL,NULL,NULL};
LOCAL	BOOL		SIGConnected[4]	= {FALSE,FALSE,FALSE,FALSE};
LOCAL	int		LBIV_GCSR_SIG[4]= {LBIV_GCSR_SIG0,LBIV_GCSR_SIG1
					       ,LBIV_GCSR_SIG2,LBIV_GCSR_SIG3};
LOCAL	int		ICLR_CSIG[4]	= {ICLR_CSIG0,ICLR_CSIG1,ICLR_CSIG2,ICLR_CSIG3};
	void		SIGInt( int sig );			/*routines follow Connect*/


STATUS	SIGConnect (int		sig,	/* particular signal (0 thru 3) */
		    VOIDFUNCPTR	routine,/* routine called at each LM interrupt */
		    int		arg)	/* argument with which to call routine           */
{
/*	void	SIGInt();*/	/* routine follows */

    if (sig > 3) return(ERROR);

    if (!SIGConnected[sig]
	&& intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_GCSR_SIG[sig]))
		       ,SIGInt[sig],sig)==ERROR )
    {
	return (ERROR);
    }
    
    SIGConnected[sig] = TRUE;

    SIGRoutine[sig]   = routine;
    SIGArg[sig]       = arg;

    return (OK);
}


/*
 * SIGInt - handle interrupts
 */

void	SIGInt( int sig )
{
	int	temp;	/* needed to force lword access */

    if (SIGRoutine[sig] != NULL) (*SIGRoutine[sig])(SIGArg[sig]);

    /* clear interrupt */
    temp=~ICLR_CSIG[sig];
    *VMECHIP2_ICLR |= ~temp;
}


/*******************************************************************************
* SIGDisable - turn off location monitor x interrupts
*
* This routine disables the vmechip2 location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: SIGEnable()
*/

LOCAL BOOL SIGRunning[4]         = {FALSE,FALSE,FALSE,FALSE};
LOCAL int  LBIER_ESIG[4]	 = {LBIER_ESIG0,LBIER_ESIG1
					,LBIER_ESIG2,LBIER_ESIG3};

STATUS SIGDisable (int sig)	/* particular signal (0 thru 3) */
{

    if (sig > 3) return(ERROR);

    if (SIGRunning[sig])
    {
	/* disable interrupts */
	*VMECHIP2_LBIER &= ~LBIER_ESIG[sig];	/* also clears status (p.2-69)*/

	SIGRunning[sig] = FALSE;
    }

    return (OK);
}


/********************************************************************************
* SIGEnable - turn on location monitor x interrupts
*
* This routine enables location monitor x interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: SIGConnect(), SIGDisable(), SIGRateSet()
*/

LOCAL int SIGInterruptLevel[4] = {ILR2_SIG0_LEVEL5,ILR2_SIG1_LEVEL5
					,ILR2_SIG2_LEVEL5,ILR2_SIG3_LEVEL5};
LOCAL int ILR2_SIG_LEVEL7[4] = {ILR2_SIG0_LEVEL7,ILR2_SIG1_LEVEL7
				,ILR2_SIG2_LEVEL7,ILR2_SIG3_LEVEL7};


STATUS SIGEnable (int sig)	/* particular signal (0 thru 3) */
{
	int	temp;	/* needed to force lword access */


    if (sig > 3) return(ERROR);

    if (!SIGRunning[sig])
    {
	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR2 &= ~ILR2_SIG_LEVEL7[sig]; /* clear bits before ORing */
	*VMECHIP2_ILR2 |= SIGInterruptLevel[sig];

	temp = ~LBIER_ESIG[sig];		/* enable the interrupt */
	*VMECHIP2_LBIER |= ~temp;		/*  ~ ~ */

	SIGRunning[sig] = TRUE;
    }

    return (OK);
}


/*******************************************************************************
* SIGLevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next SIGEnable call.
*/

LOCAL int ILR2_SIG_LEVEL1[4] = {ILR2_SIG0_LEVEL1,ILR2_SIG1_LEVEL1
				,ILR2_SIG2_LEVEL1,ILR2_SIG3_LEVEL1};
LOCAL int ILR2_SIG_LEVEL2[4] = {ILR2_SIG0_LEVEL2,ILR2_SIG1_LEVEL2
				,ILR2_SIG2_LEVEL2,ILR2_SIG3_LEVEL2};
LOCAL int ILR2_SIG_LEVEL3[4] = {ILR2_SIG0_LEVEL3,ILR2_SIG1_LEVEL3
				,ILR2_SIG2_LEVEL3,ILR2_SIG3_LEVEL3};
LOCAL int ILR2_SIG_LEVEL4[4] = {ILR2_SIG0_LEVEL4,ILR2_SIG1_LEVEL4
				,ILR2_SIG2_LEVEL4,ILR2_SIG3_LEVEL4};
LOCAL int ILR2_SIG_LEVEL5[4] = {ILR2_SIG0_LEVEL5,ILR2_SIG1_LEVEL5
				,ILR2_SIG2_LEVEL5,ILR2_SIG3_LEVEL5};
LOCAL int ILR2_SIG_LEVEL6[4] = {ILR2_SIG0_LEVEL6,ILR2_SIG1_LEVEL6
				,ILR2_SIG2_LEVEL6,ILR2_SIG3_LEVEL6};
/* 7 done above */

STATUS SIGLevelSet (int sig,	/* particular signal (0 thru 3) */
		    int	level)
{
    if (sig > 3) return(ERROR);

    switch(level)
    {
    case 0:	SIGInterruptLevel[sig] = 0;			break;
    case 1:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL1[sig]; break;
    case 2:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL2[sig]; break;
    case 3:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL3[sig]; break;
    case 4:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL4[sig]; break;
    case 5:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL5[sig]; break;
    case 6:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL6[sig]; break;
    case 7:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL7[sig]; break;
    default:	SIGInterruptLevel[sig] = ILR2_SIG_LEVEL5[sig]; break;
    }

    return (OK);
}


/*******************************************************************************
* SIGAddressSet
*
*/

STATUS SIGAddressSet (unsigned short addr)
{
	int	temp;	/* needed to force lword access */
	int	temp2;


    addr &= 0x0000fff0;		/* group and board part */
    temp2 = addr;
    temp2 <<= 16;		/* move to upper word (board is 00 from above)*/
				/*  (ref p.2-41) */
    temp = *VMECHIP2_LBTVCR;
    temp &= 0x000fffff;		/* clear bits */
    temp |= temp2;
    *VMECHIP2_LBTVCR = temp;
}


/*******************************************************************************
* mb,mw,ml
*/

LOCAL	unsigned char	*mb_addr=0;
	unsigned short	*mw_addr=0;
	unsigned int	*ml_addr=0;


STATUS mb (unsigned char *addr,
	   unsigned char  data,
	   unsigned int	  special)
{
	char		buf[80];
	char		*ptr;
	unsigned int	d_ans;

    if(addr!=0) mb_addr=addr;

    if ( (data!=0) || ((data==0) && (special!=0)) )
    {
	*addr  = data;
    }
    else
    {

	while (1)
	{
	    printf("0x%08.8x = 0x%02.2x-",mb_addr,*mb_addr);
	    ptr=gets(buf);		/* strlen will be 0 if just Return is pressed */
	    if (ptr == NULL) return(ERROR);
	    if (strlen(buf)!=0)
            {
		if (strncmp(buf,".",1)==0) return(OK);
		if (strncmp(buf,"0x",2)==0 || strncmp(2,buf,"0X")==0)
		{
		    if (sscanf(buf,"0x%x",&d_ans) != 1) return(ERROR);
		    d_ans &= 0xff;
		}
		else
		{
		    if (sscanf(buf,"%u",&d_ans) != 1) return(ERROR);
		    if (d_ans > 0xff) return(ERROR);
		}
/*		printf("the value entered was 0x%2.2x (%d)\n",d_ans,d_ans);
*/		*mb_addr=(unsigned char)d_ans;
	    }
	    mb_addr++;
	}
    }
    return(OK);
}
STATUS mw (unsigned short *addr,
	   unsigned short  data,
	   unsigned int	   special)
{
	char		buf[80];
	char		*ptr;
	unsigned int	d_ans;

    if(addr!=0) mw_addr=addr;

    if ( (data!=0) || ((data==0) && (special!=0)) )
    {
	*addr  = data;
    }
    else
    {

	while (1)
	{
	    printf("0x%08.8x = 0x%04.4x-",mw_addr,*mw_addr);
	    ptr=gets(buf);		/* strlen will be 0 if just Return is pressed */
	    if (ptr == NULL) return(ERROR);
	    if (strlen(buf)!=0)
	    {
		if (strncmp(buf,".",1)==0) return(OK);
		if (strncmp(buf,"0x",2)==0 || strncmp(2,buf,"0X")==0)
		{
		    if (sscanf(buf,"0x%x",&d_ans) != 1) return(ERROR);
		    d_ans &= 0xffff;
		}
		else
		{
		    if (sscanf(buf,"%u",&d_ans) != 1) return(ERROR);
		    if (d_ans > 0xffff) return(ERROR);
		}
/*		printf("the value entered was 0x%4.4x (%d)\n",d_ans,d_ans);
*/		*mw_addr=(unsigned short)d_ans;
	    }
	    mw_addr++;
	}
    }
    return(OK);
}
STATUS ml (unsigned int	*addr,
	   unsigned int	 data,
	   unsigned int	 special)
{
	char		buf[80];
	char		*ptr;
	unsigned int	d_ans;

    if(addr!=0) ml_addr=addr;

    if ( (data!=0) || ((data==0) && (special!=0)) )
    {
	*addr  = data;
    }
    else
    {

	while (1)
	{
	    printf("0x%08.8x = 0x%08.8x-",ml_addr,*ml_addr);
	    ptr=gets(buf);		/* strlen will be 0 if just Return is pressed */
	    if (ptr == NULL) return(ERROR);
	    if (strlen(buf)!=0)
	    {
		if (strncmp(buf,".",1)==0) return(OK);
		if (strncmp(buf,"0x",2)==0 || strncmp(2,buf,"0X")==0)
		{
		    if (sscanf(buf,"0x%8x",&d_ans) != 1) return(ERROR);
		}
		else
		{
		    if (sscanf(buf,"%u",&d_ans) != 1) return(ERROR);
		}
/*		printf("the value entered was 0x%8.8x (%d)\n",d_ans,d_ans);
*/		*ml_addr=(unsigned int)d_ans;
	    }
	    ml_addr++;
	}
    }
    return(OK);
}


/*******************************************************************************
* vmeLock
*
* This routine commands the vme bus requester hardware to request the VME bus
* such that when it gets the bus, it will hold it until commanded to release
* the bus (via vmeUnlock()).
*
* This routine is intended to be used as such:
*             vmeLock();
*             vme read
*             if read value such that uninterruptable cycle can continue,
*                then do further vme xfers (cycles) with the bus still locked.
*                 and release the bus after.
*                 vmeUnlock();
*                 break out of the loop (if looping)
*             else release the bus right way
*               vmeUnlock(); allow other vme masters to have the bus
*               loop back to top to try again (if looping)
*/

void vmeLock (void)
{
	unsigned int	temp;

    temp = *VMECHIP2_DMACR1;
    temp |= DMACR1_DWB;
    *VMECHIP2_DMACR1 = temp;
}


/*******************************************************************************
* vmeUnlock
*/

void vmeUnlock (void)
{
	unsigned int	temp;

    temp = *VMECHIP2_DMACR1;
    temp &= ~DMACR1_DWB;
    *VMECHIP2_DMACR1 = temp;
}


/*******************************************************************************
* vmeLocked
*/

int vmeLocked (void)
{
	unsigned int	temp;

    temp = *VMECHIP2_DMACR1;
    return( (temp&DMACR1_DHB)?1:0 );
}


/*******************************************************************************
* vmeSlaveMapShow
*/

void vmeSlaveMapShow (void)
{
	unsigned int	temp;
	unsigned int	vsamsr;
	unsigned int	vsar;


    vsamsr = *VMECHIP2_VSAMSR;
    if (   (  (vsamsr&VSAMSR1_SUP)||(vsamsr&VSAMSR1_USR) )
	 &&(  (vsamsr&VSAMSR1_A32)||(vsamsr&VSAMSR1_A24) )
	 &&(  (vsamsr&VSAMSR1_D64)||(vsamsr&VSAMSR1_BLK)||(vsamsr&VSAMSR1_PGM)
	    ||(vsamsr&VSAMSR1_DAT) )
       )
    {
	printf("\nSlave Window #1 valid AMs: %s %s %s %s %s %s %s %s\n",
		(vsamsr&VSAMSR1_SUP)?"SUP":"",
		(vsamsr&VSAMSR1_USR)?"USR":"",

		(vsamsr&VSAMSR1_A32)?"A32":"",
		(vsamsr&VSAMSR1_A24)?"A24":"",

		(vsamsr&VSAMSR1_D64)?"D64":"",
		(vsamsr&VSAMSR1_BLK)?"BLK":"",
		(vsamsr&VSAMSR1_PGM)?"PGM":"",
		(vsamsr&VSAMSR1_DAT)?"DAT":"" );
	printf("Slave Window #1 write posting: %sABLE\n",vsamsr&VSAMSR1_WP?"EN":"DIS");
	switch( (vsamsr& (VSAMSR1_SNP_WSD|VSAMSR1_SNP_WI)) )
	{
	default:
	case VSAMSR1_SNP_INHIBIT:
	    printf("Slave Window #1 Snoop inhibited\n");
	    break;
	case VSAMSR1_SNP_WSD:
	    printf("\
Slave Window #1 Write - Sink data; Read - Supply dirty data and leave dirty\n");
	    break;
	case VSAMSR1_SNP_WI:
	    printf("\
Slave Window #1 Write - Invalidate; Read - Supply dirty data and mark invalid\n");
	    break;
	}

	vsar = *VMECHIP2_VSAR1;
	printf("VME Slave window #1 starting address = 0x%08.8x\n",vsar<<16);
	printf("VME Slave window #1 ending   address = 0x%08.8x\n"
		,(vsar&0xffff0000)+0xffff);

	temp = *VMECHIP2_VSATR1;
	printf("VME to Localbus translation Address for Slave Window #1 = 0x%08.8x\n",
							temp&0xffff0000 );
	printf("VME to Localbus translation Select  for Slave Window #1 = 0x%08.8x\n",
							temp<<16 );
	if(vsamsr&VSAMSR1_A32)
	{
	    printf("\texample: VME A32 address => Localbus address\n" );
	    printf("\t         ---------------    ----------------\n" );
	    printf("\t           0x%08.8x         0x%08.8x\n",vsar<<16,
				(temp&(temp<<16))|((vsar<<16)&~(temp<<16)) );
	    printf("\t           0x%08.8x         0x%08.8x\n",vsar|0xffff,
				(temp&(temp<<16))|((vsar|0xffff)&~(temp<<16)) );
	}
	if(vsamsr&VSAMSR1_A24)
	{
	    printf("\texample: VME A24 address => Localbus address\n" );
	    printf("\t         ---------------    ----------------\n" );
	    temp |= 0x0000ff00;
    printf("\t             0x%06.6x           0x%06.6x\n",(vsar<<16)&0xffffff,
			(temp&(temp<<16))|(((vsar<<16)&0xffffff)&~(temp<<16)) );
    printf("\t             0x%06.6x           0x%06.6x\n",(vsar|0xffff)&0xffffff,
			(temp&(temp<<16))|(((vsar|0xffff)&0xffffff)&~(temp<<16)) );
	}
    }
    else
    {
	printf("\nSlave Window #1 is DISABLED\n");
    }

    /*----------------------------------------------------------------------*/

    if (   (  (vsamsr&VSAMSR2_SUP)||(vsamsr&VSAMSR2_USR) )
	 &&(  (vsamsr&VSAMSR2_A32)||(vsamsr&VSAMSR2_A24) )
	 &&(  (vsamsr&VSAMSR2_D64)||(vsamsr&VSAMSR2_BLK)||(vsamsr&VSAMSR2_PGM)
	    ||(vsamsr&VSAMSR2_DAT) )
       )
    {
	printf("\nSlave Window #2 valid AMs: %s %s %s %s %s %s %s %s\n",
		(vsamsr&VSAMSR2_SUP)?"SUP":"",
		(vsamsr&VSAMSR2_USR)?"USR":"",

		(vsamsr&VSAMSR2_A32)?"A32":"",
		(vsamsr&VSAMSR2_A24)?"A24":"",

		(vsamsr&VSAMSR2_D64)?"D64":"",
		(vsamsr&VSAMSR2_BLK)?"BLK":"",
		(vsamsr&VSAMSR2_PGM)?"PGM":"",
		(vsamsr&VSAMSR2_DAT)?"DAT":"" );
	printf("Slave Window #2 write posting: %sABLE\n",vsamsr&VSAMSR2_WP?"EN":"DIS");
	switch( (vsamsr& (VSAMSR2_SNP_WSD|VSAMSR2_SNP_WI)) )
	{
	default:
	case VSAMSR2_SNP_INHIBIT:
	    printf("Slave Window #2 Snoop inhibited\n");
	    break;
	case VSAMSR2_SNP_WSD:
	    printf("\
Slave Window #2 Write - Sink data; Read - Supply dirty data and leave dirty\n");
	    break;
	case VSAMSR2_SNP_WI:
	    printf("\
Slave Window #2 Write - Invalidate; Read - Supply dirty data and mark invalid\n");
	    break;
	}

	vsar = *VMECHIP2_VSAR2;
	printf("VME Slave window #2 starting address = 0x%08.8x\n",vsar<<16);
	printf("VME Slave window #2 ending   address = 0x%08.8x\n"
			,(vsar&0xffff0000)+0xffff);

	temp = *VMECHIP2_VSATR2;
	printf("VME to Localbus translation Address for Slave Window #2 = 0x%08.8x\n",
							temp&0xffff0000 );
	printf("VME to Localbus translation Select  for Slave Window #2 = 0x%08.8x\n",
							temp<<16 );
	if(vsamsr&VSAMSR2_A32)
	{
	    printf("\texample: VME A32 address => Localbus address\n" );
	    printf("\t         ---------------    ----------------\n" );
	    printf("\t           0x%08.8x         0x%08.8x\n",vsar<<16,
				(temp&(temp<<16))|((vsar<<16)&~(temp<<16)) );
	    printf("\t           0x%08.8x         0x%08.8x\n",vsar|0xffff,
				(temp&(temp<<16))|((vsar|0xffff)&~(temp<<16)) );
/*	    printf("\t           0x12345678         0x%08.8x\n",
*				(temp&(temp<<16))|(0x12345678&~(temp<<16)) );
*	    printf("\t           0x20001234         0x%08.8x\n",
*				(temp&(temp<<16))|(0x20001234&~(temp<<16)) );
*	    printf("\t           0x02123456         0x%08.8x\n",
*				(temp&(temp<<16))|(0x02123456&~(temp<<16)) );
*/	}
	if(vsamsr&VSAMSR2_A24)
	{
	    printf("\texample: VME A24 address => Localbus address\n" );
	    printf("\t         ---------------    ----------------\n" );
	    temp |= 0x0000ff00;
    printf("\t             0x%06.6x           0x%06.6x\n",(vsar<<16)&0xffffff,
			(temp&(temp<<16))|(((vsar<<16)&0xffffff)&~(temp<<16)) );
    printf("\t             0x%06.6x           0x%06.6x\n",(vsar|0xffff)&0xffffff,
			(temp&(temp<<16))|(((vsar|0xffff)&0xffffff)&~(temp<<16)) );
/*	    printf("\t             0x345678           0x%06.6x\n",
*				(temp&(temp<<16))|(0x345678&~(temp<<16)) );
*/	}
    }
    else
    {
	printf("\nSlave Window #2 is DISABLED\n");
    }
}


/*******************************************************************************
* vmeMasterMapShow
*/

void vmeMasterMapShow (void)
{
	unsigned int	temp;
	unsigned int	lbstart;
	unsigned int	lbsar;
	unsigned int	lbtvcr;


    printf("\nnon-programable maps (I/O):\n");

    lbtvcr = *VMECHIP2_LBTVCR;

    printf("  I/O window #1:");
    if (lbtvcr & LBTVCR_I1EN)
    {
	printf("\n    VME A16/D%s space at Localbus address 0xffff0000-0xffffffff   (AM=0x%x)\n"
	  ,(lbtvcr&LBTVCR_I1D16)?"16":"32",(lbtvcr&LBTVCR_I1SUP)?0x2D:0x29);
    }
    else printf(" DISABLED\n");

    printf("  I/O window #2:");
    if (lbtvcr & LBTVCR_I2EN)
    {
	printf("\n    VME A24/D16 space at Localbus address 0xf0000000-0xf0ffffff and");
	printf("\n    VME A32/D16 space at Localbus address 0xf1000000-0xfeffffff   (%s)\n"
	  ,(lbtvcr&LBTVCR_I2SUP)?"SUP":"USR");
    }
    else printf(" DISABLED\n");


    printf("\nprogramable maps:\n");

    lbsar = *VMECHIP2_LBSAR;

    printf("  window #1:");
    if (lbtvcr&LBTVCR_EN1)
    {    
	temp = *VMECHIP2_LBSAR1;
	printf("\n    Localbus to VME master window #1 0x%08.8x - 0x%08.8x\n"
			,temp<<16,(temp&0xffff0000)+0xffff);
	printf("    DTB width: %s; write posting: %sABLE; AM: 0x%02.2x\n",
	    (lbsar&LBSAR1_D16)?"D16":"D32",(lbsar&LBSAR1_WP_ENABLE)?"EN":"DIS",
	    lbsar&0x3f);
    }
    else
    {
	printf(" DISABLED\n");
    }

    printf("  window #2:");
    if (lbtvcr&LBTVCR_EN2)
    {    
	temp = *VMECHIP2_LBSAR2;
	printf("\n    Localbus to VME master window #2 0x%08.8x - 0x%08.8x\n"
			,temp<<16,(temp&0xffff0000)+0xffff);
	printf("    DTB width: %s; write posting: %sABLE; AM: 0x%02.2x\n",
	    (lbsar&LBSAR2_D16)?"D16":"D32",(lbsar&LBSAR2_WP_ENABLE)?"EN":"DIS",
	    (lbsar&0x3f00)>>8);
    }
    else
    {
	printf(" DISABLED\n");
    }

    printf("  window #3:");
    if (lbtvcr&LBTVCR_EN3)
    {    
	temp = *VMECHIP2_LBSAR3;
	printf("\n    Localbus to VME master window #3 0x%08.8x - 0x%08.8x\n"
			,temp<<16,(temp&0xffff0000)+0xffff);
	printf("    DTB width: %s; write posting: %sABLE; AM: 0x%02.2x\n",
	    (lbsar&LBSAR3_D16)?"D16":"D32",(lbsar&LBSAR3_WP_ENABLE)?"EN":"DIS",
	    (lbsar&0x3f0000)>>16);
    }
    else
    {
	printf(" DISABLED\n");
    }

    printf("  window #4:");
    if (lbtvcr&LBTVCR_EN4)
    {    
	temp = *VMECHIP2_LBSAR4;
	printf("\n    Localbus to VME master window #4 0x%08.8x - 0x%08.8x\n"
			,temp<<16,(temp&0xffff0000)+0xffff);
	printf("    DTB width: %s; write posting: %sABLE; AM: 0x%02.2x\n",
	    (lbsar&LBSAR4_D16)?"D16":"D32",(lbsar&LBSAR4_WP_ENABLE)?"EN":"DIS",
	    (lbsar&0x3f000000)>>24);

	lbstart=temp<<16;

	temp = *VMECHIP2_LBSATR1; /* should be VMECHIP2_LBSATR4 */
	if (temp&0xffff)
	{
	    printf("    Localbus to VME translation Address for Master Window #4 = 0x%08.8x\n",
							temp&0xffff0000 );
	    printf("    Localbus to VME translation Select  for Master Window #4 = 0x%08.8x\n",
							temp<<16 );
	    printf("    example: Localbus address => VMEbus address\n" );
	    printf("             ----------------    ----------------\n" );
	    printf("               0x87654321         0x%08.8x\n",
				(temp&(temp<<16))|(0x87654321&~(temp<<16)) );
	    printf("               0x01234567         0x%08.8x\n",
				(temp&(temp<<16))|(0x01234567&~(temp<<16)) );
	    printf("               0x%08.8X         0x%08.8X\n",lbstart,
				(temp&(temp<<16))|(lbstart&~(temp<<16)) );
	}
	else
	{
	    printf("    optional address translation for window #4 DISABLED\n");
	}
    }
    else
    {
	printf(" DISABLED\n");
    }

    printf("\n");
}


#ifdef mv167
/*******************************************************************************
* interruptLevelShow
*/
void interruptLevelShow (void)
{
    interruptShow ();
}

int interruptShow (void)
{
	unsigned int	t,isr,ier,i;
	unsigned int	vbr0,vbr1,iocr;
	unsigned char	c,x,y,z,w[12],icr;
	unsigned int	usp, msp, isp;		/* user/master/interrupt sp */
	unsigned short	sr;


    asm(".word 0x40c0; movew d0,%0; " : "=g" (sr) );  /* get MC68040 SR */
    asm(".word 0x4e7a; .word 0x0800; movel d0,%0; " : "=g" (usp) );
    asm(".word 0x4e7a; .word 0x0803; movel d0,%0; " : "=g" (msp) );
    asm(".word 0x4e7a; .word 0x0804; movel d0,%0; " : "=g" (isp) );

printf("\
040 supervisorMode=%s levelMask=%d,\n\
    usp=0x%08.8x, msp=0x%08.8x, isp=0x%08.8x (M state=%s)"
      , (sr&0x2000)?"super":"user", (sr>>8)&7
      , usp, msp, isp, (sr&0x1000)?"master":"interrupt" );

    iocr = *VMECHIP2_IOCR;
    vbr1 = (iocr >> 20) & 0xf0;
    vbr0 = (iocr >> 24) & 0xf0;

    isr = *VMECHIP2_LBISR;
    ier = *VMECHIP2_LBIER;

printf("\nVMEchip2 interrupts" );
#define H0	"                       (enabled/active) "
#define H1	"Interrupt          lvl/vect   e/a       "
#define H2	"---------          --------  -----      "
printf("\n" H0  H0);
printf("\n" H1  H1);
printf("\n" H2  H2);

#define P0(a,b,c)	(t>>a)&7,vbr0+b,(ier&c)?1:0,(isr&c)?1:0
#define P1(a,b,c)	(t>>a)&7,vbr1+b,(ier&c)?1:0,(isr&c)?1:0
#define PEx(a,c)	(t>>a)&7,(ier&c)?1:0,(isr&c)?1:0
#define S0	" %u/%x     %d/%d;      "
#define S1	" %u/Ext    %d/%d;      "
    t = *VMECHIP2_ILR1;

printf("\nAC Fail:            "S0"Abort button:       "S0,P0(28,0xf,LBISR_ACF),P0(24,0xe,LBISR_AB));
printf("\nmaster WP BERR:     "S0"SYSFAIL:            "S0,P0(20,0xc,LBISR_MWP),P0(16,0xd,LBISR_SYSF));
printf("\nexternal (parity):  "S0"VME IRQ edge:       "S0,P0(12,0xb,LBISR_PE),P0(8,0xa,LBISR_VI1E));
printf("\ntick timer 1:       "S0"tick timer 2:       "S0,P0( 4,0x8,LBISR_TIC1),P0(0,0x9,LBISR_TIC2));

    t = *VMECHIP2_ILR2;
printf("\nVME interrupt ACKed:"S0"DMA Controller:     "S0,P0(28,0x7,LBISR_VIA),P0(24,0x6,LBISR_DMA));
printf("\nVME SIGnal 3:       "S0"VME SIGnal 2:       "S0,P0(20,0x5,LBISR_SIG3),P0(16,0x4,LBISR_SIG2));
printf("\nVME SIGnal 1:       "S0"VME SIGnal 0:       "S0,P0(12,0x3,LBISR_SIG1),P0(8,0x2,LBISR_SIG0));
printf("\nVME LM1:            "S0"VME LM0:            "S0,P0( 4,0x1,LBISR_LM1),P0(0,0x0,LBISR_LM0));

    t = *VMECHIP2_ILR3;
printf("\nSW bit 7:           "S0"SW bit 6:           "S0,P1(28,0xf,LBISR_SW7),P1(24,0xe,LBISR_SW6));
printf("\nSW bit 5:           "S0"SW bit 4:           "S0,P1(20,0xd,LBISR_SW5),P1(16,0xc,LBISR_SW4));
printf("\nSW bit 3:           "S0"SW bit 2:           "S0,P1(12,0xb,LBISR_SW3),P1(8,0xa,LBISR_SW2));
printf("\nSW bit 1:           "S0"SW bit 0:           "S0,P1( 4,0x9,LBISR_SW1),P1(0,0x8,LBISR_SW0));

    t = *VMECHIP2_ILR4;
printf("\nSPARE?:             "S0"VME IRQ7:           "S1,P1(28,0x7,(1<<7)),PEx(24,LBISR_VME7));
printf("\nVME IRQ6:           "S1"VME IRQ5:           "S1,PEx(20,LBISR_VME6),PEx(16,LBISR_VME5));
printf("\nVME IRQ4:           "S1"VME IRQ3:           "S1,PEx(12,LBISR_VME4),PEx( 8,LBISR_VME3));
printf("\nVME IRQ2:           "S1"VME IRQ1:           "S1,PEx( 4,LBISR_VME2),PEx( 0,LBISR_VME1));

printf("\nMIEN = %d\n",(iocr&IOCR_MEIN)?1:0);

printf("Press q to quit, any other key to see PCCchip2 interrupts ");
	if (getKeyPress() == 'q') return(1);

printf("\nPCCchip2 interrupts");
printf("\n" H0  H0);
printf("\n" H1  H1);
printf("\n" H2  H2);

#define P2p(a,b,x,y)	(*a)&7,c+b,((*a)&0x10)?1:0,((*a)&0x20)?1:0,(*x)&7,c+y,((*x)&0x10)?1:0,((*x)&0x20)?1:0
#define P1p(a,b)	(*a)&7,b,((*a)&0x10)?1:0,((*a)&0x20)?1:0
    c = (*PCC2_VBR)&0xf0;			/* ref p.3-15 */
printf("\nPrinter Port-BSY:   "S0"Printer Port-PE:    "S0,P2p(PCC2_PRINTER_BUSY_IRQ,0,PCC2_PRINTER_PE_IRQ,1));
printf("\nPrinter Port-SELECT:"S0"Printer Port-FAULT: "S0,P2p(PCC2_PRINTER_SEL_IRQ,2,PCC2_PRINTER_FAULT_IRQ,3));
printf("\nPrinter Port-ACK:   "S0"SCSI IRQ:           "S0,P2p(PCC2_PRINTER_ACK_IRQ,4,PCC2_SCSI_IRQ_CR,5));
printf("\nLANC ERR:           "S0"LANC IRQ:           "S0,P2p(PCC2_LANC_BEICR,6,PCC2_LANC_IRQ_CR,7));
printf("\nTick Timer 2 IRQ:   "S0"Tick Timer 1 IRQ:   "S0,P2p(PCC2_T2_IRQ_CR,8,PCC2_T1_IRQ_CR,9));
printf("\nGPIO IRQ:           "S0                       ,P1p(PCC2_GPIICR,c+10));

printf("\n                               xIR=<en,act,eoi,0,vct1,vct0,cn1,cn0>");

    if ((*PCC2_SCC_MICR)&SCC_MICR_AVEC)		/* ref p.3-24 */
    {
printf("\nSerial Modem (group 1) IRQ (AVEC): lvl/vect=%d/%d;  enable/active=%d/%d",P1p(PCC2_SCC_MICR,c+11));
    }
    if ((*PCC2_SCC_RICR)&SCC_RICR_AVEC)
    {
printf("\nSerial RX    (group 3) IRQ (AVEC): lvl/vect=%d/%d;  enable/active=%d/%d",P1p(PCC2_SCC_RICR,c+12));
    }
    if ((*PCC2_SCC_TICR)&SCC_TICR_AVEC)
    {
printf("\nSerial TX    (group 3) IRQ( AVEC): lvl/vect=%d/%d;  enable/active=%d/%d",P1p(PCC2_SCC_TICR,c+13));
    }

#define P0p(a)		(*a)&7,((*a)&0x10)?1:0,((*a)&0x20)?1:0
#define LVLCHV		" IRQ: lvl/vect=%d/per_ch_from_cd2400  en/act=%d/%d"
    x=0;
    if (!((*PCC2_SCC_MICR)&SCC_MICR_AVEC))
    {
	x=1;
printf("\nSer.Modm(grp1)"LVLCHV,P0p(PCC2_SCC_MICR));
	for (i=0;i<4;i++)
	{
	    sr = intLock();
	    *MPCC_CAR = i;
	    w[i*3]=((*MPCC_LIVR)&0xfc)+1;
	    w[(i*3)+1]=((*MPCC_IER)&0x80)?1:0;
	    w[(i*3)+2]=((*MPCC_IER)&4)?1:0;
	    intUnlock(sr);
	}
printf("\n\
 cd2400 modm ch0-3: vects=%x,%x,%x,%x pin_en=%d,%d,%d,%d timer_en=%d,%d,%d,%d"
	,w[0],w[3],w[6],w[9],w[1],w[4],w[7],w[10],w[2],w[5],w[8],w[11]);
	/* Pete says the status registers are only valid during interrupt context */
printf("\n cd2400 modm globl: lvl=0x%x MIR=0x%x MISR?=<DSR,CD,CTSchg,-,-,-,tim2,tim1>=0x%02x"
	,*MPCC_PILR1,*MPCC_MIR
	,*MPCC_MISR);
    }

    if (!((*PCC2_SCC_RICR)&SCC_RICR_AVEC))
    {
	x=1;
printf("\nSer.RX  (grp3)"LVLCHV,P0p(PCC2_SCC_RICR));
	for (i=0;i<4;i++)
	{
	    sr = intLock();
	    *MPCC_CAR = i;
	    w[i*3]=((*MPCC_LIVR)&0xfc)+3;
	    w[(i*3)+1]=((*MPCC_IER)&0x20)?1:0;
	    w[(i*3)+2]=((*MPCC_IER)&0x8)?1:0;
	    intUnlock(sr);
	}
printf("\n\
 cd2400 RX : vct(dat/exc)=%x/%x,%x/%x,%x/%x,%x/%x TOexcEn=%d,%d,%d,%d dat+excEn=%d%d%d%d"
	,w[0],w[0]-3,w[3],w[3]-3,w[6],w[6]-3,w[9],w[9]-3,w[1],w[4],w[7],w[10],w[2],w[5],w[8],w[11]);
	/* Pete says the status registers are only valid during interrupt context */
	/* RISRL definition is also protocol specific */
printf("\n cd2400 RX globl: lvl=0x%02x RIR=0x%02x RISR?=ASYNC<TO,spChar(3bits),OE,PE,FE,brk>=0x%02x \
DMA_Berr=%d EOF=%d EOB=%d BB/BA=%d?)"
	,*MPCC_PILR3,*MPCC_RIR
	,*MPCC_RISRL,(*MPCC_RISRH&0x80)?1:0,(*MPCC_RISRH&0x40)?1:0,(*MPCC_RISRH&0x20)?1:0,(*MPCC_RISRH&8)?1:0);
    }

    if (!((*PCC2_SCC_TICR)&SCC_TICR_AVEC))
    {
	x=1;
printf("\nSer.TX  (grp2)"LVLCHV,P0p(PCC2_SCC_TICR));
	for (i=0;i<4;i++)
	{
	    sr = intLock();
	    *MPCC_CAR = i;
	    w[i*3]=((*MPCC_LIVR)&0xfc)+2;
	    w[(i*3)+1]=((*MPCC_IER)&2)?1:0;
	    w[(i*3)+2]=(*MPCC_IER)&1;
	    intUnlock(sr);
	}
printf("\n cd2400 TX ch0-3: vects=%x,%x,%x,%x TX_empt_en=%d,%d,%d,%d TX_excpt_en=%d,%d,%d,%d"
	,w[0],w[3],w[6],w[9],w[1],w[4],w[7],w[10],w[2],w[5],w[8],w[11]);
	/* Pete says the status registers are only valid during interrupt context */
printf("\n cd2400 TX globl: lvl=0x%02x TIR=0x%02x TISR?=<Ber,EOF,EOB,UE,BB/BA,0,Empt,FIF>=0x%02x"
	,*MPCC_PILR2,*MPCC_TIR
	,*MPCC_TISR);
    }


/*If any non-avec, then show STK reg p88 */
    if (x==1)
    {
printf("\n cd2400 Stack Reg {STK} = 0x%02x",*MPCC_STK);
    }

printf("\nMIEN = %d,  IPL reg = %d,  Interrupt Mask Level reg = %d\n"
	,((*PCC2_GCR)&GCR_MIEN_ON)?1:0, *PCC2_IPLR, *PCC2_IMLR); /* ref p.3-13,3-43-44 */
}
#endif

/*******************************************************************************
* vmeSlaveMap1
*/

void vmeSlaveMap1 (unsigned int am, 
		   unsigned int vmestart,
                   unsigned int vmeend, 
		   unsigned int lbstart)
{
	int	temp;	/* needed to force lword access */
	int	x;


    temp = *VMECHIP2_VSAMSR;
    temp &=0xffff0000;		/* clear out map 1 part */
    if ( (am<0x40)&&(am>0x37) )
    {
	temp |= VSAMSR1_A24;
    }
    else if ( (am<0x10)&&(am>0x07) )
    {
	temp |= VSAMSR1_A32;
    }
    else if (am==0)
    {
	/* this disables the mapping */
	temp &= ~(VSAMSR1_SUP|VSAMSR1_USR|VSAMSR1_A32|VSAMSR1_A24|VSAMSR1_D64|VSAMSR1_BLK|VSAMSR1_PGM|VSAMSR1_DAT);
	*VMECHIP2_VSAMSR =temp;
	return;
    }
    else
    {
	/* invalide am */
	return;
    }

    /* NOW DEFAULT SNOOP and WRITE POST */
    temp |= (VSAMSR1_SNP_WSD | VSAMSR1_WP);

    if (am&4) temp |= VSAMSR1_SUP;
    else      temp |= VSAMSR1_USR;
    switch(am&3)
    {
    case 0: temp|=VSAMSR1_D64; break;
    case 1: temp|=VSAMSR1_DAT; break;
    case 2: temp|=VSAMSR1_PGM; break;
    case 3: temp|=VSAMSR1_BLK; break;
    }
    *VMECHIP2_VSAMSR = temp;

    *VMECHIP2_VSAR1 = ((vmeend&0xffff0000)|(vmestart>>16));


    if(vmestart!=lbstart)
    {
	/* select part */
	temp = ((vmeend&0xffff0000)-(vmestart&0xffff0000));
	temp >>= 16;
	x=0;
	while ( temp ) {temp >>= 1; x++;}
	temp = 0xffff << x;
	/* address part is just lbstart */
	if ( (am<0x40)&&(am>0x37) ) /* standard addressing */ temp |= 0xff00;
	*VMECHIP2_VSATR1 = (lbstart&0xffff0000) | (temp&0x0000ffff);
    }
    else *VMECHIP2_VSATR1 = 0;
}


/*******************************************************************************
* vmeSlaveMap2
*/

void vmeSlaveMap2 (unsigned int am, 
		   unsigned int vmestart,
                   unsigned int vmeend, 
		   unsigned int lbstart)
{
	int	temp;	/* needed to force lword access */
	int	x;


    temp = *VMECHIP2_VSAMSR;
    temp &=0x0000ffff;		/* clear out map 2 part */
    if ( (am<0x40)&&(am>0x37) )
    {
	temp |= VSAMSR2_A24;
    }
    else if ( (am<0x10)&&(am>0x07) )
    {
	temp |= VSAMSR2_A32;
    }
    else if (am==0)
    {
	temp &= ~(VSAMSR2_SUP|VSAMSR2_USR|VSAMSR2_A32|VSAMSR2_A24|VSAMSR2_D64|VSAMSR2_BLK|VSAMSR2_PGM|VSAMSR2_DAT);
	*VMECHIP2_VSAMSR =temp;
	return;
    }
    else
    {
	/* invalide am */
	return;
    }

    /* NOW DEFAULT SNOOP and WRITE POST */
    temp |= (VSAMSR2_SNP_WSD | VSAMSR2_WP);

    if (am&4) temp |= VSAMSR2_SUP;
    else      temp |= VSAMSR2_USR;
    switch(am&3)
    {
    case 0: temp|=VSAMSR2_D64; break;
    case 1: temp|=VSAMSR2_DAT; break;
    case 2: temp|=VSAMSR2_PGM; break;
    case 3: temp|=VSAMSR2_BLK; break;
    }
    *VMECHIP2_VSAMSR = temp;

    *VMECHIP2_VSAR2 = ((vmeend&0xffff0000)|(vmestart>>16));


    if(vmestart!=lbstart)
    {
	/* select part */
	temp = ((vmeend&0xffff0000)-(vmestart&0xffff0000));
	temp >>= 16;
	x=0;
	while ( temp ) {temp >>= 1; x++;}
	temp = 0xffff << x;
	/* address part is just lbstart */
	if ( (am<0x40)&&(am>0x37) ) /* standard addressing */ temp |= 0xff00;
	*VMECHIP2_VSATR2 = (lbstart&0xffff0000) | (temp&0x0000ffff);
    }
    else *VMECHIP2_VSATR2 = 0;
}


/*******************************************************************************
* vmeSlaveMap1Am
*/

int vmeSlaveMap1Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_VSAMSR;
    if ( (am<0x40)&&(am>0x37) )
    {
	temp |= VSAMSR1_A24;
    }
    else if ( (am<0x10)&&(am>0x07) )
    {
	temp |= VSAMSR1_A32;
    }
    else if (am==0)
    {
	/* this disables the mapping */
	temp &= ~(VSAMSR1_SUP|VSAMSR1_USR|VSAMSR1_A32|VSAMSR1_A24|VSAMSR1_D64|VSAMSR1_BLK|VSAMSR1_PGM|VSAMSR1_DAT);
	*VMECHIP2_VSAMSR =temp;
	return(am);
    }
    else
    {
	/* invalid am */
	return (-1);
    }

    if (am&4) temp |= VSAMSR1_SUP;
    else      temp |= VSAMSR1_USR;
    switch(am&3)
    {
    case 0: temp|=VSAMSR1_D64; break;
    case 1: temp|=VSAMSR1_DAT; break;
    case 2: temp|=VSAMSR1_PGM; break;
    case 3: temp|=VSAMSR1_BLK; break;
    }
    *VMECHIP2_VSAMSR = temp;
    return (am);
}


/*******************************************************************************
* vmeSlaveMap2Am
*/

int vmeSlaveMap2Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_VSAMSR;
    if ( (am<0x40)&&(am>0x37) )
    {
	temp |= VSAMSR2_A24;
    }
    else if ( (am<0x10)&&(am>0x07) )
    {
	temp |= VSAMSR2_A32;
    }
    else if (am==0)
    {
	/* this disables the mapping */
	temp &= ~(VSAMSR2_SUP|VSAMSR2_USR|VSAMSR2_A32|VSAMSR2_A24|VSAMSR2_D64|VSAMSR2_BLK|VSAMSR2_PGM|VSAMSR2_DAT);
	*VMECHIP2_VSAMSR =temp;
	return (am);
    }
    else
    {
	/* invalid am */
	return (-1);
    }

    if (am&4) temp |= VSAMSR2_SUP;
    else      temp |= VSAMSR2_USR;
    switch(am&3)
    {
    case 0: temp|=VSAMSR2_D64; break;
    case 1: temp|=VSAMSR2_DAT; break;
    case 2: temp|=VSAMSR2_PGM; break;
    case 3: temp|=VSAMSR2_BLK; break;
    }
    *VMECHIP2_VSAMSR = temp;
    return (am);
}


/*******************************************************************************
* vmeSlaveMap1SnWp()
*/

void vmeSlaveMap1SnWp (int snoop_write_post)
{
	int	temp;	/* needed to force lword access */

    snoop_write_post &= 0x7;
    
    temp = *VMECHIP2_VSAMSR;
    temp &= ~(VSAMSR1_SNP_WSD|VSAMSR1_SNP_WI|VSAMSR1_WP);	/* clear */

    switch(snoop_write_post)
    {
    case 0:
    case 6:	/* nothing to do */
	break;
    case 1:
    case 7:	temp |= VSAMSR1_WP;
	break;
    case 2:	temp |= VSAMSR1_SNP_WSD;
	break;
    case 3:	temp |= (VSAMSR1_WP|VSAMSR1_SNP_WSD);
	break;
    case 4:	temp |= VSAMSR1_SNP_WI;
	break;
    case 5:	temp |= (VSAMSR1_WP|VSAMSR1_SNP_WI);
	break;
    default:
	break;
    }

    *VMECHIP2_VSAMSR = temp;
}



/*******************************************************************************
* vmeSlaveMap2SnWp()
*/

void vmeSlaveMap2SnWp (int snoop_write_post)
{
	int	temp;	/* needed to force lword access */

    snoop_write_post &= 0x7;
    
    temp = *VMECHIP2_VSAMSR;
    temp &= ~(VSAMSR2_SNP_WSD|VSAMSR2_SNP_WI|VSAMSR2_WP);	/* clear */

    switch(snoop_write_post)
    {
    case 0:
    case 6:	/* nothing to do */
	break;
    case 1:
    case 7:	temp |= VSAMSR2_WP;
	break;
    case 2:	temp |= VSAMSR2_SNP_WSD;
	break;
    case 3:	temp |= (VSAMSR2_WP|VSAMSR2_SNP_WSD);
	break;
    case 4:	temp |= VSAMSR2_SNP_WI;
	break;
    case 5:	temp |= (VSAMSR2_WP|VSAMSR2_SNP_WI);
	break;
    default:
	break;
    }

    *VMECHIP2_VSAMSR = temp;
}



/*******************************************************************************
* vmeMasterMap1
*/

void vmeMasterMap1 (unsigned int am,
		    unsigned int vmestart,
		    unsigned int vmeend,
		    unsigned int width)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN1; /* disable it */
	*VMECHIP2_LBTVCR =temp;
	return;
    }

    temp = *VMECHIP2_LBSAR ;
    if (width== 0) temp &= ~LBSAR1_D16;
    else 	   temp |= LBSAR1_D16;

    temp &= ~(0x3f<<0);
    temp |= (am<<0);
    *VMECHIP2_LBSAR = temp;

    temp = *VMECHIP2_LBTVCR;
    temp |= LBTVCR_EN1;
    *VMECHIP2_LBTVCR = temp;
    *VMECHIP2_LBSAR1 = (vmeend&0xffff0000) | (vmestart>>16);
}


/*******************************************************************************
* vmeMasterMap2
*/

void vmeMasterMap2 (unsigned int am,
		    unsigned int vmestart,
		    unsigned int vmeend,
		    unsigned int width)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN2; /* disable it */
	*VMECHIP2_LBTVCR =temp;
	return;
    }

    temp = *VMECHIP2_LBSAR ;
    if (width== 0) temp &= ~LBSAR2_D16;
    else 	   temp |= LBSAR2_D16;

    temp &= ~(0x3f<<8);
    temp |= (am<<8);
    *VMECHIP2_LBSAR = temp;

    temp = *VMECHIP2_LBTVCR;
    temp |= LBTVCR_EN2;
    *VMECHIP2_LBTVCR = temp;

    *VMECHIP2_LBSAR2 = (vmeend&0xffff0000) | (vmestart>>16);
}


/*******************************************************************************
* vmeMasterMap3
*/

void vmeMasterMap3 (unsigned int am,
		    unsigned int vmestart,
		    unsigned int vmeend,
		    unsigned int width)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN3; /* disable it */
	*VMECHIP2_LBTVCR =temp;
	return;
    }

    temp = *VMECHIP2_LBSAR ;
    if (width== 0) temp &= ~LBSAR3_D16;
    else 	   temp |= LBSAR3_D16;

    temp &= ~(0x3f<<16);
    temp |= (am<<16);
    *VMECHIP2_LBSAR = temp;

    temp = *VMECHIP2_LBTVCR;
    temp |= LBTVCR_EN3;
    *VMECHIP2_LBTVCR = temp;

    *VMECHIP2_LBSAR3 = (vmeend&0xffff0000) | (vmestart>>16);

}


/*******************************************************************************
* vmeMasterMap4
*/

int vmeMasterMap4 (unsigned int am,
		    unsigned int vmestart,
		    unsigned int vmeend,
		    unsigned int lbstart,
		    unsigned int width)
{
	int	temp;	/* needed to force lword access */
	int	x;


    if ( am == 0 )
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN4; /* disable it */
	*VMECHIP2_LBTVCR =temp;
	return ( 1 );
    }

    if ( lbstart == 0 ) return ( -1 );  /* way too dangerous  */


    temp = *VMECHIP2_LBSAR ;
    if (width== 0) temp &= ~LBSAR4_D16;
    else 	   temp |= LBSAR4_D16;

    temp &= ~(0x3f<<24);
    temp |= (am<<24);
    *VMECHIP2_LBSAR = temp;

    temp = *VMECHIP2_LBSATR1;/* set up to disable address xlation in case ... */
    temp &= 0xffff0000;

    if(vmestart!=lbstart)
    {
	/* select part */
	temp = ((vmeend&0xffff0000)-(vmestart&0xffff0000));
	*VMECHIP2_LBSAR4 = ((lbstart+temp)&0xffff0000) | (lbstart>>16);
	temp >>= 16;
	x=0;
	while ( temp ) {temp >>= 1; x++;}
	temp = 0xffff << x;
	/* address part is just vmestart (LBSATR1 should be LBSATR4) */
	*VMECHIP2_LBSATR1 = (vmestart&0xffff0000) | (temp&0x0000ffff);
    }
    else
    {
	/* VMECHIP2_LBSATR1 should be VMECHIP2_LBSATR4 */
	*VMECHIP2_LBSATR1 = temp;
	*VMECHIP2_LBSAR4 = (vmeend&0xffff0000) | (lbstart>>16);
    }

    temp = *VMECHIP2_LBTVCR;
    temp |= LBTVCR_EN4;
    *VMECHIP2_LBTVCR = temp;

    return ( 1) ;
}


/*******************************************************************************
* vmeMasterIoMap2
*/

void vmeMasterIoMap2 (unsigned int am_mode)
{
	int	temp;	/* needed to force lword access */

	/* we wont be using program space - just data, so don't do anything
	   with I2PROG (I2DATA) */
    temp = *VMECHIP2_LBTVCR;
    if ( (am_mode==0) || (am_mode==0x29) || (am_mode==0x39) )
    {
	temp &= ~LBTVCR_I2SUP;
	temp |= LBTVCR_I2EN;
    }
    else if ( (am_mode==1) || (am_mode==0x2d) || (am_mode==0x3d) )
    {
	temp |= LBTVCR_I2SUP;
	temp |= LBTVCR_I2EN;
    }
    else
    {
	temp &= ~LBTVCR_I2EN;
    }
    *VMECHIP2_LBTVCR = temp;
}


/*******************************************************************************
* vmeMasterIoMap1
*/

void vmeMasterIoMap1 (unsigned int am_mode,
		      unsigned int width)
{
	int	temp;	/* needed to force lword access */


    if (am_mode==0x29)
    {
	temp = *VMECHIP2_LBTVCR;
	if(width==0) temp &= ~LBTVCR_I1D16;
	else         temp |= LBTVCR_I1D16;
	temp &= ~LBTVCR_I1SUP;
	temp |= LBTVCR_I1EN;
	*VMECHIP2_LBTVCR = temp;
    }
    else if (am_mode==0x2d)
    {
	temp = *VMECHIP2_LBTVCR;
	if(width==0) temp &= ~LBTVCR_I1D16;
	else         temp |= LBTVCR_I1D16;
	temp |= LBTVCR_I1SUP;
	temp |= LBTVCR_I1EN;
	*VMECHIP2_LBTVCR = temp;
    }
    else
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_I1EN;
	*VMECHIP2_LBTVCR = temp;
    }

}


/*******************************************************************************
* vmeMasterMap1Am
*/

void vmeMasterMap1Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN1; /* disable it */
	*VMECHIP2_LBTVCR = temp;
    }
    else
    {
	temp = *VMECHIP2_LBSAR;
	temp &= ~(0x3f<<0);	/* clear first */
	temp |= (am&0x3f) << 0;	/* OR in am    */
	*VMECHIP2_LBSAR =temp;
    }
}


/*******************************************************************************
* vmeMasterMap2Am
*/

void vmeMasterMap2Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN2; /* disable it */
	*VMECHIP2_LBTVCR = temp;
    }
    else
    {
	temp = *VMECHIP2_LBSAR;
	temp &= ~(0x3f<<8);	/* clear first */
	temp |= (am&0x3f) << 8;	/* OR in am    */
	*VMECHIP2_LBSAR = temp;
    }
}


/*******************************************************************************
* vmeMasterMap3Am
*/

void vmeMasterMap3Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
  	temp &= ~LBTVCR_EN3; /* disable it */
	*VMECHIP2_LBTVCR = temp;
    }
    else
    {
	temp = *VMECHIP2_LBSAR;
	temp &= ~(0x3f<<16);	/* clear first */
	temp |= (am&0x3f) << 16;	/* OR in am    */
	*VMECHIP2_LBSAR = temp;
    }
}


/*******************************************************************************
* vmeMasterMap4Am
*/

void vmeMasterMap4Am (unsigned int am)
{
	int	temp;	/* needed to force lword access */

    if(am==0)
    {
	temp = *VMECHIP2_LBTVCR;
	temp &= ~LBTVCR_EN4; /* disable it */
	*VMECHIP2_LBTVCR = temp;
    }
    else
    {
	temp = *VMECHIP2_LBSAR;
	temp &= ~(0x3f<<24);	/* clear first */
	temp |= (am&0x3f) << 24;	/* OR in am    */
	*VMECHIP2_LBSAR = temp;
    }
}


/*******************************************************************************
* vmeMasterMap1Wp
*/

void vmeMasterMap1Wp (int enable)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_LBSAR ;
    if (enable== 0) temp &= ~LBSAR1_WP_ENABLE;
    else 	    temp |=  LBSAR1_WP_ENABLE;
    *VMECHIP2_LBSAR = temp;
}


/*******************************************************************************
* vmeMasterMap2Wp
*/

void vmeMasterMap2Wp (int enable)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_LBSAR ;
    if (enable== 0) temp &= ~LBSAR2_WP_ENABLE;
    else 	    temp |=  LBSAR2_WP_ENABLE;
    *VMECHIP2_LBSAR = temp;
}


/*******************************************************************************
* vmeMasterMap3Wp
*/

void vmeMasterMap3Wp (int enable)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_LBSAR ;
    if (enable== 0) temp &= ~LBSAR3_WP_ENABLE;
    else 	    temp |=  LBSAR3_WP_ENABLE;
    *VMECHIP2_LBSAR = temp;
}


/*******************************************************************************
* vmeMasterMap4Wp
*/

void vmeMasterMap4Wp (int enable)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_LBSAR ;
    if (enable== 0) temp &= ~LBSAR4_WP_ENABLE;
    else 	    temp |=  LBSAR4_WP_ENABLE;
    *VMECHIP2_LBSAR = temp;
}


/*******************************************************************************
* vmeBR
*/

void vmeBR (int level)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_DMACR1;
    temp &= ~DMACR1_LVREQ_L3;	/* clear it first */
    switch(level)
    {
    case 0:	temp |= DMACR1_LVREQ_L0; break;
    case 1:	temp |= DMACR1_LVREQ_L1; break;
    case 2:	temp |= DMACR1_LVREQ_L2; break;
    case 3:	temp |= DMACR1_LVREQ_L3; break;
    default:	temp |= DMACR1_LVREQ_L0; break;
    }
    *VMECHIP2_DMACR1 = temp;
}



/*******************************************************************************
* dmaShow
*/

void dmaShow (void)
{
	int	temp;

    temp= *VMECHIP2_DMACR1;
    printf("p.2-51     DMAC Control Register 1 = 0x%02.2x\n",temp&0xff);
    printf("             dma VME req level = %d\n",temp&3);
    printf("             dma release mode: ");
    switch(temp&DMACR1_TIMER_OR_BRX)
    {
    case DMACR1_TIMER_BRX:
	printf("Release when timer expire AND BRx active\n");
	break;
    case DMACR1_TIMER:
	printf("Release when timer expire\n");
	break;
    case DMACR1_BRX:
	printf("Release when BRx active\n");
	break;
    case DMACR1_TIMER_OR_BRX:
	printf("Release when timer expire OR BRx active\n");
	break;
    }
    printf("             dma fair mode: %sabled\n",temp&(1<<4)?"en":"dis");
    printf("             dma table snoop mode: %d\n",(temp>>18)&3);
    temp= *VMECHIP2_TIMEOUTCR;
    printf("p.2-60     DMA max time on VME =");
    switch(temp&TIMEOUTCR_ON_DONE)
    {
    case TIMEOUTCR_ON_16US:
	printf("16 us");
	break;
    case TIMEOUTCR_ON_32US:
	printf("32 us");
	break;
    case TIMEOUTCR_ON_64US:
	printf("64 us");
	break;
    case TIMEOUTCR_ON_128US:
	printf("128 us");
	break;
    case TIMEOUTCR_ON_256US:
	printf("256 us");
	break;
    case TIMEOUTCR_ON_512US:
	printf("512 us");
	break;
    case TIMEOUTCR_ON_1024US:
	printf("1024 us");
	break;
    case TIMEOUTCR_ON_DONE:
	printf("no max, till done");
	break;
    }
#ifndef TIMEOUTCR_OFF_16US
# define TIMEOUTCR_OFF_16US TIMEOURCR_OFF_16US
#endif
    printf(";  min time off VME = ");
    switch(temp&TIMEOUTCR_OFF_1024US)
    {
    case TIMEOUTCR_OFF_0US:
	printf("0 us");
	break;
    case TIMEOUTCR_OFF_16US:
	printf("16 us");
	break;
    case TIMEOUTCR_OFF_32US:
	printf("32 us");
	break;
    case TIMEOUTCR_OFF_64US:
	printf("64 us");
	break;
    case TIMEOUTCR_OFF_128US:
	printf("128 us");
	break;
    case TIMEOUTCR_OFF_256US:
	printf("256 us");
	break;
    case TIMEOUTCR_OFF_512US:
	printf("512 us");
	break;
    case TIMEOUTCR_OFF_1024US:
	printf("1024 us");
	break;
    }
    printf("\n");
    printf("p.2-62     Prescale Control Register = %x\n",temp&0xff);

    temp= *VMECHIP2_DMACR2;
    printf("p.2-52,53  DMAC Control Register 2 = 0x%04.4x\n",temp);
    printf("           DMA xfers snoop mode: %d\n",(temp>>13)&3);
    printf("           DMA interrupt for chain packet complete: %sabled\n"
	   ,((temp>>15)&1)? "EN" : "DIS" );
    printf("           DMA address counters incremented during transfer -\n");
    printf("                      VME: %s\n",((temp>>11)&1)?"YES":"NO" );
    printf("                LOCAL BUS: %s\n",((temp>>10)&1)?"YES":"NO" );
    temp= *VMECHIP2_DMACLBAC;
    printf("p.2-54     DMAC Local Bus Address Counter = 0x%08.8x\n",temp);
    temp= *VMECHIP2_DMACVAC;
    printf("p.2-54     DMAC VME   Bus Address Counter = 0x%08.8x\n",temp);
    temp= *VMECHIP2_DMACBC;
    printf("p.2-55     DMAC Byte Counter = 0x%08.8x\n",temp);
    temp= *VMECHIP2_ICR & 0xff;
    printf("p.2-58     DMAC Status Register = 0x%02.2x\n",temp);

}


/*******************************************************************************
* dmaConnect
*/

LOCAL	VOIDFUNCPTR	dmaRoutine	= NULL;
LOCAL	int		dmaArg		= NULL;


STATUS	dmaConnect(  VOIDFUNCPTR	routine	/* routine called at each dma interrupt */
		   , int		arg )	/* argument with which to call routine           */
{
	void	dmaInt();	/* DMA done routine (routine follows) */

    if (intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0 + LBIV_DMAC))
                       ,dmaInt,NULL)==ERROR )
    {
        return (ERROR);
    }

    dmaRoutine   = routine;
    dmaArg       = arg;

    return (OK);
}


/*
* dmaInt - handle dma interrupts
*/

void dmaInt (void)
{
        int     temp;

    if (dmaRoutine != NULL) (*dmaRoutine)(dmaArg);

    /* reset/clear interrupt */
    temp=~ICLR_CDMA;
    *VMECHIP2_ICLR |= ~temp;
}



/*******************************************************************************
* dmaDisable
*/

LOCAL BOOL dmaRunning         = FALSE;


void dmaDisable (void)
{
    if (dmaRunning)
    {
        /* disable interrupts */
        *VMECHIP2_LBIER &= ~LBIER_EDMA;

        dmaRunning = FALSE;
    }
}


/*******************************************************************************
* dmaEnable
*/

LOCAL int dmaInterruptLevel = ILR2_DMA_LEVEL5;

void dmaEnable (void)
{
	int	temp;	/* needed to force lword access */


    if (!dmaRunning)
    {
        /* enable interrupt, and set IRQ level */

        temp = *VMECHIP2_ILR2;
	temp &= ~ILR2_DMA_LEVEL7;	/* clear bits before ORing */
	temp |=  dmaInterruptLevel;	/* I chose the default to be level 5 */
        *VMECHIP2_ILR2 = temp;

	/* kind of a bug - clear interrupt before enabe -	*/
	/*	incase there's one pending.			*/
	temp=~ICLR_CDMA;
	*VMECHIP2_ICLR |= ~temp;

        temp = *VMECHIP2_LBIER;
	temp |= LBIER_EDMA;         /* enable the interrupt */
	*VMECHIP2_LBIER = temp;

        dmaRunning = TRUE;
    }
}


/*******************************************************************************
* dmaLevelSet
*/

void dmaLevelSet (int level)
{
    switch(level)
    {
    case 0:	dmaInterruptLevel = 0;			break;
    case 1:	dmaInterruptLevel = ILR2_DMA_LEVEL1; break;
    case 2:	dmaInterruptLevel = ILR2_DMA_LEVEL2; break;
    case 3:	dmaInterruptLevel = ILR2_DMA_LEVEL3; break;
    case 4:	dmaInterruptLevel = ILR2_DMA_LEVEL4; break;
    case 5:	dmaInterruptLevel = ILR2_DMA_LEVEL5; break;
    case 6:	dmaInterruptLevel = ILR2_DMA_LEVEL6; break;
    case 7:	dmaInterruptLevel = ILR2_DMA_LEVEL7; break;
    default:	dmaInterruptLevel = ILR2_DMA_LEVEL5; break;
    }
}


/*******************************************************************************
* dmaSn
*	This routine set the global var used in the various dma routines.
*  It also updates the value in the control register so it can be seen
*  by dmaShow.
*/

int	dmaSnoop = DMACR2_SINK_DATA;

void dmaSn (int snoop_mode)	/* ref p2-53 in programmer's */
{
	int	temp;	/* needed to force lword access */

    snoop_mode &= 0x3;
    
    temp = *VMECHIP2_DMACR2;
    temp &= ~(DMACR2_SINK_DATA|DMACR2_INVALIDATE);	/* clear */
    dmaSnoop = 0;

    switch(snoop_mode)
    {
    case 0:
    case 3:	/* nothing to do */
	break;
    case 1:	dmaSnoop = DMACR2_SINK_DATA;
		temp |= dmaSnoop;
	break;
    case 2:	dmaSnoop = DMACR2_INVALIDATE;
		temp |= dmaSnoop;
	break;
    default:
	break;
    }

    *VMECHIP2_DMACR2 = temp;
}


/*******************************************************************************
* dmaChainTblSn
*
*	This routine controls the snoop mode used when the dmac in reading
* command table (i.e. commands packets) in chain mode.
*/

void dmaChainTblSn (int snoop_mode)		/* ref p2-53 in programmer's */
{
	int	temp;	/* needed to force lword access */

    snoop_mode &= 0x3;
    
    temp = *VMECHIP2_DMACR1;
    temp &= ~(DMACR1_SINK_DATA|DMACR1_INVALIDATE);	/* clear */

    switch(snoop_mode)
    {
    case 0:
    case 3:	/* nothing to do */
	break;
    case 1:	temp |= DMACR1_SINK_DATA;
	break;
    case 2:	temp |= DMACR1_INVALIDATE;
	break;
    default:
	break;
    }

    *VMECHIP2_DMACR1 = temp;
}


/*******************************************************************************
* dmaChainTblInt
*
*     When this routine is passed a zero value, interrupting when chain
* table entries are processed is disabled.
*/

int	dmaChainTableInt = 0;

void dmaChainTblInt( int int_flag )	/* ref p2-52,53 in programmer's */
{
	int	temp;	/* needed to force lword access */

    
    temp = *VMECHIP2_DMACR2;

    if (int_flag)
    {   dmaChainTableInt = DMACR2_INTE;
	temp |=  DMACR2_INTE;
    }
    else
    {   dmaChainTableInt = 0;
	temp &= ~DMACR2_INTE;
    }

    *VMECHIP2_DMACR2 = temp;
}


#ifdef mv167
/*******************************************************************************
* dmaStart
*/

STATUS dmaStart (caddr_t      vmeadd,
		 caddr_t      localadd,
		 unsigned int bytecnt,
		 int          dir,
		 int          dtb_mode,
		 int          non_blk_am)
{
    int	temp;	/* needed to force lword access */


    /* error if local address greater than physically configured mem size */
    if((int)localadd > (0x400000<<(*MEMC_MCR & 0x7)) ) return(ERROR);

    *VMECHIP2_DMACLBAC = (int)localadd;
    *VMECHIP2_DMACVAC = (int)vmeadd;
    *VMECHIP2_DMACBC = bytecnt;

    /* now biuld the control register 2 from the ground up */
    temp = dmaChainTableInt | dmaSnoop | (DMACR2_VINC|DMACR2_LINC);

    switch(dtb_mode)
    {
    case D16:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D16));
	break;
    case D32:
	temp |= (non_blk_am&0x3f);
	break;
    case D32BLK:
	temp |= (non_blk_am&0x3f);
	temp |= DMACR2_D32_BLOCK;
	break;
    case D64BLK:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D64_BLOCK));
	break;
    default:
	return(ERROR);
    }

    if (dir&TOVME)         temp |=  DMACR2_TO_VME;
    if (dir&NO_ADDR_INC_LB) temp &= ~DMACR2_LINC;
    if (dir&NO_ADDR_INC_VME) temp &= ~DMACR2_VINC;

    *VMECHIP2_DMACR2 = temp;

    temp = *VMECHIP2_DMACR1;
    temp &= ~DMACR1_DTBL;	/* make sure were not in chaining mode */
    temp |= DMACR1_DEN;		/* start DMA bit */
    *VMECHIP2_DMACR1 = temp;

    return(OK);
}
#endif

/*******************************************************************************
* dmaStatus
*/

int dmaStatus (void)
{
	int	temp;	/* needed to force lword access */

    temp = *VMECHIP2_ICR;
    temp &= 0x7f;		/* just get DMAC Status Register */

    return(temp);
}


/*******************************************************************************
* dmaVmeBR
*/

void dmaVmeBR (int level)
{
	int	temp;	/* needed to force lword access */


    temp = *VMECHIP2_DMACR1;
    temp &= ~DMACR1_DREQ_L3;	/* clear it first */
    switch(level)
    {
    case 0:	temp |= DMACR1_DREQ_L0; break;
    case 1:	temp |= DMACR1_DREQ_L1; break;
    case 2:	temp |= DMACR1_DREQ_L2; break;
    case 3:	temp |= DMACR1_DREQ_L3; break;
    default:	temp |= DMACR1_DREQ_L0; break;
    }
    *VMECHIP2_DMACR1 = temp;
}


/*******************************************************************************
* dmaStartChain
*/

void dmaStartChain (struct dma_chain	*chain_addr)
{
	int	temp;	/* needed to force lword access */


    *VMECHIP2_TAC = (int)chain_addr;
    temp = *VMECHIP2_DMACR1;
    temp |= (DMACR1_DTBL | DMACR1_DEN); /* set start and chaining mode */
    *VMECHIP2_DMACR1 = temp;
}


/*******************************************************************************
* dmaChainAlloc
*	I'm wondering if there is anyway I can store the allocation size?
* The chain is not initialization in any way and dmaChainHead should probably
* be called next.
*/

struct dma_chain *dmaChainAlloc( int links )
{
	struct dma_chain	*ch_p;

    ch_p = (struct dma_chain *)malloc( links * sizeof(struct dma_chain) );
    return (ch_p);
}


/*******************************************************************************
* dmaChainHead
*
*	These routines assume a pointer to an array of dma_chain structures
* in passed to them.
*/

int			 dmaDefaultMaxChainLinks = 20;
struct dma_chain	*dmaDefaultChainPtr = 0;


struct dma_chain *dmaChainHead(  struct dma_chain	chain_a[]
			       , caddr_t		vmeadd
			       , caddr_t		localadd
			       , unsigned int		bytecnt
			       , int			dir
			       , int			dtb_mode
			       , int			non_blk_am
			     )
{
	int	temp;	/* needed to force lword access */

    if (chain_a == 0)
    {   if (dmaDefaultChainPtr == 0)
	{    dmaDefaultChainPtr
		 = (struct dma_chain *)malloc( dmaDefaultMaxChainLinks
					      * sizeof(struct dma_chain) );
	}
	chain_a = dmaDefaultChainPtr;
    }


    chain_a[0].Local_A = localadd;
    chain_a[0].VME_A   = vmeadd;
    chain_a[0].Nbytes  = bytecnt;

    /* now biuld the control register 2 from the ground up */
    temp = dmaChainTableInt | dmaSnoop | (DMACR2_VINC|DMACR2_LINC);

    switch(dtb_mode)
    {
    case D16:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D16));
	break;
    case D32:
	temp |= (non_blk_am&0x3f);
	break;
    case D32BLK:
	temp |= (non_blk_am&0x3f);
	temp |= DMACR2_D32_BLOCK;
	break;
    case D64BLK:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D64_BLOCK));
	break;
    default:
	return ((struct dma_chain *)-1);
    }

    if (dir&TOVME)         temp |=  DMACR2_TO_VME;
    if (dir&NO_ADDR_INC_LB) temp &= ~DMACR2_LINC;
    if (dir&NO_ADDR_INC_VME) temp &= ~DMACR2_VINC;


    chain_a[0].Cont = temp;

    chain_a[0].Next = DMACHAIN_END;

    return (chain_a);
}


/*******************************************************************************
* dmaChainAdd -
*
*	Returns the number of links in the chain
*  unless using dmaDefaultChainPtr in which case it may return -1 if
*  trying to add more than dmaDefaultMaxChainLinks
*/

int dmaChainAdd(  struct dma_chain	chain_a[]
		, caddr_t		vmeadd
		, caddr_t		localadd
		, unsigned int		bytecnt
		, int			dir
		, int			dtb_mode
		, int			non_blk_am
	       )
{
	int	temp;	/* needed to force lword access */
	int	links = 0;

    while (chain_a[links++].Next != DMACHAIN_END) ;

    if (   (chain_a == dmaDefaultChainPtr)
	&& (links >= dmaDefaultMaxChainLinks) )  return (-1);

    chain_a[links - 1].Next = &(chain_a[links]);

    chain_a[links].Local_A = localadd;
    chain_a[links].VME_A   = vmeadd;
    chain_a[links].Nbytes  = bytecnt;

    /* now biuld the control register 2 from the ground up */
    temp = dmaChainTableInt | dmaSnoop | (DMACR2_VINC|DMACR2_LINC);

    switch(dtb_mode)
    {
    case D16:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D16));
	break;
    case D32:
	temp |= (non_blk_am&0x3f);
	break;
    case D32BLK:
	temp |= (non_blk_am&0x3f);
	temp |= DMACR2_D32_BLOCK;
	break;
    case D64BLK:
	temp |= ((non_blk_am&0x3f) | (DMACR2_D64_BLOCK));
	break;
    default:
	return(ERROR);
    }

    if (dir&TOVME)         temp |=  DMACR2_TO_VME;
    if (dir&NO_ADDR_INC_LB) temp &= ~DMACR2_LINC;
    if (dir&NO_ADDR_INC_VME) temp &= ~DMACR2_VINC;

    chain_a[links].Cont = temp;

    chain_a[links].Next = DMACHAIN_END;

    return (++links);
}



/*******************************************************************************
* dmaDone
*/

int dmaDone (void)
{
    return(*VMECHIP2_ICR 
	& (ICR_DONE|ICR_VME|ICR_TBL|ICR_DLTO|ICR_DLOB|ICR_DLPE|ICR_DLBE));

    /* 0 = not done
       1 = done and OK  (DMA_DONE_OK)
       any other =      (DMA_DONE_ERROR)
    */
}




/*******************************************************************************
* dmaThrottle
*/

void dmaThrottle (int release_mode,	/* ref p 2-51,60 programmer's */
		  int time_on,
		  int time_off)
{
	int	temp;	/* needed to force lword access */


    temp = *VMECHIP2_DMACR1;
    temp &= ~(DMACR1_DFAIR | DMACR1_TIMER_OR_BRX);	/* clear first */
/*DMACR1_DFAIR DMACR1_TIMER_BRX DMACR1_TIMER DMACR1_BRX DMACR1_TIMER_OR_BRX
*/
    release_mode &= 7;
    release_mode <<= 2;
    temp |= release_mode;
    *VMECHIP2_DMACR1 = temp;

    temp = *VMECHIP2_TIMEOUTCR;
    temp &= ~(TIMEOUTCR_OFF_1024US | TIMEOUTCR_ON_DONE);  /* clear first */
/*TIMEOUTCR_ON_{16,32,64,128,256,512,1024}US TIMEOUTCR_ON_DONE
  TIMEOUTCR_OFF_{0,16,32,64,128,256,512,1024}US
*/
    time_on &= 7;
    time_off &=7;
    time_on <<= 18;
    time_off <<= 21;
    temp |= (time_on | time_off);
    *VMECHIP2_TIMEOUTCR = temp;
}



/*******************************************************************************
* cacheControlShow
*/

void cacheControlShow (void)
{
	int	x;

    x = 0;
    asm("movec cacr,d0; movel d0,%0" : "=g" (x) );
    printf("Data Cache: %sABLED; Instruction Cache %sABLED\n",
	   (x & 0x80000000) ? "EN":"DIS", (x & 0x8000) ? "EN":"DIS");
}

/*******************************************************************************
* vmeIackedConnect
*/

LOCAL	VOIDFUNCPTR	vmeIackedRoutine	= NULL;
LOCAL	int		vmeIackedArg		= NULL;
LOCAL	BOOL		vmeIackedConnected	= FALSE;


STATUS	vmeIackedConnect(  VOIDFUNCPTR	routine
			 , int		arg )	/* argument with which to call routine           */
{
	void	vmeIackedInt();	/* routine follows */

    if (   vmeIackedConnected
	|| intConnect( INUM_TO_IVEC((UTIL_INT_VEC_BASE0+LBIV_VIA))
		      ,vmeIackedInt,NULL)==ERROR )
    {
	return (ERROR);
    }
    
    vmeIackedConnected = TRUE;

    vmeIackedRoutine   = routine;
    vmeIackedArg       = arg;

    return (OK);
}


/*
* vmeIackedInt - handle auxiliary clock interrupts
*/

void vmeIackedInt (void)
{
	int	temp;	/* needed to force lword access */

    if (vmeIackedRoutine != NULL) (*vmeIackedRoutine)(vmeIackedArg);

    /* reset/clear interrupt */
    temp=~ICLR_CVIA;
    *VMECHIP2_ICLR |= ~temp;
}



/*******************************************************************************
* vmeIackedDisable
*/

LOCAL BOOL vmeIackedRunning         = FALSE;


void vmeIackedDisable (void)
{
	int	temp;	/* needed to force lword access */

    if (vmeIackedRunning)
    {
        /* disable interrupts */
	temp =  ~LBIER_EVIA;
        *VMECHIP2_LBIER &= temp;

        vmeIackedRunning = FALSE;
    }
}


/********************************************************************************
* vmeIackedEnable - turn on system clock interrupts
*
* This routine enables system clock interrupts.
*
* RETURNS: N/A
*
* SEE ALSO: vmeIackedConnect(), vmeIackedDisable()
*/

LOCAL int vmeIackedInterruptLevel = ILR2_VIA_LEVEL5;


void vmeIackedEnable (void)
{
	int	temp;	/* needed to force lword access */


    if (!vmeIackedRunning)
    {

	/* enable interrupt, and set IRQ level */

	*VMECHIP2_ILR2 &= ~ILR2_VIA_LEVEL7;	/* clear bits before ORing */
	*VMECHIP2_ILR2 |= vmeIackedInterruptLevel;

#ifdef NOTDEF
	/* kind of a bug - clear interrupt before enabe -	*/
	/*	incase there's one pending.			*/
	temp=~ICLR_CVIA;
	*VMECHIP2_ICLR |= ~temp;
#endif

	temp = ~LBIER_EVIA;			/* enable the interrupt */
	*VMECHIP2_LBIER |= ~temp;

	vmeIackedRunning = TRUE;
    }
}


/*******************************************************************************
* vmeIackedLevelSet - set interupt level, default is level5.  Level will take
*		 effect at time of next clkxEnable call.
*/

void vmeIackedLevelSet(int level)
{
    switch(level)
    {
    case 0:	vmeIackedInterruptLevel = 0;			break;
    case 1:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL1; break;
    case 2:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL2; break;
    case 3:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL3; break;
    case 4:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL4; break;
    case 5:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL5; break;
    case 6:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL6; break;
    case 7:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL7; break;
    default:	vmeIackedInterruptLevel = ILR2_VIA_LEVEL5; break;
    }
}


/*******************************************************************************
* vmeIntLevelSet - set interupt level, default level is that of the irq.
*		 This routine augments the sysInt{En,Dis}able routines provided
*		 by Wind River.
*		 cpu_level can be set to 0 for polling mode - ref 2-69.
*/

static int vmeIntLevels[7][8] = { {0,ILR4_VIRQ1_LEVEL1,ILR4_VIRQ1_LEVEL2,ILR4_VIRQ1_LEVEL3,ILR4_VIRQ1_LEVEL4,ILR4_VIRQ1_LEVEL5,ILR4_VIRQ1_LEVEL6,ILR4_VIRQ1_LEVEL7}
				 ,{0,ILR4_VIRQ2_LEVEL1,ILR4_VIRQ2_LEVEL2,ILR4_VIRQ2_LEVEL3,ILR4_VIRQ2_LEVEL4,ILR4_VIRQ2_LEVEL5,ILR4_VIRQ2_LEVEL6,ILR4_VIRQ2_LEVEL7}
				 ,{0,ILR4_VIRQ3_LEVEL1,ILR4_VIRQ3_LEVEL2,ILR4_VIRQ3_LEVEL3,ILR4_VIRQ3_LEVEL4,ILR4_VIRQ3_LEVEL5,ILR4_VIRQ3_LEVEL6,ILR4_VIRQ3_LEVEL7}
				 ,{0,ILR4_VIRQ4_LEVEL1,ILR4_VIRQ4_LEVEL2,ILR4_VIRQ4_LEVEL3,ILR4_VIRQ4_LEVEL4,ILR4_VIRQ4_LEVEL5,ILR4_VIRQ4_LEVEL6,ILR4_VIRQ4_LEVEL7}
				 ,{0,ILR4_VIRQ5_LEVEL1,ILR4_VIRQ5_LEVEL2,ILR4_VIRQ5_LEVEL3,ILR4_VIRQ5_LEVEL4,ILR4_VIRQ5_LEVEL5,ILR4_VIRQ5_LEVEL6,ILR4_VIRQ5_LEVEL7}
				 ,{0,ILR4_VIRQ6_LEVEL1,ILR4_VIRQ6_LEVEL2,ILR4_VIRQ6_LEVEL3,ILR4_VIRQ6_LEVEL4,ILR4_VIRQ6_LEVEL5,ILR4_VIRQ6_LEVEL6,ILR4_VIRQ6_LEVEL7}
				 ,{0,ILR4_VIRQ7_LEVEL1,ILR4_VIRQ7_LEVEL2,ILR4_VIRQ7_LEVEL3,ILR4_VIRQ7_LEVEL4,ILR4_VIRQ7_LEVEL5,ILR4_VIRQ7_LEVEL6,ILR4_VIRQ7_LEVEL7}
				};

int vmeIntLevelSet(int irq, int cpu_level)
{
	int	temp;	/* needed to force lword access */


    if ( (irq == 0) || (irq > 7) || (cpu_level > 7) ) return (-1);

    temp = *VMECHIP2_ILR4;

    temp &= ~vmeIntLevels[irq-1][7];

    temp |= vmeIntLevels[irq-1][cpu_level];

    *VMECHIP2_ILR4 = temp;
    return (1);
}


/*******************************************************************************
* Time of Day (TOD) support routines.
* These routines interface to the Time of Day (TOD) MK48T08 clock on 
* the mvme167.
*/

/* Control field bits. */

#define tod_ST (1<<7)	/* Stop clock bit. */
#define tod_W (1<<6)		/* Enable write bit. */
#define tod_R (1<<5)		/* Enable read bit. */

/* TOD hardware memory map. */

typedef struct
  {
  unsigned char control;	/* Control Field. */
  unsigned char sec;	        /* Seconds. */
  unsigned char mmin;	        /* Minutes. */
  unsigned char hour;		/* Hours. */
  unsigned char day;		/* Day of the week. */
  unsigned char ddate;		/* Day of the month. */
  unsigned char month;		/* Month */
  unsigned char year;		/* Year. */
  } tod_struct;

char *todDayOfWeek[] = {"Unused", "Sunday", "Monday", "Tuesday", "Wednesday",
		        "Thursday", "Friday", "Saturday"};

char *todMonth[] = {"Unused", "January","February","March","April","May","June",
		    "July","August", "September", "October", "November", 
	            "December"};

/* These macros do the conversions between tod internal format and
   decimal.  The work for all numbers 0..99 (10). */

#define todHexToDec(n) (n - n/0x10*6)
#define todDecToHex(n) (n + n/10*6)

/* Returns as a string, the current date and time. */

void  tod_DateStr (char *s) /* OUT: current date and time.  Must be large enough. */
  {
  tod_struct *datePtr = (tod_struct *) TOD_CLOCK;

  sprintf(s,"%02x:%02x:%02x %s, %02x-%s-%02x\n", 
         datePtr->hour, datePtr->mmin, datePtr->sec, todDayOfWeek[datePtr->day],
         datePtr->ddate, todMonth[todHexToDec(datePtr->month)], datePtr->year);

  }


/* tod_DateGet Returns the current date and time as integers.  The TOD clock
   represents numbers in hex such that they appear as decimal in
   a hex dump.  However, we will convert these to real decimal
   numbers and return them. */

void tod_DateGet(int *hour, int *mmin,  int *sec, 
		 int *year, int *month, int *ddate, int *day)
  {
  tod_struct *datePtr = (tod_struct *) TOD_CLOCK;

  *year = todHexToDec(datePtr->year);
  *month = todHexToDec(datePtr->month);
  *ddate = todHexToDec(datePtr->ddate);
  *day = todHexToDec(datePtr->day);
  *hour = todHexToDec(datePtr->hour);
  *mmin = todHexToDec(datePtr->mmin);
  *sec = todHexToDec(datePtr->sec);
  }

/* Prints out the current date and time to stdout. */

void Date (void)
  {
  char s[40];
  
  tod_DateStr(s);
  printf(s);
  }

/* Sets the current date and time. */

void tod_DateSet(int hour, int mmin,  int sec, 
		 int year, int month, int ddate, int day)
  {
  tod_struct *datePtr = (tod_struct *) TOD_CLOCK;

  datePtr->control = tod_ST | tod_W;	/* Enable Date Change. */
  datePtr->year = todDecToHex(year);
  datePtr->month = todDecToHex(month);
  datePtr->ddate = todDecToHex(ddate);
  datePtr->day = todDecToHex(day);
  datePtr->hour = todDecToHex(hour);
  datePtr->mmin = todDecToHex(mmin);
  datePtr->sec = todDecToHex(sec);
  datePtr->control = tod_R;		/* Disable date change. */
  }

/* Prompts the user for the current date and time. */

void dateSet (void)
  {

  int hour, mmin, sec, year, month, ddate, day;

  /* We have to scan these in hex since that is how the TOD clock
     represents them. */
  
  printf("Enter HH MM SS: ");
  scanf("%d %d %d", &hour, &mmin, &sec);
  printf("Enter YY MM DD: ");
  scanf("%d %d %d", &year, &month, &ddate);
  printf("Enter day of week number (1=Sunday, 7=Saturday): ");
  scanf("%d", &day);
  tod_DateSet(hour, mmin, sec, year, month, ddate, day);
  }


/*******************************************************************************
* addressTranslation routines
*
*/


#define TRANS_DEBUG 0		/* 1 to enable */


STATUS addressTranslationShow ( unsigned int start_addr,
				unsigned int end_addr,
				unsigned int show_mapping_mem)
{
	unsigned int	pag_siz_8k;
	unsigned int	tc;
	unsigned int	*srp;
	unsigned int	*flp;		/* first level pointer */
	unsigned int	*slp;		/* second level pointer */
	unsigned int	*indirectp;
	unsigned int	z1, z2, z3, zi;
	unsigned int	l1,    l2,    l3;
	unsigned int	l1end, l2end, l3end;
	unsigned int	logical_addr, physical_addr;
	unsigned int	cm,wp;
	unsigned int	lines=0;
	char		line_buf[80];
	unsigned int	itt0,itt1;
	unsigned int	dtt0,dtt1, dtt0_table_search;
	unsigned int	cacr;
	unsigned int	smt = sysMemTop();
	unsigned int	skipped = 0;


    /* GET Translation Control register */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );
    pag_siz_8k = tc&0x4000;

    /* GET Supervisor Root Pointer - CHECK BELOW */
    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    srp = 0;
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );

    /* GET tts */
    /*asm("movec itt0,d0; movel d0,%0" : "=g" (itt0) );*/
    itt0=0; itt1=0; dtt0=0; dtt1=0;
    asm(".word 0x4e7a, 0x0004; movel d0,%0" : "=g" (itt0) );
    asm(".word 0x4e7a, 0x0005; movel d0,%0" : "=g" (itt1) );
    asm(".word 0x4e7a, 0x0006; movel d0,%0" : "=g" (dtt0) );
    asm(".word 0x4e7a, 0x0007; movel d0,%0" : "=g" (dtt1) );

    dtt0_table_search = (0x00ffe040 | (dtt0&0x00000300));

    if (show_mapping_mem != 2)
    {
	cacr = 0;
	asm("movec cacr,d0; movel d0,%0" : "=g" (cacr) );
	printf(  "\n\
From CACR (master control) ==> Data Cache: %sABLED; Instruction Cache %sABLED\n"
	   , (cacr & 0x80000000) ? "EN":"DIS", (cacr & 0x8000) ? "EN":"DIS");

    printf("NOTE: The default cache mode (with cache enabled above) if both\n");
    printf("      transparent and paged translations disabled is writethrough.\n");
    printf("NOTE: When both transparent and paged translations are disabled,\n");
    printf("      for either instruction or data, the behavior is that of\n");
    printf("      transparent translation (with cache controlled from cacr).\n");

#   if TRANS_DEBUG == 1
	printf("itt0=%x,itt1=%x,dtt0=%x,dtt1=%x\n");
#   endif

	printf("\
\n\
*TRANSPARENT* translations:\n\
                        itt0        itt1        dtt0        dtt1\n\
enable              %8x    %8x    %8x    %8x\n\
base              0x%08x  0x%08x  0x%08x  0x%08x\n\
mask (don't care) 0x%08x  0x%08x  0x%08x  0x%08x\n\
cache mode*         %8x    %8x    %8x    %8x\n\
write prot          %8x    %8x    %8x    %8x\n\
total reg.**      0x%08x  0x%08x  0x%08x  0x%08x\n\
*  0=writethrough 1=copyback 2=inhibited,serialized 3=inhibited,nonserialized\n\
** p.6-38 '040 USER'S MAN.\n\
\n"
	,(itt0&0x8000)?1:0,(itt1&0x8000)?1:0,(dtt0&0x8000)?1:0,(dtt1&0x8000)?1:0
	,itt0&0xff000000,itt1&0xff000000,dtt0&0xff000000,dtt1&0xff000000
	, ((itt0&0x00ff0000)<<8)|0x00ffffff,((itt1&0x00ff0000)<<8)|0x00ffffff
	 ,((dtt0&0x00ff0000)<<8)|0x00ffffff,((dtt1&0x00ff0000)<<8)|0x00ffffff
	,(itt0&0x60)>>5,(itt1&0x60)>>5,(dtt0&0x60)>>5,(dtt1&0x60)>>5
	,(itt0&0x4)>>2,(itt1&0x4)>>2,(dtt0&0x4)>>2,(dtt1&0x4)>>2
	,itt0,itt1,dtt0,dtt1  );

	printf(   "*PAGED* Address Translations are %sABLED\n"
		, tc&0x8000? "EN":"DIS"  );
	printf("Supervisor Root Pointer: 0x%08x\n",srp);
    }

    if ( (tc&0x8000) || end_addr )
    {
	if (show_mapping_mem != 2)
	{   if (srp==0) return(-1); /* CHECK SRP! */
	    printf(  "Paged Address Translation page size is %cK\n"
        	   , pag_siz_8k? '8':'4');

	    if ( isatty(STD_OUT) )
	    {
printf("\nPage Table follows, press q to quit, any other key to continue...");
		if (getKeyPress() == 'q') return(1);
		printf("\n");
	    }
	}

	l1 = (start_addr&0xfe000000)>>25;	/* bits 31-25 (7 bits) */
	l2 = (start_addr&0x01fc0000)>>18;	/* bits 24-18 (7 bits) */
	l3 = (start_addr&0x0003f000)>>(pag_siz_8k?13:12);   /* 5 or 6 bits */
	l1end = l2end = 127;		/* default ends */
	l3end = (pag_siz_8k?31:63);
	if ( end_addr && (end_addr>=start_addr) )
	{   l1end = (end_addr&0xfe000000)>>25;
	    l2end = (end_addr&0x01fc0000)>>18;
	    l3end = (end_addr&0x0003f000)>>(pag_siz_8k?13:12);
	}

        for(;l1<=l1end;l1++)
	{
	    z1 = addressTranslationReadPhys(srp+l1);
	    if ( ((z1&3)<2) || (z1>=smt)  )
	    {   if (!skipped)
		{   printf("skipping...\n");
		    skipped++;
		    lines++;
		}
		continue;  /* entry invalid */
	    }
	    flp = (unsigned int *)( z1&0xfffffe00 ); /*mask out udt,etc */
	    for(;l2<=l2end;l2++)
	    {
		z2 = addressTranslationReadPhys(flp+l2);
		if ( ((z2&3)<2) || (z2>=smt) )
		{   if (!skipped)
		    {   printf("skipping...\n");
			skipped++;
			lines++;
		    }
                    continue; /*entry invalid*/
		}
		slp = (unsigned int *)(z2&(pag_siz_8k?0xffffff80:0xffffff00) );
		for(;l3<=l3end;l3++)
		{
		    z3 = addressTranslationReadPhys(slp+l3);
		    if (       (    ( (z3)&3 )==0       ) 
			||     (    ( (z3)&3 )==3       )   ) 
		    {   if (!skipped)
			{   printf("skipping...\n");
                            skipped++;
			    lines++;
			}
			continue;
		    }
		    if (    ((z3)&3)==2    ) /*indirect page descriptor*/
		    {
			if (z3>=smt)
			{   if (!skipped)
			    {   printf("skipping...\n");
				skipped++;
				lines++;
			    }
			    continue;
			}
			indirectp = (unsigned int *)((z3)&0xfffffffc);
			zi = addressTranslationReadPhys(indirectp);
			if( (zi&3)!=1    )
			{   if (!skipped)
			    {   printf("skipping...\n");
				skipped++;
				lines++;
			    }
			    continue;
			}
	physical_addr = zi & (pag_siz_8k?0xffffe000:0xfffff000);
	printf("l1idx:%2x;l2idx:%2x;page:%2x;indirect:%x-- "
			    ,l1,l2,l3,indirectp );
			cm=(zi>>5)&3;
			wp=(zi>>2)&1;
		    }
		    else
		    {
	physical_addr = (z3) & (pag_siz_8k?0xffffe000:0xfffff000);
	printf("l1idx:%2x;l2idx:%2x;page:%2x-- "
			    ,l1,l2,l3);
			cm=((z3)>>5)&3;
			wp=((z3)>>2)&1;
		    }

	logical_addr = (l1<<25) | (l2<<18) | (l3<<(pag_siz_8k?13:12));
	printf("log.addr: %08x = phys.addr: %08x; cm=%u; wp=%u\n"
        	,logical_addr,physical_addr,cm,wp);
		    skipped = 0;
		    if (show_mapping_mem == 1)
		    {   printf(" ");
			addressTranslationShow(   (unsigned)(slp+l3)
						, (unsigned)(slp+l3), 2  );
			++lines;
		    }
		    if ( isatty(STD_OUT) && (++lines>=16) )
		    {
		printf("press q to quit, any other key to continue...");
			if (getKeyPress() == 'q') return(1);
			lines=0;
		    }
		}
		l3 = 0;
	    }
	    l2 = 0;
	}
    }
    if (show_mapping_mem != 2) printf("\n");
    return(OK);
}



unsigned *addressTranslationLogToPhys( unsigned 	*logical)
{
	unsigned int	logical_addr, physical_addr;
	unsigned int	page_offset;
	unsigned int	pag_siz_8k;
	unsigned int	tc;
	unsigned int	*srp;
	unsigned int	*flp;		/* first level pointer */
	unsigned int	*slp;		/* second level pointer */
	unsigned int	*indirectp;
	unsigned int	z1, z2, z3, zi;
	unsigned int	l1,    l2,    l3;
	unsigned int	l1end, l2end, l3end;
	unsigned int	dtt0, dtt1, dtt0_table_search;
	unsigned int	dttbase, dttmask;
	unsigned int	smt = sysMemTop();


    /* GET Translation Control register */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    /* GET dtts */
    /*asm("movec itt0,d0; movel d0,%0" : "=g" (itt0) );*/
    dtt0=0;
    asm(".word 0x4e7a, 0x0006; movel d0,%0" : "=g" (dtt0) );
    asm(".word 0x4e7a, 0x0007; movel d0,%0" : "=g" (dtt1) );


    if (   ((tc&0x8000)==0)
	&& ((dtt0&0x8000)==0)
	&& ((dtt1&0x8000)==0) ) return ( logical );

    dttbase = (dtt0&0xff000000);
    dttmask = (dtt0&0x00ff0000);
    dttmask = (dttmask<<8) | 0x00ffffff; /* assume contiguous blocks! */
    dttmask |= dttbase;			 /* make upper limit */
    if (   (dtt0&0x8000)
	&& (logical>=dttbase)
	&& (logical<=dttmask) ) return ( logical );

    dttbase = (dtt1&0xff000000);
    dttmask = (dtt1&0x00ff0000);
    dttmask = (dttmask<<8) | 0x00ffffff; /* assume contiguous blocks! */
    dttmask |= dttbase;			 /* make upper limit */
    if (   (dtt1&0x8000)
	&& (logical>=dttbase)
	&& (logical<=dttmask) ) return ( logical );

    /* at this point, if *paged* translations are not enabled, it's an error */
    if ( (tc&0x8000)==0 )       return ( (void *)0xffffffff );


    pag_siz_8k = tc&0x4000;
    dtt0_table_search = (0x00ffe040 | (dtt0&0x00000300));

    /* GET Supervisor Root Pointer - CHECK BELOW */
    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    srp = 0;
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    

    l1 = ((unsigned)logical&0xfe000000)>>25;	/* bits 31-25 (7 bits) */
    l2 = ((unsigned)logical&0x01fc0000)>>18;	/* bits 24-18 (7 bits) */
    l3 = ((unsigned)logical&0x0003f000)>>(pag_siz_8k?13:12);   /* 5 or 6 bits */
    page_offset = ((unsigned)logical&(pag_siz_8k?0x1fff:0xfff));

    z1 = addressTranslationReadPhys(srp+l1);
    if ( ((z1&3)<2) || (z1>=smt) ) return ( (void *)0xffffffff );
    flp = (unsigned int *)( z1&0xfffffe00 ); /*mask out udt,etc */
    z2 = addressTranslationReadPhys(flp+l2);
    if ( ((z2&3)<2) || (z2>=smt) ) return ( (void *)0xffffffff );
    slp = (unsigned int *)(z2&(pag_siz_8k?0xffffff80:0xffffff00) );
    z3 = addressTranslationReadPhys(slp+l3);
    if (       (    ( (z3)&3 )==0       ) 
	||     (    ( (z3)&3 )==3       )   ) return ( (void *)0xffffffff );
    if (    ((z3)&3)==2    ) /*indirect page descriptor*/
    {
	if (z3>=smt) return ( (void *)0xffffffff );
	indirectp = (unsigned int *)((z3)&0xfffffffc);
	zi = addressTranslationReadPhys(indirectp);
	if( (zi&3)!=1    ) return ( (void *)0xffffffff );
	physical_addr = zi & (pag_siz_8k?0xffffe000:0xfffff000);
    }
    else
    {   physical_addr = (z3) & (pag_siz_8k?0xffffe000:0xfffff000);
    }
    return ( (void *)(physical_addr | page_offset) );
}



	unsigned int	RootTable[128*2];   /* needs to be 128 lword aligned! */

int addressTranslationRoot (unsigned int pagesize)
{
extern	unsigned int	RootTable[];
	unsigned int	*srp;
	unsigned int	i;
	unsigned int	tc;


    /* GET Translation Control register */	/* to preserve page size */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    tc &= 0xbfff;
    tc |= ((pagesize&1)<<14);

    /* WRITE TO tc */
    /*asm("movel %0,d0; movec d0,tc; " : "=g" (tc) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0003" : "=g" (tc) );


    /* WRITE TO srp */
    srp = (unsigned int *)((unsigned int)RootTable & 0xfffffe00);  /*make srp 128 lword aligned*/
    srp += 128;
    /*asm("movel %0,a0; movec d0,srp; " : "=g" (srp) );*/
    asm("movel %0,a0; .word 0x4e7b, 0x8807" : "=g" (srp) );
#   if TRANS_DEBUG == 1
    printf("RootTable=%x;srp=%x\n",RootTable,srp);
    taskDelay(100);
#   endif

    for(i=0;i<128;i++)
    {
#	if TRANS_DEBUG == 1
	if ( (srp+i) > &RootTable[(128*2)-1]  )
	{
	    printf("initialization overflow - should've used calloc\n");
	    break;
	}
#	endif
	*(srp+i)=0;
    }

    /* GET srp */

    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    return((int)srp);
}



int ATCalloc (unsigned int elem_siz)	/* for AddressTranslationCache alloc */
{
	int	stat, i;

    stat = (int)memalign(elem_siz,elem_siz);
    if (stat == 0)
    {   printf("ATCalloc FAILED\n");
    }
    else
    {   for (i=(elem_siz/4); i--;)  *((int *)stat+i)=0;
    }
    return(stat);
}


#ifdef mv167
/**********************************************************************
* Currently this routine my not work in the environment where logical
* addresses are not equal to physical.  The routine should be modified to
* translate any logical address space allocated for the table and also
* make sure any write are made to physical addresses.
*/

int addressTranslation (unsigned int log_start,
			unsigned int log_end,
			unsigned int phys_start,
			unsigned int cache_mode,
			unsigned int write_prot,
			unsigned int expert)
{
	unsigned int 	pag_siz_8k;
	unsigned int	num_pages;
	unsigned int	page_siz;
	unsigned int	mem_siz;
        unsigned int     tc;
	unsigned int	*srp;
	unsigned int	*flp;		/* first level pointer */
	unsigned int	*slp;		/* second level pointer */
	unsigned int	l1indx,l2indx,l3indx;


    /* GET srp - check it! */

    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    srp = 0;
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    if (srp==0) return(-1);

    if (log_end < log_start ) return(-1);

    /* get page size and then number of pages */

    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );
    pag_siz_8k = (tc&0x4000);
    num_pages =  (log_end  >>(pag_siz_8k?13:12))
               - (log_start>>(pag_siz_8k?13:12));
    num_pages += 1;
#   if TRANS_DEBUG == 1
    printf("num_pages=0x%x\n",num_pages);
#   endif
    page_siz = 1<<(pag_siz_8k?13:12);

    phys_start &= (pag_siz_8k?0xffffe000:0xfffff000);

    /* return if log mem siz is greater than physical mem size */
    /* ref h/drv/memc040.h (vxworks) or 167 programmer's ref p.4-7*/
    mem_siz = (0x00400000<<(*MEMC_MCR & 0x7));
    if ( (num_pages * page_siz) > mem_siz ) return(-1);
/*  i.e. VME mapping
    if ( (phys_start+(num_pages * page_siz)) >= mem_siz ) return(-1);
*/
    /*-------------------------------------------------------------------*/

    /* 3 level looping - to handle cross index mapping */

    /* get indexes - ref. p6-6 of 040 32 bit micro. user's manual */
    l1indx = log_start>>25;
    l2indx = (log_start&0x01fc0000)>>18;
    l3indx = (log_start&(pag_siz_8k?0x0003e000:0x0003f000))>>(pag_siz_8k?13:12);

    for ( ; l1indx<128; l1indx++)
    {
	/* check level 2 index */
	if ( ((*(srp+l1indx))&2)==0 )	/* check 2nd bit (upper bit of UDT) */
	{				/* for invalid */
	    (*(srp+l1indx))=(ATCalloc(128*4) | 3); /* or in udt*/
	    if ( *(srp+l1indx) == 3 ) return(-1);   /* ERROR */
#	    if TRANS_DEBUG == 1
	    printf("ATCalloc(128*4) returned %x\n",(*(srp+l1indx))&0xfffffffc);
#	    endif
	}
	flp = (unsigned int *)((*(srp+l1indx))&0xfffffff0); /* mask out udt */
	for( ; l2indx<128; l2indx++)
	{
	    /* check level 3 index for INVALID*/
	    if ( (((*(flp+l2indx))&3)==0) || (((*(flp+l2indx))&3)==3) )
	    {
		*(flp+l2indx) = ( ATCalloc((pag_siz_8k?32:64)*4) | 3 ) /* or in udt*/;
		if ( *(flp+l2indx) == 3 ) return(-1);   /* ERROR */
#		if TRANS_DEBUG == 1
		printf("ATCalloc(%d*4) returned %x\n"
		    , (pag_siz_8k?32:64), (*(flp+l2indx))&0xfffffffc );
#		endif
	    }
	    slp = (unsigned int *)((*(flp+l2indx))&0xfffffff0); /* mask out udt */
	    for ( ; l3indx<(pag_siz_8k?32:64); l3indx++)
	    {
		*(slp+l3indx) = phys_start | (cache_mode<<5) 
				| (write_prot<<2) | 1 | expert;
		phys_start += page_siz;
		if (--num_pages == 0) goto trans_done;
	    }
	    l3indx = 0;
	}
	l2indx = 0;
    }
trans_done:
    return(1);
}


unsigned int addressTranslationClear (unsigned int log_start,
				      unsigned int log_end)
{
	unsigned int 	pag_siz_8k;
	unsigned int	num_pages;
	unsigned int	page_siz;
	unsigned int	mem_siz;
        unsigned int     tc;
	unsigned int	*srp;
	unsigned int	*flp;		/* first level pointer */
	unsigned int	*slp;		/* second level pointer */
	unsigned int	l1indx,l2indx,l3indx;
	unsigned int	stat=1;


    if (log_end < log_start ) return(-1);

    /* GET srp */

    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    srp = 0;
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    if (srp==0) return(-1);

    /* get page size and then number of pages */

    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );
    pag_siz_8k = (tc&0x4000);
    num_pages =  (log_end  >>(pag_siz_8k?13:12))
               - (log_start>>(pag_siz_8k?13:12));
    num_pages += 1;
    page_siz = 1<<(pag_siz_8k?13:12);


    /* return if log mem siz is greater than physical mem size */
    /* ref h/drv/memc040.h (vxworks) or 167 programmer's ref p.4-7*/
    mem_siz = (0x00400000<<(*MEMC_MCR & 0x7));
    if ( (num_pages * page_siz) > mem_siz ) return(-1);

    /*-------------------------------------------------------------------*/

    /* 3 level looping - to handle cross index mapping */

    /* get indexes - ref. p6-6 of 040 32 bit micro. user's manual */
    l1indx = log_start>>25;
    l2indx = (log_start&0x01fc0000)>>18;
    l3indx = (log_start&(pag_siz_8k?0x0003e000:0x0003f000))>>(pag_siz_8k?13:12);

    for ( ; l1indx<128; l1indx++)
    {
	/* check level 2 index */
	if ( (*(srp+l1indx))==0 )
	{
	    num_pages -= (pag_siz_8k?0x1000:0x2000);/* 4k/8K pgs at this level*/
	    if ( num_pages <= 0 ) break;
	    else /*just*/ continue;
	}
	flp = (unsigned int *)((*(srp+l1indx))&0xfffffff0); /* mask out udt */
	for( ; l2indx<128; l2indx++)
	{
	    /* check level 3 index */
	    if ( (*(flp+l2indx))==0 )
	    {
		num_pages -= (pag_siz_8k?32:64);/* 32/64 pgs at this level*/
		if ( num_pages <= 0 ) break;
		else /*just*/ continue;
	    }
	    slp = (unsigned int *)((*(flp+l2indx))&0xfffffff0); /* mask out udt */
	    for ( ; l3indx<(pag_siz_8k?32:64); l3indx++)
	    {
		*(slp+l3indx) = 0;
#		if TRANS_DEBUG == 1
		printf("clearing l1indx:%x,l2indx:%x,page:%d\n"
		    ,l1indx,l2indx,l3indx);
#		endif
		if (--num_pages == 0) break;
	    }

	    /* CHECK TO SEE IF WE CAN FREE */
	    for(l3indx=0;l3indx<(pag_siz_8k?32:64);l3indx++)
	    {
		if ( (*(slp+l3indx))!=0 ) break;
	    }
	    if (l3indx == (pag_siz_8k?32:64) )
	    {
		stat = cfree(slp);
#		if TRANS_DEBUG == 1
		printf("cfree(%x) returned %x\n", slp, stat );
#		endif
		*(flp+l2indx) = 0;
	    }

	    l3indx = 0;
	}

	/* CHECK TO SEE IF WE CAN FREE */
	for(l2indx=0;l2indx<128;l2indx++)
	{
	    if ( (*(flp+l2indx))!=0 ) break;
	}
	if (l2indx == 128 )
	{
	    stat = cfree(flp);
#	    if TRANS_DEBUG == 1
	    printf("cfree(%x) returned %x\n", flp, stat );
#	    endif
	    *(srp+l1indx) = 0;
	}

	l2indx = 0;
    }
    return(stat);
}

/*******************************************************************************
* addressTranslationDisable
*
*   Disables *PAGED* translations, 
*   sets *transparent* translations to a safe configuration.
*/

void addressTranslationDisable (void)
{
	unsigned int	tc;
	unsigned int	dtt0,dtt1,mem_siz;


 /* We must have the MMU active (i.e. some translations) inorder for the
    board to work, i.e. cache disabled for I/O register area.  And if
    any of the translations are active, then translations are active and
    we must have valid translations for system memory as well as for the
    I/O area.  So, when disabling *paged* address translations,
    *transparent* translations should be set up.
  */
    /* ref h/drv/memc040.h (vxworks) or 167 programmer's ref p.4-7*/
    mem_siz = (0x00400000<<(*MEMC_MCR & 0x7));
    dtt0 = (mem_siz>>9) |  0xe000 | (1<<5);
    dtt1 = 0x00ff0000 | 0xe000 | (2<<5);  /* dtt1 is check *after* dtt0 */

    /* WRITE TO tts */
    /*asm("movel %0,d0; movec d0,dtt0; " : "=g" (dtt0) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0) );
    asm("movel %0,d0; .word 0x4e7b, 0x0007" : "=g" (dtt1) );


    /* GET Translation Control register */	/* to preserve page size */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    tc &= 0x7fff;

    /* WRITE TO tc */
    /*asm("movel %0,d0; movec d0,tc; " : "=g" (tc) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0003" : "=g" (tc) );
}



/*******************************************************************************
* addressTranslationDisablePaged
*
*   Disables *PAGED* translations.
*   This routine can only be executed if the either
*	a) valid transparent translations have been setup for system DRAM
*	   and the I/O page, or
*       b) both data transparent translation register are disabled and the 
*	   data cache is disabled in the cacr
*/

void addressTranslationDisablePaged (void)
{
	unsigned int	tc;


    /* GET Translation Control register */	/* to preserve page size */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    tc &= 0x7fff;

    /* WRITE TO tc */
    /*asm("movel %0,d0; movec d0,tc; " : "=g" (tc) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0003" : "=g" (tc) );
}



/*******************************************************************************
* addressTranslationEnable
*
*   Enables *PAGED* translations,
*   there are hooks *transparent* translation cache modes,
*   dtt1 is set to a "safe" value.
*/


/* VXWORKS CACHE MODE DEFAULTS ARE:  0 FOR ENTIRE INSTRUCTION SPACE
				     1 FOR PHYS MEM DATA SPACE
				 AND 2 FOR ALL OTHER DATA SPACE

   WHEN USING dtt1base_override, VALUES WITH UPPER BITS SET MUST BE USED!
			0x00000000	this can be considered a default
			0x80000000
			0xC0000000
			0xE0000000
			0XF0000000
			0XF8000000
			 ...
			0XFE000000
		     OR 0XFF000000

For safety -- this program tries to asure the continued operation
of the hardware register at 0xffxxxxxx.
*/

int addressTranslationEnable (unsigned int dtt1base_override, /* 32bit base */
			      unsigned int itt0cm,
			      unsigned int itt1cm,
			      unsigned int dtt0cm,
			      unsigned int dtt1cm)
{
	unsigned int	mem_siz;
	unsigned int	mem_top;
	unsigned int	tc;
	unsigned int	itt0,itt1;
	unsigned int	dtt0,dtt1;
	unsigned int	*srp;
	unsigned int	dtt1base,dtt1mask;


    /* GET srp - check it! */

    /*asm("movec srp,d0; movel d0,%0" : "=g" (srp) );*/
    srp = 0;
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    if (srp==0) return(-1);


    /* first setup TT registers */

    /* GET tts */
    /*asm("movec itt0,d0; movel d0,%0" : "=g" (itt0) );*/
    itt0=0; itt1=0; dtt0=0; dtt1=0;
    asm(".word 0x4e7a, 0x0004; movel d0,%0" : "=g" (itt0) );
    asm(".word 0x4e7a, 0x0005; movel d0,%0" : "=g" (itt1) );
    asm(".word 0x4e7a, 0x0006; movel d0,%0" : "=g" (dtt0) );
    asm(".word 0x4e7a, 0x0007; movel d0,%0" : "=g" (dtt1) );

#define TTREGS 0	/* vxworks sets cm for tt regs - leave hooks in - */
			/* in case I every want to change cm */
#if TTREGS == 1
    itt0cm = (itt0cm & 3) << 5;		/* set up so it can be ORed in */
    itt1cm = (itt1cm & 3) << 5;
    dtt0cm = (dtt0cm & 3) << 5;
    dtt1cm = (dtt1cm & 3) << 5;
#else
    itt0cm = itt0&0x60;
    itt1cm = itt1&0x60;
    dtt0cm = dtt0&0x60;
    dtt1cm = dtt1&0x60;
#endif

    itt0 &= ~0x60;	/* clear out cache mode -- it will be ORed in below */
    itt1 &= ~0x60;
    dtt0 &= ~0x60;
    dtt1 &= ~0x60;

    itt0 |= itt0cm;
    itt1 |= itt1cm;
    dtt0 |= dtt0cm;
    dtt1 |= dtt1cm;

#ifdef NOTDEF
    /* ref h/drv/memc040.h (vxworks) or 167 programmer's ref p.4-7*/
    mem_siz = (0x00400000<<(*MEMC_MCR & 0x7));
    mem_top = mem_siz-1;
    /* tt regs map 16MEG chunks (contiguous! -ref 6-39) */
    dtt0 = ((mem_top&0xff000000)>>8) | 0xe000 | dtt0cm;  /* e000 = enable + */
						      /*  user or super mode */ 
#endif

    /* NOW CHANGE (dtt1) MASK FROM VXWORKS DEFAULT TO ALLOW TRANSLATION */
    /* BETWEEN FIRST AND LAST 16M BLOCKS */

    if (dtt1base_override)
    {
	dtt1base = dtt1base_override;
	switch(dtt1base_override)
	{
	case 0x00000000: dtt1mask = 0x00ff0000; break;
	case 0x80000000: dtt1mask = 0x007f0000; break;
	case 0xc0000000: dtt1mask = 0x003f0000; break;
	case 0xe0000000: dtt1mask = 0x001f0000; break;
	case 0xf0000000: dtt1mask = 0x000f0000; break;
	case 0xf8000000: dtt1mask = 0x00070000; break;
	case 0xfc000000: dtt1mask = 0x00030000; break;
	case 0xfe000000: dtt1mask = 0x00010000; break;
	default: dtt1base = 0xff000000; dtt1mask = 0; break;
	}
	dtt1 = dtt1base | dtt1mask | 0xe000 | dtt1cm; /* last (upper) 16meg block */
    }

#   if TRANS_DEBUG == 1
    printf("dtt0=%x\n",dtt0);
#   endif

    /* WRITE TO tts */
    /*asm("movel %0,d0; movec d0,itt0; " : "=g" (itt0) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0004" : "=g" (itt0) );
    asm("movel %0,d0; .word 0x4e7b, 0x0005" : "=g" (itt1) );
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0) );
    asm("movel %0,d0; .word 0x4e7b, 0x0007" : "=g" (dtt1) );


   /* NOW DO *PAGED* TRANSLATION */

    /* FLUSH ATCs ref p6-37 in 040 32 bit micro user's man */
    /*asm("pflusha"); */   /* ref 4-36 in programmer's ref */
    asm(".word 0xf518"); /* pflusha */

    /* GET Translation Control register */	/* to preserve page size */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    tc |= 0x8000;	/* SET ENABLE BIT */

#   if TRANS_DEBUG == 1
    printf("tc=%x\n",tc);
#   endif

    /* WRITE TO tc */
    /*asm("movel %0,d0; movec d0,tc; " : "=g" (tc) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0003" : "=g" (tc) );

    return(1);
}
#endif



int addressTranslationEnablePaged( void )
{
	unsigned int	tc;


    /* FLUSH ATCs ref p6-37 in 040 32 bit micro user's man */
    /*asm("pflusha"); */   /* ref 4-36 in programmer's ref */
    asm(".word 0xf518");

    /* GET Translation Control register */	/* to preserve page size */
    /*asm("movec tc,d0; movel d0,%0" : "=g" (tc) ); */
    asm(".word 0x4e7a, 0x0003; movel d0,%0" : "=g" (tc) );

    tc |= 0x8000;	/* SET ENABLE BIT */

#   if TRANS_DEBUG == 1
    printf("tc=%x\n",tc);
#   endif

    /* WRITE TO tc */
    /*asm("movel %0,d0; movec d0,tc; " : "=g" (tc) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0003" : "=g" (tc) );

    return(1);
}




int addressTranslationdtt0 (unsigned int base,
			    unsigned int mask,
			    unsigned int enable,
			    unsigned int cm,
			    unsigned int wp,
			    unsigned int expert)
{
	unsigned int	dtt0;

    base >>= 24;
    mask >>= 24;
    enable &= 1;
    cm &= 3;
    wp &= 1;
    if ( expert == 0 ) expert = 0x4000; /* ignore user/super distinction */
					/* unless explicitly told not too */

    dtt0 = (base<<24) | (mask<<16) | (enable<<15) | (cm<<5) | (wp<<2) | expert;

    /* WRITE TO tts */
    /*asm("movel %0,d0; movec d0,dtt0; " : "=g" (dtt0) );*/
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0) );
    return(1);
}



int addressTranslationTT  (unsigned int TTreg,
			   unsigned int base,
			   unsigned int mask,
			   unsigned int enable,
			   unsigned int cm,
			   unsigned int wp,
			   unsigned int expert)
{
	unsigned int	tt;

    TTreg &= 3;		/*  4 or 0 ==> itt0       (or 8, 0xc, etc. )
			    5 or 1 ==> itt1
			    6 or 2 ==> dtt0
			    7 or 3 ==> dtt1  */
    base >>= 24;
    mask >>= 24;
    enable &= 1;
    cm &= 3;
    wp &= 1;
    if ( expert == 0 ) expert = 0x4000; /* ignore user/super distinction */
					/* unless explicitly told not too */

    tt = (base<<24) | (mask<<16) | (enable<<15) | (cm<<5) | (wp<<2) | expert;

    /* WRITE TO tt */
    /*asm("movel %0,d0; movec d0,tt; " : "=g" (tt) );*/
    switch(TTreg)
    {
    case 0:
	asm("movel %0,d0; .word 0x4e7b, 0x0004" : "=g" (tt) );
	break;
    case 1:
	asm("movel %0,d0; .word 0x4e7b, 0x0005" : "=g" (tt) );
	break;
    case 2:
	asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (tt) );
	break;
    case 3:
	asm("movel %0,d0; .word 0x4e7b, 0x0007" : "=g" (tt) );
	break;
    }
    return(1);
}



int addressTranslationSRPSet( unsigned int	*srp )
{
    asm("movel %0,a0; .word 0x4e7b, 0x8807" : "=g" (srp) );
    asm(".word 0x4e7a, 0x0807; movel d0,%0" : "=g" (srp) );
    return((int)srp);
}


/**********************************************************************
* The main reason for thses addressTranslation{Read,Write}Phys routines
* is to be compatible with the CPU's address translation table search
* which searches physical memory as describe on p. 6-24. Basically,
* cacheable/writethrough but do not allocate in the cache for misses.
* Although, these routines, as they are not actually doing table searches
* (in a hardware sense), will allocate in the cache for misses.
*/

void addressTranslationWritePhys( unsigned	*phys
				, unsigned	x       )
{
	unsigned	dtt0, dtt0_table_search;
	unsigned short	sr;

    asm(".word 0x4e7a, 0x0006; movel d0,%0" : "=g" (dtt0) );
    dtt0_table_search = (0x00ffe000 | (dtt0&0x00000300));
    asm(".word 0x40c0; movew d0,%0; andiw #0xefff,d0; .word 0x46c0; " : "=g" (sr) );    /* DISABLE INTS */
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0_table_search) );
    *phys = x;
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0) );
    asm("movew %0,d0; .word 0x46c0;" : "=g" (sr) );	/* RESTORE SR */
}


unsigned addressTranslationReadPhys( unsigned	*phys )
{
	unsigned	dtt0, dtt0_table_search;
	unsigned	z1;
	unsigned short	sr;

    asm(".word 0x4e7a, 0x0006; movel d0,%0" : "=g" (dtt0) );
    dtt0_table_search = (0x00ffe000 | (dtt0&0x00000300));
    asm(".word 0x40c0; movew d0,%0; andiw #0xefff,d0; .word 0x46c0; " : "=g" (sr) );    /* DISABLE INTS */
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0_table_search) );
    z1 = (*phys);
    asm("movel %0,d0; .word 0x4e7b, 0x0006" : "=g" (dtt0) );
    asm("movew %0,d0; .word 0x46c0;" : "=g" (sr) );	/* RESTORE SR */

    return ( z1 );
}

/* ------------------ End 0f AddressTranslation Routines ------------------ */


/* This routine performs a VME sys reset on a mvme167. */

void vmeSysReset (void)
  {
  unsigned long int *resetAddr = ( unsigned long int *) 0xfff40060;
  unsigned long int temp;	/* To guarantee long word accesses. */

  printf("Resetting the VMEbus\n");
  temp = *resetAddr;
  temp |= 0x800000;	/* Set the reset bit. */
  *resetAddr = temp;
  }


/* This function sets the specified task priority after a delay.
   B. MacKinnon: 22-March-1993.

   The purpose is so tShell's priority can be adjusted by the startup script.
   Normally, tShell cannot be adjusted during system startup since it get's
   respawned. */

void delayTaskPrioritySet (char *tname,     /* IN: Task name to set. */
			   int   prio,      /* IN: Priority to set it to. */
			   int   delay)     /* IN: Delay in ticks to wait before setting the priority. */
 {
  taskDelay(delay);
  taskPrioritySet(taskNameToId(tname), prio);
  printf("delayTaskPrioritySet: Changed task %s to priority %d\n", tname, prio);
  }



void sysReset( int vme_else_local )		/* 0=local, other=sys */
{
	unsigned	x;

    /* Mask out all interrupts */
    asm(".word 0x40c0; oriw #0700,d0; .word 0x46c0;");/* DISABLE INTS */

    /* clear dwto period to 512us */
    x = *(unsigned *)0xfff4004c;
    x &= ~0x00000f00;
    *(unsigned *)0xfff4004c = x;

    /* setup the wd control */
    x = *(unsigned *)0xfff40060;
    x |= 0x00030000;
    x &= ~0x00040000;	/* reset to LRESET */
    if (vme_else_local) x |= 0x00040000;

    /* start the dog and halt */
    *(unsigned *)0xfff40060 = x;
    asm("stop #0x2700"); /* priv instruction, ref. p.4-77 */
}


	struct memctl
	{   struct memc
	    {   int	cid:8;
		int	u1:24;
		int	rev:8;
		int	u2:24;
		int	cfg:8;
		int	u3:24;
		int	altSts:8;
		int	u4:24;
		int	altCtl:8;
		int	u5:24;
		int	baseAdr:8;
		int	u6:24;
		int	ramCtl:8;
		int	u7:24;
		int	busClk:8;
		int	u8:24;
	    }			memc_s;
	    int			dataCtl:8;
	    int			u1:24;
	    int			scrubCtl:8;
	    int			u2:24;
	    int			scrubPeriodB1:8;
	    int			u3:24;
	    int			scrubperiodB0:8;
	    int			u4:24;
	    int			chipPrescale:8;
	    int			u5:24;
	    int			scrubTimeOnOff:8;
	    int			u6:24;
	    int			scrubPrescaleB2:8;
	    int			u7:24;
	    int			scrubPrescaleB1:8;
	    int			u8:24;
	    int			scrubPrescaleB0:8;
	    int			u9:24;
	    int			scrubTimerB1:8;
	    int			ua:24;
	    int			scrubTimerB0:8;
	    int			ub:24;
	    int			scrubAdrCntrB3:8;
	    int			uc:24;
	    int			scrubAdrCntrB2:8;
	    int			ud:24;
	    int			scrubAdrCntrB1:8;
	    int			ue:24;
	    int			scrubAdrCntrB0:8;
	    int			uf:24;
	    int			errLogger:8;
	    int			ug:24;
	    int			errAdrB3:8;
	    int			uh:24;
	    int			errAdrB2:8;
	    int			ui:24;
	    int			errAdrB1:8;
	    int			uj:24;
	    int			errAdrB0:8;
	    int			uk:24;
	    int			errSyndrome:8;
	    int			ul:24;
	    int			defaults1:8;
	    int			um:24;
	    int			defaults2:8;
	    int			un:24;
	};
void printMemctl(  struct memctl	*memctl_sp
		 , int			 mcecc )
{
    /* NOT COMPLETE */

    printf(  "conf    = 0x%02.2x (ro)\n", memctl_sp->memc_s.cfg );
    printf(  "          fastRead=%s extParityEn=%s WritePerBitMode=%s\n"
	   , (memctl_sp->memc_s.cfg&0x20)?"YES":"NO"
	   , (memctl_sp->memc_s.cfg&0x10)?"YES":"NO"
	   , (memctl_sp->memc_s.cfg&0x08)?"NO" :"YES" );
    printf(  "          msiz=%d Mb\n", 4<<(memctl_sp->memc_s.cfg&0x07) );
    printf(  "altSts  = 0x%02.2x (ro)\n", memctl_sp->memc_s.altSts );
    printf(  "altCtl  = 0x%02.2x (rw)\n", memctl_sp->memc_s.altCtl );
    printf(  "baseAdr = 0x%02.2x (rw)\n", memctl_sp->memc_s.baseAdr );
    printf(  "ramCtl  = 0x%02.2x (rw)\n", memctl_sp->memc_s.ramCtl );
    printf(  "          BERR_EN=%s\n"
	   , (memctl_sp->memc_s.ramCtl&0x02)?"YES":"NO" );
    printf(  "busClk  = 0x%02.2x (rw)\n", memctl_sp->memc_s.busClk );
}

int	memBoardShow(void)
{
	struct memctl		memctl_s;
	unsigned int		ii, i, *i_p;

    /* NOT COMPLETE */

    i_p = (int *)0xfff43000;
    ii = *i_p;
    printf(  "1st M%s -\n", ((ii>>24)==0x80)?"EMC040":"CECC" );
    if ((ii>>24) == 0x80)
    {   for (i=sizeof(struct memc)/sizeof(int); i--; )
	    *(((int *)&memctl_s)+i) = *(i_p+i);
    }
    else
    {   for (i=sizeof(struct memctl)/sizeof(int); i--; )
	    *(((int *)&memctl_s)+i) = *(i_p+i);
    }

    printMemctl( &memctl_s, (ii>>24)==0x81 );

    if (vxMemProbe(0xfff43104,VX_READ,4,&ii) != ERROR)
    {   printf( "2nd M%s present\n", ((ii>>24)==0x80)?"EMC040":"CECC" );
	for (i=sizeof(struct memc)/sizeof(int); i--; )
	    *(((int *)&memctl_s)+i) = *(i_p+i);
    }

    return (OK);
}

