#ifndef mv162

/* ========================================================================== */
/*    Routine to read the EBIs           */
/*    G. Zioulas     4/7/93              */
/*                                       */
/*    Modifications: 
 * 01-dec-1995  M.Vittone  Conditionalizing the entire code not to use it
 *			   for the mv162 since it does not support dma
 *			   transfers.
*/

/* readEbi - Read up to 10 ebi buffers 
   nebis - Number of buffers to read
   addrebi - Array of pointers to the 0xXXXX0000 addresses (Read address)
   chkebi - Array of pointers to the 0xXXXX8004 addresses (Status flag)
   nwords - Total number of 32-bit words read
   data - Array of words read (First word: Word count for first buffer (=N1)
                               Second ... N1+1 : Data from buffer one
                               N1+2 : Word count for second buffer N2 (=N2)
                               N1+3...N1+N2+2 : Data from buffer two   etc.

   returns 0 if all OK, 1 if buffers out of Synch
*/
/*
** Modifications
** 23dec93	das	VxWorks 5.1, ansi fixes
**
*/

#include <vxWorks.h>
#include <stdioLib.h>
#include <math.h>

#include "mvsup.h"
#include "mvsup_prototypes.h"

#include <taskLib.h>
#include <wdLib.h>
#define WAITFORDMA  { while (!dmaDone()) ; }

int readEbi (unsigned int nebis, 
	     int 	 *addrebi[], 
	     int 	 *chkebi[],
	     unsigned int *nwords,
	     unsigned int data[])
{
	int nbytes,i; 
	int nwca,nwcb,a,sync;
	unsigned int nwebi[10]; 

	*nwords=0;
	
	for(i=0; i<nebis; ++i){
		while (((a=*chkebi[i])&0x8) == 0x8){
			taskDelay(30);
		}
	}
	
	a= *addrebi[0]; 
	sync=a & 0xf000;
	nwca=a & 0xfff;
	nwebi[0]=(int)nwca/2;
	if(nwca&0x1) ++nwebi[0]; 
	for(i=1; i<nebis; ++i){
	   a= *addrebi[i]; 
	   if(sync != (a&0xf000)){
		printf("*** VDAS out of synch!!!  %x %x\n",sync,a&0xf000);
		return 1;
	   }
	   nwcb=a & 0xfff;
	   nwebi[i]=(int)nwcb/2;
	   if(nwcb&0x1) ++nwebi[i]; 
	}
	
	for(i=0; i<nebis; ++i){
	   nbytes=nwebi[i]*4;
	   data[*nwords]=nwebi[i];
	   ++*nwords;
	   dmaStart((caddr_t)addrebi[i],(caddr_t)(data+*nwords),nbytes,0,1,9);
	   WAITFORDMA
	   *nwords += nwebi[i];
	}
	return 0;
	}

/*    Routine to read one EBI            */
/*    G. Zioulas     4/7/93              */
/*                                       */
/* readEbiOne - Read one ebi buffers 
   addrebi - Pointer to the 0xXXXX0000 address (Read address)
   chkebi - Pointer to the 0xXXXX8004 address (Status flag)
   nwords - Total number of 32-bit words read
   data - Array of words read (First word: Word count for buffer (=N1)
                               Second ... N1+1 : Data from buffer 

   returns 0 if all OK, 1 if buffers out of Synch
*/

int readEbiOne (int 	     *addrebi,
		int 	     *chkebi,
		unsigned int *nwords,
		unsigned int  data[])
{
	int nbytes; 
	int nwca,a,nwebi;

	*nwords=0;
	
	while (((a=*chkebi)&0x8) == 0x8){
		taskDelay(30);
	}
	
	a= *addrebi; 
	nwca=a & 0xfff;
	nwebi=(int)nwca/2;
	if(nwca&0x1) ++nwebi; 
	
	nbytes=nwebi*4;
	data[*nwords]=nwebi;
	++*nwords;
	dmaStart((caddr_t)addrebi,(caddr_t)(data+*nwords),nbytes,0,1,9);
	   WAITFORDMA
	*nwords += nwebi;

	return 0;
	}
/* ============================================================================ */

#endif		/* on mv162  */
