/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1992 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/*
 
File:           sysTools.c
=====

Author:         Jim Pangburn
=======         CDF on-line computing
                Fermilab MS 318
                P.O. Box 500
                Batavia, IL  60510
                (708) 840-2207

        e-mail: "pangburn@fnal.fnal.gov"

Creation date:  4-Feb-1994
==============

Revision history:-
=================

4-Feb-1994      created


17-mar-95	 M.Vittone: adding some changes and conditionals to build
                           under VxWorks v5_1_1.
                           Also adding include files: stdio.h, logLib.h and
                           taskHookLib.h
			   Also fix replaced sigHandlers strcuture member
			   with sigt_vec .
			   Fixed called for logMsg.
			   Need still to fix the call to trcStack
Description:-
============

	Tools for uptime, crash analysis and system logging for VxWorks.
	Motivated by those hanging, crashing boards.  Note how I have followed
	the WRS naming convention, except for some routines; in other words,
	I have followed the WRS naming convention.


Include files:-
==============
*/

#include "vxWorks.h"
#include "taskLib.h"
#include "sigLib.h"
#include "string.h"

#include "stdio.h"      /* needed to include this */

#include "sysLib.h"
#include "symbol.h"
#include "sysSymTbl.h"
#include "rebootLib.h"
#include "intLib.h"
#include "iv.h"
#include "esf.h"
#include "dsmLib.h"
#include "trcLib.h"
#include "configAll.h"
#include "config.h"
#include "tod_user.h"
#include "taskHookLib.h"
#include "excLib.h"
#include "logLib.h"
#include "sysTools.h"

#include <time.h>                       /* time defines                     */


#if CPU_FAMILY==MIPS_R3K
#include "sysToolsR3k.h"
#endif		/* R3000 */

#if CPU_FAMILY==MC680X0
#include "sysTools68k.h"
#endif		/* M68k */


/*
Macros:-
=======
*/


/*
Global declarations:-
====================
*/

#if CPU_FAMILY==MIPS_R3K
#define SYMFINDBYVALUE	symFindByValueR3k
/* copied from usrConfig.c */
typedef struct
    {
    unsigned short      lui_at;
    unsigned short      upper;
    unsigned short      ori_at;
    unsigned short      lower;
    unsigned long       jr_at;
    unsigned long       nop;
    } JUMP_TBL;

extern JUMP_TBL		*romJumpTbl;
char romSymbolName[MAX_SYS_SYM_LEN+4];

#endif		/* R3000 */

#if CPU_FAMILY==MC680X0
#define SYMFINDBYVALUE	symFindByValue

extern int vxIntStackBase;
extern void sysToolsExcStub();

/* Got rid here of conditional on VXWORKS_5_1_1 for excJobAdd */

typedef struct
    {
    unsigned short sr;                  /* status register */
    INSTR *pc;                          /* return program counter */
    unsigned short vectorOffset;        /* vector offset */
    unsigned long effectiveAddr;        /* effective address */
    unsigned long aa;                   /* fault address */
    } ESF4;

/* Got rid here of conditional on VXWORKS_5_1_1 */

#endif          /* M68k */

/*
  Local declarations:-
  ===================
*/



#ifdef SYSTOOLS_UTILS
/************************************************************************
*  uptime() -								*
*	provides display similar to Unix uptime(), as applicable to	*
*	VxWorks.							*
************************************************************************/
VOID uptime()

  {
  time_t current_time;
  char current_time_ascii[26];
  time_t up_time;
  int search;

  
  current_time = time(NULL);

  bcopy( (char*)asctime(localtime(&current_time)), current_time_ascii, 26 );

  /* get rid of that annoying newline in there */
  for( search = 0; search < 26; search++ )
    if( current_time_ascii[search] == '\n' )
      current_time_ascii[search] = '\0';
  up_time = current_time - UPTIME;
  printf( "%s  up %d days, %d:%02d\n", current_time_ascii, 
	(up_time / 86400), ((up_time % 86400) / 3600),
	(((up_time % 86400) % 3600) / 60) );
  return;
  }
#endif		/* utils */



#ifdef SYSTOOLS_UTILS
/************************************************************************
*  crashShow() -							*
*	Show info about the last crash, if we have it.			*
************************************************************************/
VOID crashShow()

  {
  time_t crashtime;
  char crash_message[SYSTEM_CRASH_MESSAGE_SIZE];
  unsigned long crash_regs[SYSTEM_CRASH_REGS_SIZE>>2];
  char task_name[32];


  sysNvRamGet( (char *) &crashtime, 4, SYSTOOLS_NVRAM_CRASH );
  if( (int)crashtime == 0 )
    printf( "No crash information saved.\n" );
  else
    {
    /* get & display exception message */
    sysNvRamGet( crash_message, SYSTEM_CRASH_MESSAGE_SIZE,
	SYSTOOLS_NVRAM_CRASH+4 );
    printf( "After crash, system booted at %s", 
	asctime(localtime(&crashtime)) );
    printf( "Reason for crash:\n" );
    printf( "%s", crash_message );
    printf( "Registers at exception time:\n" );
    /* get & display registers at time of crash */
    sysNvRamGet( (char *) crash_regs, SYSTEM_CRASH_REGS_SIZE,
      SYSTOOLS_NVRAM_REGS );
    esfShow( crash_regs );
#ifdef SYSTOOLS_TASKWATCH
    /* get & display task running at the time */
    sysNvRamGet( task_name, SYSTEM_TASK_INFO_SIZE, SYSTOOLS_NVRAM_TASK );
    printf( "Last task running: %s\n", task_name );
#endif		/* taskwatch */
    }
  return;
  }
#endif		/* utils */



#ifdef SYSTOOLS_TASKWATCH
/************************************************************************
*  sysToolsTaskSwitchHook() -						*
*	Somewhat naive attempt to determine cause of mysterious board	*
*	hangs by recording in NVRAM, at each task switch, the task	*
*	about to be switched in, assuming it might be the culprit.	*
************************************************************************/
VOID sysToolsTaskSwitchHook( oldTCB, newTCB )
  WIND_TCB *oldTCB;
  WIND_TCB *newTCB;

  {
  char *task_name_ptr;


  task_name_ptr = taskName( (int) newTCB );
  if( task_name_ptr != NULL )

  bcopy( task_name_ptr, (char*)SYSTEM_TASK_INFO, SYSTEM_TASK_INFO_SIZE );

  return;
  }
#endif		/* taskwatch */



#ifdef SYSTOOLS_UTILS
/************************************************************************
*  search() -								*
*	Searches memory for a pattern.  Very flexible in search method.	*
*	Call parameters for starting location, length (bytes), the	*
*	number of bytes to increment each search iteration, the pattern	*
*	to search for, and a mask specifying which byte(s) of a "word"	*
*	to match to the pattern, where a "word" is defined as the unit	*
*	derived by incrementing by the increment parameter.		*
************************************************************************/
int	search( start, len, inc, pattern, bytemask )
  char *start;		/* starting location for search */
  int len;		/* length (in bytes) to continue search */
  int inc;		/* increment (in bytes) for each iteration */
  unsigned long pattern;	/* pattern to search for (right-aligned) */
  int bytemask;		/* mask for byte #s to match in search "word" */

  {

  unsigned long s;
  unsigned long search_word;
  char b0, b1, b2, b3;
  unsigned short h0, h1;
  unsigned long mask;


  mask = 0;
  if( (bytemask & 1) != 0 )
    mask |= 0xff;
  if( (bytemask & 2) != 0 )
    mask |= 0xff00;
  if( (bytemask & 4) != 0 )
    mask |= 0xff0000;
  if( (bytemask & 8) != 0 )
    mask |= 0xff000000;

  if( (inc < 0) || (inc == 3) || (inc > 4) )
    {
    printf( "search: only support byte, halfword, and word increments.\n" );
    return(-1);
    }

  for( s = (unsigned long) start; s < ((unsigned long) start + len);
	s += inc )
    {
    switch( inc )
      {
      case 1:
	b0 = *(char *)s;
	b1 = *(char *)(s+1);
	b2 = *(char *)(s+2);
	b3 = *(char *)(s+3);
	search_word = ((b0 << 24) + (b1 << 16)) | (b2 << 8) + b3;
	break;

      case 2:
	h0 = *(unsigned short *)s;
	h1 = *(unsigned short *)(s+2);
	search_word = (h0 << 16) | h1;
	break;

      case 4:
	search_word = *(unsigned long *)s;
	break;
      }
    if( (search_word & mask) == (pattern & mask) )
      printf( "%x found at %08x\n", pattern, s );

    }

  return(0);
  }
#endif		/* utils */



/************************************************************************
*  sysToolsInit() -							*
*	Initializes the "systools" package.  MUST be called at system	*
*	startup time to ensure that all facilities are enabled.		*
************************************************************************/
STATUS sysToolsInit()
  {
  time_t dummy;
  int iloop;
  int vector;


#ifdef SYSTOOLS_UTILS
  /* first, get current time */
  UPTIME = time(NULL);

  bcopy( (char*)asctime(localtime(&UPTIME)), UPTIME_ASCII, 26 );

  /* first, copy last crash message to permanent area, if there was one */
  if( *(long *)SYSTEM_CRASH_MESSAGE != 0 )
    {
    if( *SYSTEM_CRASH_MESSAGE == 0 )
      *SYSTEM_CRASH_MESSAGE = '.';
    sysNvRamSet( (char *) &UPTIME, 4, SYSTOOLS_NVRAM_CRASH );
    sysNvRamSet( (char *) SYSTEM_CRASH_MESSAGE, SYSTEM_CRASH_MESSAGE_SIZE, 
	SYSTOOLS_NVRAM_CRASH+4 );
    sysNvRamSet( (char *) SYSTEM_CRASH_REGS, SYSTEM_CRASH_REGS_SIZE,
	SYSTOOLS_NVRAM_REGS );
    bzero( (char *) SYSTEM_CRASH_MESSAGE, SYSTEM_CRASH_MESSAGE_SIZE );
    sysNvRamSet( (char *) SYSTEM_TASK_INFO, SYSTEM_TASK_INFO_SIZE,
	SYSTOOLS_NVRAM_TASK );
    }
#endif		/* utils */

#ifdef SYSTOOLS_EXCEPTIONS
/* connect to your own favorite vectors, as defined in header file */
  for( iloop = 0; iloop < INSTALL_TABLE_SIZE; iloop++ )
    {
    if( systoolsExcInstallTable[iloop].install_flag == TRUE )
      {
      vector = systoolsExcInstallTable[iloop].vector_number;
#if CPU_FAMILY==MIPS_R3K
      systoolsExcInstallTable[iloop].old_handler =
	intVecGet( INUM_TO_IVEC(vector) );
      intVecSet( INUM_TO_IVEC(vector), (FUNCPTR) sysToolsR3kExcHandle );
#endif		/* R3000 */
#if CPU_FAMILY==MC680X0
      systoolsExcInstallTable[iloop].old_handler =
	intVecGet( vector );
/*	intVecGet( (FUNCPTR *)vector );  */
#if CPU==MC68000 || CPU==MC68008
      if( (vector == IV_BUS_ERROR) || (vector == IV_ADDRESS_ERROR) )
	intVecSet( vector, (FUNCPTR) sysToolsAccessExcStub );
      else
	intVecSet( vector, (FUNCPTR) sysToolsGenExcStub );
#else
      intVecSet( vector, sysToolsExcStub );
/*      intVecSet( vector, (FUNCPTR) sysToolsExcStub ); */
#endif		/* primitive or decent stack frame CPU */
#endif		/* 68k family */
      }
    }
#endif		/* exceptions */

#ifdef SYSTOOLS_TASKWATCH
  /* install "watchdog" to log tasks running to catch someone hanging board */
  if( taskSwitchHookAdd( (FUNCPTR) sysToolsTaskSwitchHook ) == ERROR )

    logMsg( "Failed to add task switch hook; task watchdog not installed!\n",0,0,0,0,0,0 );


#endif		/* taskwatch */

  return(OK);
  }




#ifdef SYSTOOLS_EXCEPTIONS
/************************************************************************
*  esfShow() -								*
*	This routine prints the contents of an exception stack frame.	*
************************************************************************/
void esfShow( esf )
  volatile unsigned long *esf;
  {

#if CPU_FAMILY==MIPS_R3K
  printf( "CR=%08x, BADA=%08x, SR=%08x, LO=%08x, HI=%08x\n",
	esf[E_STK_CAUSE>>2], esf[E_STK_BADVADDR>>2], 
	esf[E_STK_SR>>2], esf[E_STK_LO>>2], esf[E_STK_HI>>2] );
  printf( "V0=%08x, V1=%08x, A0=%08x, A1=%08x, A2=%08x, A3=%08x\n",
	esf[E_STK_V0>>2], esf[E_STK_V1>>2], esf[E_STK_A0>>2],
	esf[E_STK_A1>>2], esf[E_STK_A2>>2], esf[E_STK_A3>>2] );
  printf( "T0=%08x, T1=%08x, T2=%08x, T3=%08x, T4=%08x, T5=%08x\n",
	esf[E_STK_T0>>2], esf[E_STK_T1>>2], esf[E_STK_T2>>2],
	esf[E_STK_T3>>2], esf[E_STK_T4>>2], esf[E_STK_T5>>2] );   
  printf( "T6=%08x, T7=%08x, T8=%08x, T9=%08x, S0=%08x, S1=%08x\n",
	esf[E_STK_T6>>2], esf[E_STK_T7>>2], esf[E_STK_T8>>2], 
	esf[E_STK_T9>>2], esf[E_STK_S0>>2], esf[E_STK_S1>>2] );
  printf( "S2=%08x, S3=%08x, S4=%08x, S5=%08x, S6=%08x, S7=%08x\n",
	esf[E_STK_S2>>2], esf[E_STK_S3>>2], esf[E_STK_S4>>2], 
	esf[E_STK_S5>>2], esf[E_STK_S6>>2], esf[E_STK_S7>>2] );
  printf( "K0=%08x, K1=%08x, GP=%08x, FP=%08x, RA=%08x, AT=%08x\n",
	esf[E_STK_K0>>2], esf[E_STK_K1>>2], esf[E_STK_GP>>2], 
	esf[E_STK_FP>>2], esf[E_STK_RA>>2], esf[E_STK_AT>>2] );
  printf( "SP=%08x, EPC=%08x\n",
	esf[E_STK_SP>>2], esf[E_STK_EPC>>2] );
#endif		/* R3000 */
#if CPU_FAMILY==MC680X0
  printf( "D0=%08x, D1=%08x, D2=%08x, D3=%08x\n", 
    esf[0], esf[1], esf[2], esf[3] );
  printf( "D4=%08x, D5=%08x, D6=%08x, D7=%08x\n", 
    esf[4], esf[5], esf[6], esf[7] );
  printf( "A0=%08x, A1=%08x, A2=%08x, A3=%08x\n", 
    esf[8], esf[9], esf[10], esf[11] );
  printf( "A4=%08x, A5=%08x, A6=%08x, A7=%08x\n", 
    esf[12], esf[13], esf[14], esf[15] );
#endif		/* M68k */

  return;
  }
#endif		/* exceptions */



#ifdef SYSTOOLS_UTILS
#if CPU_FAMILY==MIPS_R3K
/************************************************************************
*  symFindByValueR3k() -						*
*	This routine is only necessary because, like so many other	*
*	things, symFindByValue does not work on R3000!  This probably	*
*	stems from Wind River's insistence upon using signed integers	*
*	where unsigned integers belong, because symFindByValue() DOES	*
*	work for exact matches, but not for anything else, and their	*
*	comparison will not behave as they expect, since the base	*
*	address in MIPS-land often has bit 31 (the sign bit!) set.	*
************************************************************************/
STATUS symFindByValueR3k( symTblId, value, name, pValue, pType )
  SYMTAB_ID	symTblId;	/* ID of symbol table to find name in */
  int		value;		/* value of symbol to find */
  char		*name;		/* pointer where to return symbol name string */
  int		*pValue;	/* pointer where to return symbol value */
  SYM_TYPE	*pType;		/* pointer where to return symbol type */

  {
  int local_value;


  /* first, convert us to a standard (Kseg0) address space */
  local_value = (K0BASE | (value & 0x3fffffff));

  /* if we are past ROM, don't bother */
  if( (unsigned long) local_value > (ROM_BASE_ADRS + ROM_SIZE) )
    return( ERROR );

  /* initialize search variables */
  sysToolsSearchSym = local_value;
  sysToolsFoundSym = 0;
  sysToolsMinDiff = 0xffffffff;

  /* special case for ROM in R3K because of jump table */
  if( (unsigned long) local_value >= (ROM_BASE_ADRS) )
    {
    romSymFindByValue( value, name, pValue, pType );
    return( OK );
    }

  /* execute our search routine for every symbol in table to find closest */
  (VOID) symEach( symTblId, (FUNCPTR) sysToolsR3kSymEach, 0 );
  if( sysToolsFoundSym != 0 )
    {
    symFindByValue( symTblId, sysToolsFoundSym, name, pValue, pType );
    return( OK );
    }
  else
    return( ERROR );

  }
#endif		/* R3000 */
#endif		/* utils */



#ifdef SYSTOOLS_UTILS
#if CPU_FAMILY==MIPS_R3K
/************************************************************************
*  sysToolsR3kSymEach() -						*
*	This routine is only necessary because, like so many other	*
*	things, symFindByValue does not work on R3000!  This probably	*
*	stems from Wind River's insistence upon using signed integers	*
*	where unsigned integers belong, because symFindByValue() DOES	*
*	work for exact matches, but not for anything else, and their	*
*	comparison will not behave as they expect, since the base	*
*	address in MIPS-land often has bit 31 (the sign bit!) set.	*
************************************************************************/
BOOL sysToolsR3kSymEach( name, val, type, arg )
  char *name;		/* entry name */
  int val;		/* value associated with the entry */
  SYM_TYPE type;	/* entry type */
  int arg;		/* arbitrary user-supplied argument */

  {
  unsigned long diff;


  /* ignore symbols with values greater than our search symbol */
  if( (unsigned long) val > sysToolsSearchSym )
    return(TRUE);

  diff = sysToolsSearchSym - val;
  if( diff < sysToolsMinDiff )
    {
    sysToolsMinDiff = diff;
    sysToolsFoundSym = (unsigned long) val;
    }

  return(TRUE);

  }
#endif		/* R3000 */
#endif		/* utils */



#ifdef SYSTOOLS_UTILS
#if CPU_FAMILY==MIPS_R3K
/************************************************************************
*  romSymFindByValue() -						*
*	This routine is necessary because of the ROM jump table,	*
*	which apparently needed to be created for ROM-resident		*
*	VxWorks/R3000 in order to allow the linker to link with the	*
*	text in ROM, due to internal linker relocation limitations.	*
*	Because of the ROM jump table, the symbols in the system	*
*	symbol table actually point to ROM jump stubs, which call the	*
*	actual routines in ROM, thus rendering the system symbol table	*
*	relatively useless when it comes to routines in ROM.  For	*
*	example, if a ROM routine causes an unhandled exception, the	*
*	exception PC is VERY difficult to trace back to a symbol table	*
*	entry manually.							*
************************************************************************/
STATUS romSymFindByValue( value, name, pVal, pType )
  int value;		/* value we are attempting to place */
  char *name;		/* entry name */
  int *pVal;		/* value associated with the entry */
  SYM_TYPE *pType;	/* entry type */

  {
  JUMP_TBL *romTableEntry;
  unsigned long jump_val;
  unsigned long diff;


  for( romTableEntry = (JUMP_TBL *) romJumpTbl; 
    ((romTableEntry->ori_at == 0x3421) && (romTableEntry->jr_at == 0x00200008));
    romTableEntry++ )
    {
    jump_val = (romTableEntry->upper << 16) | romTableEntry->lower;
    diff = (unsigned long) value - jump_val;
    if( diff < sysToolsMinDiff )
      {
      sysToolsMinDiff = diff;
      sysToolsFoundSym = (unsigned long) romTableEntry;
      }
    }
  symFindByValue( sysSymTbl, sysToolsFoundSym, name, pVal, pType );
  *pVal = value + sysToolsMinDiff;
  return(OK);
  }
#endif		/* R3000 */
#endif		/* utils */



#ifdef SYSTOOLS_EXCEPTIONS
#if CPU_FAMILY==MIPS_R3K
/************************************************************************
*  sysToolsR3kExcHandle() -						*
*	Replaces the standard VxWorks exception handling, performing	*
*	the same functions, plus printing the SYMBOLIC interpretation	*
*	of the PC where the exception occured, and for the case of	*
*	exceptions at interrupt level, logging a complete message	*
*	with the above information plus all CPU registers, and		*
*	(optionally) the task running at the time of the crash.		*
*	In the future, I also intend to support customization of the	*
*	action to be taken for exceptions at interrupt level, on an	*
*	interrupt-by-interrupt and exception-by-exception basis.	*
*	At this time, the only action supported is rebooting, just	*
*	like VxWorks, so the main utility of this package is to provide	*
*	a more detailed, symbolic interpretation of the crash.		*
*	Compatibility with sigLib is maintained; if a task causes an	*
*	exception for which it has the appropriate signal handler	*
*	installed, the signal handler will be invoked, and the task	*
*	will be able to continue unless its handler suspends it.	*
************************************************************************/
void sysToolsR3kExcHandle( vector, esf_par )
  int vector;			/* exception vector number */
  unsigned long esf_par;	/* our exception stack frame */
  {

  ESFR3K *esf_s;		/* ptr to Wind River ESF struct */
  unsigned long epc;		/* grab saved EPC from exception handler */
  unsigned long cr;		/* grab saved CR from exception handler */
  unsigned long sr;		/* grab saved SR from exception handler */
  unsigned long sp;		/* grab saved SP from exception handler */
  int exccode;			/* exception code in above */
  unsigned long inst;		/* opcode of instruction from EPC */
  int bd;			/* flag from CPU CR for branch delay slot */
  unsigned long ra;		/* (possibly) address of who called us */
  unsigned long access_addr;	/* (probable) access address if applicable */
  char sym_name[MAX_SYS_SYM_LEN];	/* symbol name from symFindByValue() */
  unsigned long sym_value;	/* symbol value from "" */
  SYM_TYPE sym_type;		/* symbol type from "" */
  char sym_loc1[MAX_SYS_SYM_LEN]; 	/* where we were (from symbol table) */
  char sym_loc2[MAX_SYS_SYM_LEN];	/* (possibly) who called us */
  unsigned long diff;		/* difference from symbol found to us */
  int offender_found;		/* flag that we found offender in bus err */
  unsigned long *scan_addr;	/* scan address looking for offender above */
  int sig_num;			/* to be compatible with sigLib */
  int sig_code;			/* ditto */
  VOIDFUNCPTR sig_handler;	/* ditto */
  SIGCONTEXT passed_context;


  esf_s = (ESFR3K *) esf_par;

  /* grab EPC and CR */
  epc = *(unsigned long *)(esf_par + E_STK_EPC);
  ra  = *(unsigned long *)(esf_par + E_STK_RA);
  cr  = *(unsigned long *)(esf_par + E_STK_CAUSE);
  sr  = *(unsigned long *)(esf_par + E_STK_SR);
  sp  = *(unsigned long *)(esf_par + E_STK_SP);
  exccode = (cr & CAUSE_EXCMASK) >> CAUSE_EXCSHIFT;
  bd = (cr & CAUSE_BD) >> 31;

  /* print extra warning if we somehow get called for an interrupt */
  if( exccode == 0 )
    {
    logMsg( "CPU vector table error; this should not happen!\n" );
    }

  /* get symbol name for EPC */
  sym_loc1[0] = '\0';
  if( SYMFINDBYVALUE(sysSymTbl, epc, sym_name, (int *)(&sym_value), 
	&sym_type) == OK)
    {
    diff = epc - sym_value;
    sprintf( sym_loc1, "%s+%d", sym_name, diff );
    }
  else
    sprintf( sym_loc1, "%s", "_symbolLookupFailed+0" );

  /* get symbol name for RA (therefore possibly who called routine) */
  sym_loc2[0] = '\0';
  if( SYMFINDBYVALUE(sysSymTbl, ra, sym_name, (int *)(&sym_value), 
	&sym_type) == OK)
    {
    diff = ra - sym_value;
    sprintf( sym_loc2, "%s+%d", sym_name, diff );
    }
  else
    sprintf( sym_loc2, "%s", "_symbolLookupFailed+0" );

  /* establish access address, or 0 if not applicable */
  access_addr = 0;
  if( (exccode == CAUSE_ADEL) || (exccode == CAUSE_ADES) )
    access_addr = *(unsigned long *)(esf_par + E_STK_BADVADDR);
  if( exccode == CAUSE_IBE )
    access_addr = epc;
  if( exccode == CAUSE_DBE )
    {
    offender_found = 0;
    for( scan_addr = (unsigned long *) epc; offender_found == 1; --scan_addr )
      {
      /* get the opcode field from (possibly) offending instruction */
      inst = *scan_addr & OPCODE_MASK;
      /* NOTE!  we support LW, SW, LH, SH, LHU, SB, LB, LBU insts. only! */
      if( (inst == LW_INSTR) || (inst == SW_INSTR) ||
	  (inst == LH_INSTR) || (inst == SH_INSTR) ||
	  (inst == 0x94000000) || (inst == 0xa0000000) ||
	  (inst == 0x80000000) || (inst == 0x90000000) )
	{
	offender_found = 1;
	access_addr = (unsigned long) scan_addr;
	}
      }
    }

  /* if we are at interrupt level, log message to RAM, reboot */
  if( intContext() == TRUE )
    {
    sprintf( (char *) EXC_MSG_ADRS,
	"%s at %#x (%s)\nAccess Address=%#x\nPossible Caller=%s\n",
	EXCR3K_TXT[exccode], epc, sym_loc1, access_addr, sym_loc2 );
    bcopy( (char *) esf_par, (char *) SYSTEM_CRASH_REGS, SYSTEM_CRASH_REGS_SIZE );
    reboot(BOOT_WARM_AUTOBOOT);
    }
  /* if we are at task level, see if it is the shell:  if so, kill
     the shell to force it to restart; otherwise, if offending task has a 
     signal handler for the corresponding signal, invoke the signal handler;
     if not, log a message and suspend the task */
  else
    {
    if( (int)taskIdCurrent == shellTaskId )
      {
      printf( "\n\007%s at %#x (%s)\n", EXCR3K_TXT[exccode], epc, sym_loc1 );
      printf( "Access address (0=N/A) = %#x\n", access_addr );
      printf( "Possibly called by %#x (%s)\n", ra, sym_loc2 );
      printf( "SR=%08x, CR=%08x, SP=%08x\n", sr, cr, sp );
      kill( shellTaskId, SIGKILL );
      }
    else
      {
      sig_num = systoolsExcToSigMap[exccode].signal_number;

      sig_handler = taskIdCurrent->pSignalInfo->sigt_vec[sig_num].sa_handler;

      if( sig_handler == NULL )
	{
	printf( "\n\007%s at %#x (%s)\n", EXCR3K_TXT[exccode], epc, sym_loc1 );
	printf( "Access address (0=N/A) = %#x\n", access_addr );
	printf( "Possibly called by %#x (%s)\n", ra, sym_loc2 );
	printf( "SR=%08x, CR=%08x, SP=%08x\n", sr, cr, sp );
	printf( "Offending task (%s) suspended.\n", 
	  taskName((int)taskIdCurrent) );
	taskSuspend( (int)taskIdCurrent );
	taskIdCurrent->excInfo.vecNum = vector;
	taskIdCurrent->excInfo.epc = (INSTR *) epc;
	taskIdCurrent->excInfo.badVa = access_addr;
	taskIdCurrent->excInfo.statusReg = sr;
	taskIdCurrent->excInfo.causeReg = cr;
	taskIdCurrent->excInfo.valid = (EXC_VEC_NUM | EXC_EPC |
		EXC_ACCESS_ADDR | EXC_STATUS_REG | EXC_CAUSE_REG);
	/* update copy of task's registers */
	bcopy( (char *) &esf_s->esfRegs, (char *) &taskIdCurrent->regs, 
	  sizeof(REG_SET) );
	}
      else
	{
	sig_code = systoolsExcToSigMap[exccode].signal_code;

	passed_context.sc_onstack = 
	  taskIdCurrent->pSignalInfo->sigHandlers[sig_num].sv_flags & 
	  SV_ONSTACK;

	passed_context.sc_onstack = 
	  taskIdCurrent->pSignalInfo->sigHandlers[sig_num].sv_flags & 
	  SV_ONSTACK;

	passed_context.sc_mask =
	  taskIdCurrent->pSignalInfo->sigHandlers[sig_num].sv_mask;
	bcopy( (char *) &esf_s->esfRegs, (char *) &passed_context.sc_regs, 
	  sizeof(REG_SET) );
	(sig_handler)( sig_num, sig_code, passed_context );
	}	/* END:  if task had signal handler */
      }		/* END:  if it was not the shell */
    }
  return;
  }

#endif		/* R3000 */


#if CPU_FAMILY==MC680X0
/************************************************************************
*  sysTools68kExcHandle() -						*
*	Replaces the standard VxWorks exception handling, performing	*
*	the same functions, plus printing the SYMBOLIC interpretation	*
*	of the PC where the exception occured, and for the case of	*
*	exceptions at interrupt level, logging a complete message	*
*	with the above information plus all CPU registers, and		*
*	(optionally) the task running at the time of the crash.		*
*	In the future, I also intend to support customization of the	*
*	action to be taken for exceptions at interrupt level, on an	*
*	interrupt-by-interrupt and exception-by-exception basis.	*
*	At this time, the only action supported is rebooting, just	*
*	like VxWorks, so the main utility of this package is to provide	*
*	a more detailed, symbolic interpretation of the crash.		*
*	Compatibility with sigLib is maintained; if a task causes an	*
*	exception for which it has the appropriate signal handler	*
*	installed, the signal handler will be invoked, and the task	*
*	will be able to continue unless its handler suspends it.	*
************************************************************************/
void sysTools68kExcHandle( esf_par )
  unsigned long esf_par;
  {

  unsigned long psf;		/* previous stack frame */
  short fmtvec;			/* format and vector offset word */
  short vector;			/* vector # */
  short format;			/* format # */
  unsigned long epc;		/* exception PC */
  unsigned long rpc;		/* PC to restore (for signal handlers) */
  unsigned long access_addr;	/* (probable) access address if applicable */
  char sym_name[MAX_SYS_SYM_LEN];	/* symbol name from symFindByValue() */
  unsigned long sym_value;	/* symbol value from "" */
  SYM_TYPE sym_type;		/* symbol type from "" */
  char sym_loc1[MAX_SYS_SYM_LEN]; 	/* where we were (from symbol table) */
  unsigned long diff;		/* difference from symbol found to us */
  int sig_num;			/* to be compatible with sigLib */
  int sig_code;			/* ditto */
  VOIDFUNCPTR sig_handler;	/* ditto */
  SIGCONTEXT passed_context;


/* 68000/8 not currently supported! */
#if( (CPU == MC68000) || (CPU == MC68008) )
#else
  fmtvec = *((short *)((char *)esf_par + 6));
  vector = (short) IVEC_TO_INUM( fmtvec & 0x0fff );
  format = (fmtvec & 0xf000) >> 12;
  epc = access_addr = rpc = 0;
  switch( format )
    {
    /* 0 & 1 are programmatically identical */
    case 0x0:
    case 0x1:
      epc = rpc = (unsigned long)((ESF0 *)(esf_par))->pc;
      psf = esf_par + sizeof(ESF0);
      break;
    case 0x2:
      epc = (unsigned long)((ESF2 *)esf_par)->aa;
      rpc = (unsigned long)((ESF2 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF2);
      break;
    case 0x3:
      epc = (unsigned long)((ESF3 *)esf_par)->effectiveAddr;
      rpc = (unsigned long)((ESF3 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF3);
      break;
    case 0x4:
      epc = (unsigned long)((ESF4 *)esf_par)->aa;
      rpc = (unsigned long)((ESF4 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF4);
      break;
    case 0x7:
      epc = (unsigned long)((ESF7 *)esf_par)->effectiveAddr;
      rpc = (unsigned long)((ESF7 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF7);
      access_addr = (unsigned long)((ESF7 *)esf_par)->aa;
      ((ESF7 *)esf_par)->fn &= ~0x100;
      break;
    case 0x8:
      epc = (unsigned long)((ESF8 *)esf_par)->aa;
      rpc = (unsigned long)((ESF8 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF8);
      ((ESF8 *)esf_par)->fn &= ~0x100;
      break;
    case 0x9:
      epc = (unsigned long)((ESF9 *)esf_par)->aa;
      rpc = (unsigned long)((ESF9 *)esf_par)->pc;
      psf = esf_par + sizeof(ESF9);
      break;
    case 0xa:
      epc = rpc = (unsigned long)((ESFA *)esf_par)->pc;
      psf = esf_par + sizeof(ESFA);
      access_addr = (unsigned long)((ESFA *)esf_par)->aa;
      ((ESFA *)esf_par)->fn &= ~0x100;
      break;
    case 0xb:
      epc = rpc = (unsigned long)((ESFB *)esf_par)->pc;
      psf = esf_par + sizeof(ESFB);
      access_addr = (unsigned long)((ESFB *)esf_par)->aa;
      ((ESFB *)esf_par)->fn &= ~0x100;
      break;
    case 0xc:
      epc = rpc = (unsigned long)((ESFC *)esf_par)->pc;
      psf = esf_par + sizeof(ESFC);
      access_addr = (unsigned long)((ESFC *)esf_par)->aa;
      ((ESFC *)esf_par)->fn &= ~0x100;
      break;
    default:
      break;
    }

  /* get symbol name for EPC */
  sym_loc1[0] = '\0';
  if( SYMFINDBYVALUE(sysSymTbl, epc, sym_name, (int *)(&sym_value), 
	&sym_type) == OK)
    {
    diff = epc - sym_value;
    sprintf( sym_loc1, "%s+%#x", sym_name, diff );
    }
  else
    sprintf( sym_loc1, "%s", "_symbolLookupFailed+0" );

  /* if we are at interrupt level, log message to RAM, reboot */
  if( intContext() == TRUE )
    {
    sprintf( (char *) EXC_MSG_ADRS,
	"%s at %#x (%s)\nAccess Address=%#x\n",
	EXC68K_TXT[vector-2], epc, sym_loc1, access_addr );
    bcopy( (char *)(esf_par - 64), (char *) SYSTEM_CRASH_REGS, 
	SYSTEM_CRASH_REGS_SIZE );
    /* we make the assumption that it is too dangerous to attempt a
	stack trace in this situation, because of the risk of a double
	bus fault halting the CPU (if the stack is corrupt) */
    reboot(BOOT_WARM_AUTOBOOT);
    }
  /* if we are at task level, see if it is the shell:  if so, kill
     the shell to force it to restart; otherwise, if offending task has a 
     signal handler for the corresponding signal, invoke the signal handler;
     if not, log a message and suspend the task */
  else
    {
    if( (int)taskIdCurrent == shellTaskId )
      {
      printf( "\n\007%s at %#x (%s)\n", EXC68K_TXT[vector-2], epc, sym_loc1 );
      printf( "Access address (0=N/A) = %#x\n", access_addr );
      printf( "Traceback follows:\n" );
      /* killing shell does automatic traceback */
      kill( shellTaskId, SIGKILL );
      }
    else
      {
      sig_num = systoolsExcToSigMap[vector-2].signal_number;

      sig_handler = taskIdCurrent->pSignalInfo->sigt_vec[sig_num].sa_handler;

      if( sig_handler == NULL )
	{
	printf( "\n\007%s at %#x (%s)\n", EXC68K_TXT[vector-2], epc, sym_loc1 );
	printf( "Access address (0=N/A) = %#x\n", access_addr );
	esfShow( (unsigned long *)(esf_par - 64) );
	printf( "Traceback from %#x follows:\n", psf );

#ifdef VXWORKS_5_0_2b
	trcStack( (int *)(*((int *)(esf_par - 8))), (int *)psf,
	  (INSTR *)epc, sysTools68kTrcPrint );
#else
	printf ("\n trcStack has changed for VxWorks 5.1.1 \n");
#endif
	taskIdCurrent->excInfo.vecNum = vector;
	taskIdCurrent->excInfo.pc = (INSTR *) epc;
	taskIdCurrent->excInfo.accessAddr = access_addr;
	taskIdCurrent->excInfo.statusReg = *(short *)esf_par;
	taskIdCurrent->excInfo.valid = (EXC_VEC_NUM | EXC_PC |
		EXC_ACCESS_ADDR | EXC_STATUS_REG);
	printf( "Offending task (%s) suspended.\n", 
	  taskName((int)taskIdCurrent) );
	taskSuspend( (int)taskIdCurrent );
/* these have no effect for some reason:
	bcopy( (char *)(esf_par - 64), (char *)&(taskIdCurrent->regs.dataReg),
	  64 );
	taskIdCurrent->regs.pc = (INSTR *)epc;
*/
	}
      else
	{
	sig_code = systoolsExcToSigMap[vector-2].signal_code;

	passed_context.sc_onstack = 
	  taskIdCurrent->pSignalInfo->sigt_vec[sig_num].sa_flags & 
	  SV_ONSTACK;

	passed_context.sc_mask =
	  taskIdCurrent->pSignalInfo->sigt_vec[sig_num].sa_mask;

	passed_context.sc_regs.pc = (INSTR *) rpc;
	/* SR is first word in all stack frame formats */
	passed_context.sc_regs.sr = *(short *)esf_par;
	bcopy( (char *)(esf_par - 64),
	  (char *)&(passed_context.sc_regs.dataReg), 64 );
	(sig_handler)( sig_num, sig_code, passed_context );
	}	/* END: whether task had signal handler or not */
      }		/* END:  if it was the shell */
    }
#endif		/* greater than 68000/8 */
  return;
  }



/************************************************************************
*  sysTools68kTrcPrint() -						*
*	Until the task-stack-wrecking bug is fixed, this routine is	*
*	used along with trcStack() to do a symbolic traceback on an	*
*	exception.  The bug is that after the offending task is		*
*	suspended, its stack pointer ends up in the wrong place and	*
*	a "tt" on it produces a traceback of the routine calls that	*
*	this software package (sysTools) made, not those of the task.	*
************************************************************************/
VOID sysTools68kTrcPrint( callAdrs, rtnAdrs, nargs, args )
  INSTR *callAdrs;  /* address from which routine was called */
  int rtnAdrs;      /* address of routine called */
  int nargs;        /* number of arguments in call */
  int *args;        /* pointer to arguments */

  {
  char sym_name[MAX_SYS_SYM_LEN];	/* symbol name from symFindByValue() */
  unsigned long sym_value;	/* symbol value from "" */
  SYM_TYPE sym_type;		/* symbol type from "" */
  unsigned long diff;		/* difference between symbol found and us */
  int arg;


  sym_name[0] = '\0';
  diff = 0;


  if( SYMFINDBYVALUE(sysSymTbl, (int)callAdrs, sym_name, (int *)(&sym_value), 
    &sym_type) == OK)
    {
    diff = (unsigned long)callAdrs - sym_value;
    if( diff > 0 )
      printf( "%s+%#x: ", sym_name, diff );
    else
      printf( "%s: ", sym_name );
    }
  else
    printf( "(%#x): ", callAdrs );

  sym_name[0] = '\0';
  diff = 0;
  if( SYMFINDBYVALUE(sysSymTbl, rtnAdrs, sym_name, (int *)(&sym_value), 
    &sym_type) == OK)
    {
    diff = (unsigned long)rtnAdrs - sym_value;
    if( diff > 0 )
      printf( "%s+%#x (", sym_name, diff );
    else
      printf( "%s (", sym_name );
    }
  else
    printf( "(%#x) (", rtnAdrs );

  if( nargs == 0 )
    printf( ")\n" );
  for( arg = 0; arg < nargs; arg++ )
    {
    if( arg == (nargs-1) )
      printf( "%#x)\n" );
    else
      printf( "%#x, ", args[arg] );
    }

  return;
  }

#endif		/* M68k */

#endif		/* exceptions */

