/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1991 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/* ==========================================================================
**
** DESCRIPTION
**	Assembly routines for sysTools/68k.
**
** DEVELOPERS
**      Jim Pangburn
**
** MODIFICATIONS - modification history (if any) should be kept in source
**                 code management system. (don't clutter this file)
**
*/

/*
SYNOPSIS:

No routines contained in this library are intended to be user-callable.
*/

/*
#ifdef VXWORKS_5_0_2b
#define ASMLANGUAGE
#else
#define _ASMLANGUAGE
#endif
*/

#define _ASMLANGUAGE
#include "vxWorks.h"

#if CPU_FAMILY==MC680X0

#if CPU==MC68000 || CPU == MC68008
/*
68000 & 68008 use 2 possible primitive stack frame types, depending
upon vector, so the appropriate routine must be attached to the appropriate
vector type, since there is no "type" field in these frames for a routine
to use to distinguish them.
*/

	/* internals */

	.globl  _sysToolsGenExcStub
	.globl  _sysToolsAccessExcStub

	/* externals */

	.text

_sysToolsGenExcStub:
	moveml	d0-d7/a0-a7,sp@-	/* save scratch regs for C routines */
	pea	a7@(64)			/* push exception stack frame */
	jsr	_sysTools68kGenExc	/* call C exception handler */
	moveml	sp@+,d0-d7/a0-a7	/* restore regs */
	rte				/* return from exception */

_sysToolsAccessExcStub:
	moveml	d0-d7/a0-a7,sp@-	/* save scratch regs for C routines */
	pea	a7@(64)			/* push exception stack frame */
	jsr	_sysTools68kAccessExc	/* call C exception handler */
	moveml	sp@+,d0-d7/a0-a7	/* restore regs */
	rte				/* return from exception */

#else

/*
68010 & higher have a "type" field in their stack frames, so a routine
need only read this to determine what type of exception occured.  In fact,
in the Motorola manuals, it says that routines should be able to handle
all possible stack frame types, and should not make assumptions about
a certain exception generating a certain frame.  This was basically due
to the fact that Motorola realized that as pipeline stages grew deeper,
the possibility for more complex stack frame types to be generated
depending upon pipeline filling would be a factor.
*/

	/* internals */

	.globl  _sysToolsExcStub

	/* externals */

	.text
	.even

_sysToolsExcStub:
	moveml	d0-d7/a0-a7,sp@-	/* save scratch regs for C routines */
	moveal	a7,a5			/* save current stack pointer */
	pea	a5@(64)			/* push exception stack frame */
	jsr	_sysTools68kExcHandle	/* call C exception handler */
	moveal	a5,a7			/* restore stack pointer */
	moveml	sp@+,d0-d7/a0-a7	/* restore regs */
	rte				/* return from exception */

#endif		/* CPU */

#endif		/* CPU_FAMILY==MC680X0 */
