/* =================================================================== *
   Test to handle task variables using dvx task variable package.
   Please notice that due to the use of task variables, the test
   must be spawned and not run interactively.

   Two exit handlers are registered; then three DVX task variables 
   are created.

 *  ================================================================== */
#include "dvx.h"

/* Prototypes		*/

int  testdvx(void);

/* Exit handlers	*/

void exHand1 (void);
int  exHand2 (int);

int  scalarHandle;

/*==================================================================== */
/* This exit handler is registered using atexit			       */

void exHand1()
{
	printf ("\n exHand1: Hello from exiting \n");
	return;
}

/* =================================================================== */
/* This exit handler accepts one argument and it is registered using
   dvx_atexit, an extension of the standard atexit that allows to pass
   one argument.                                                       */
 
int exHand2 (int tvar)
{
  STATUS ret_status;
  int    rd_var;
 
        printf ("\n exHand2: Hello from exiting task Id = 0x%x\n", tvar);

/* Accessing a task variable */

        ret_status = dvx_get_scalar (tvar, scalarHandle,&rd_var);
        printf ("\n exHand2: tid = 0x%x, data read = 0x%x\n",tvar, rd_var);
 
        return (OK);
}
 
/* =================================================================== */
/* Main program 	*/

int testdvx()
{

   STATUS ret_status;
   int ret_atex; 

   int  tId;
   int	dat;

   int  ptrHandle;
   int  newptrHandle;

   char *malptr;
   char *real_ptr;

   int  malsiz = 16;	/* In bytes */

   int  **rdptr;

   int  rdVar;
   int  i;


/* ---------------------------------------------------- */

/* Getting task id					*/

   tId = taskIdSelf();

/* Registering two exit handlers			*/

   ret_atex = atexit (exHand1);
   if ( ret_atex != 0)
	printf ("\n ====== Unable to register function, table is full\n");

   ret_atex = dvx_atexit (exHand2, tId);
   if ( ret_atex != 0)
	printf ("\n ====== Unable to register function, table is full\n");


/* Creating a task variable of type pointer: dvx malloc the space needed for 
   the user. The space will be freed either if the task variable is deleted
   by the user or when the task exits					   */

   ret_status = dvx_create_ptr (tId, &ptrHandle, malsiz, (DVX_PTR **)&malptr);
   if (ret_status != OK) 
      {printf ("\n testdvx: Error in creating pointer malptr\n"); }
   else
   {
      printf ("\n testdvx: handle ptr = 0x%x ,malptr = 0x%x \n", 
		ptrHandle,(int)malptr);
/*       for (i=0;i<malsiz;i++)     malptr[i]=(char)i;  */
   }

   ret_status = dvx_get_ptr (tId, ptrHandle, (DVX_PTR **)&rdptr);
   if (ret_status != OK)
   {
        printf ("\n testdvx:  Error in retrieving pointer = %d \n", ret_status);
        return (ret_status);
   }
   else
   {
        printf ("\n testdvx:  retrieving pointer = 0x%x ,handle = %d\n", rdptr,ptrHandle );

   }


/* Creating a task variable of type scalar, setting the value 
   and then retrieving it.  					*/

   dat = 0xabab;

   ret_status = dvx_create_scalar (tId, &scalarHandle);
   if (ret_status != OK)
   {
        printf ("\n testdvx: Error in creating scalar\n");
        return (ret_status);
   }

   ret_status = dvx_set_scalar (tId, scalarHandle, dat);

   ret_status = dvx_get_scalar (tId, scalarHandle, &rdVar);

   printf ("\n testdvx: rdVar = 0x%x\n",rdVar);

   printf ("\n Creating another pointer \n");

   ret_status = dvx_create_ptr (tId, &newptrHandle, 0, (DVX_PTR **)&real_ptr);
   if (ret_status != OK) 
      {printf ("\n testdvx: Error in creating pointer real_ptr\n"); }

   ret_status = dvx_var_info ( tId);
   if (ret_status != OK)
   {
      printf ("\n testdvx: error from dvx_var_info = %d\n",ret_status);
      return(ret_status);
   }

/*
   printf ("\n testdvx: Deleting POINTER task variable \n");

   ret_status = dvx_delete_taskvar ( tId, ptrHandle);
   if (ret_status != OK)
   {
      printf ("\n testdvx: error, non existing handle \n");
      return(ret_status);
   }
*/

/* 
   To see how to use dvx_var_info from the vxwork shell 
   uncomment the following lines; resume the task then
   from the shell and the exit handlers will be then
   executed.

*/

/*   taskSuspend(tId); */

   printf ("\n *** Filling malloc space now \n");
   for (i=0;i<malsiz;i++)     malptr[i]=(char)i; 

   ret_status = dvx_var_info ( tId);
   if (ret_status != OK)
   {
      printf ("\n testdvx: error from dvx_var_info = %d\n",ret_status);
      return(ret_status);
   }


   return (OK);
}    
