/************************************************************************/
/* File: tod_test.c							*/
/*									*/
/* User Set/Get Time-Of-Day and timezone information			*/
/*									*/
/*
** Modifications
** 17dec93	das	changed to include of VxWorks time.h instead of
**			private timevw.h
*/

/*
SCCS release:
@(#) tod_test.c 1.8 Delta: 94/01/12 12:34:11 Extraction: 94/01/13 11:48:49 @(#)
*/

#include <vxWorks.h>			/* VxWorks types		*/
#ifndef MIPS_R3000
#include <stdioLib.h>			/* VxWorks standard I/O library	*/
#else
#include <fioLib.h>
#endif

#include <time.h>			/* VxWorks time defines 	*/

#include "tod_time.h"			/* time structures and defs.	*/
#include "tod_prototypes.h"		/* tod_ function prototypes   	*/

#ifndef INCtod_prototypesh
/* prototypes */

extern int      tod_show_tzrules(void);
extern int      tod_get_tzrules(void);
extern int      tod_set_tzrules(INT32 tz_hhmm,
                                INT32 atz_hhmm,
                                INT32 atzBegin_mmddhhmmd,
                                INT32 atzEnd_mmddhhmmd,
                                char *tz_name,
                                char *atz_name);
extern int      tod_setgmt(INT32 yymmdd, INT32 hhmmss);
extern int      tod_setlocal(INT32 yymmdd, INT32 hhmmss);
#endif /* INCtod_prototypesh */


int tod_test(void);

/************************************************************************/
/*									*/
/* ROUTINE TOD_SET_TZRULES	Assign timezone rules and names		*/
/*									*/
/* Arguments:								*/
/*									*/
/*	(in)	timezone_hhmm	INT32 timezone (+E/-W) Hour Minute	*/
/*	(in)	altzone_hhmm	INT32 alttimezone (+E/-W) Hour Minute	*/
/*	(in)	altzoneBegin_mmddhhmmd	INT32 Month Day Hour Minute DoW	*/
/*	(in)	altzoneEnd_mmddhhmmd	INT32 Month Day Hour Minute DoW	*/
/*	(in)	*timezone_name	char	"nnn" 3 character description	*/
/*	(in)	*altzone_name	char	"nnn" 3 character description	*/
/*									*/
/* Return:								*/
/*		 0 Success						*/
/*		-1 Invalid						*/
/*									*/
/* Timezone Values and Rules: (decimal)					*/
/*									*/
/* tod_set_tzrules(	INT32 timezone_hhmm,				*/
/*			INT32 altzone_hhmm,				*/
/*			INT32 altzoneBegin_mmddhhmmd,			*/
/*			INT32 altzoneEnd_mmddhhmmd,			*/
/*			char *timezone_name,				*/
/*			char *altzone_name);				*/
/*									*/
/* Zone Format:  +/- hhmm (+East, -West GMT)				*/
/* 			hh = hours	(0-23)				*/
/*			mm = minutes	(00-59)				*/
/*									*/
/* Conditional Format:	altzoneBegin/altzoneEnd mmddhhmmd		*/
/*			mm = Month	(1 - 12)			*/
/*			dd = Day	(01 - 31) / 00,01 Conditional	*/
/*			hh = Hour	(00 - 23)			*/
/*			mm = Minute	(00 - 59)			*/
/*			 d = DayOfWeek	0=Exact Date / (1 - 7)Condition	*/
/*					1=Sunday, 7=Saturday		*/
/*									*/
/* Format:								*/
/* mmddhhmmd	Month, Day, Hour, Minutes, 0	Exact Day (1-31)	*/
/* mmddhhmmd	Month,  00, Hour, Minutes, 1 	Condition First Sunday	*/
/* mmddhhmmd	Month,  01, Hour, Minutes, 1	Condition Last Sunday	*/
/* mmddhhmmd	Month,  00, Hour, Minutes, 0 	IGNORE rules (GMT only)	*/
/*									*/
/* Example: tod_set_tzrules(-600,-500,40002001,100102001,"CST","CDT");	*/
/*	timezone = GMT - 6hr, 0min;	tzname   = CST			*/
/*	altzone  = GMT - 5hr, 0min;	altzname = CDT			*/
/*									*/
/* Example: tod_set_tzrules(0,0,0,0,"GMT","  ");			*/
/*	timezone = GMT time;		tzname   = GMT			*/
/*	altzone  = none;		altzname = none			*/
/*									*/
/*									*/

/************************************************************************/
/*									*/
/* ROUTINE TOD_TEST 		Test Tod device, Zones and Rules	*/
/*									*/
/* Arguments:								*/
/*									*/
/*		none							*/
/* Return:								*/
/*									*/
/*	 0 = Success							*/
/*									*/
int tod_test(void)
{
	INT32	yymmdd = 930404;
	INT32	hhmmss = 143000;

	INT32	tz_hhmm = -600;
	INT32	atz_hhmm = -500;
	INT32	atzBegin_mmddhhmmd = 40002001;
	INT32	atzEnd_mmddhhmmd = 100102001;
	char	*tz_name ="CST";
	char	*atz_name ="CDT";


	struct tm	*tmnow;
	time_t		seconds;
	char		*asciiseconds;
	char		*asciitm;
	


printf("SHOWTZRULES:\n");

	tod_show_tzrules();

printf("SETTZRULES:\n");

	tod_set_tzrules(tz_hhmm, atz_hhmm,
			atzBegin_mmddhhmmd,
			atzEnd_mmddhhmmd,
			tz_name, atz_name);

printf("GETTZRULES:\n");

	tod_get_tzrules();

printf("SETLOCAL:\n");

	tod_setlocal( yymmdd, hhmmss);

printf("SETGMT:\n");

	tod_setgmt( yymmdd, hhmmss);

	time(&seconds);

	asciiseconds = ctime(&seconds);

	printf("SECONDS SINCE JAN1,1970 %d = %s\n",seconds, asciiseconds);

	tmnow = localtime(&seconds);

	printf("LOCALTIME() YDay:%d   %02d-%02d-%02d %02d:%02d:%02d  isDST:%d\n",
		tmnow->tm_yday, tmnow->tm_mon,tmnow->tm_mday, tmnow->tm_year,
		tmnow->tm_hour,tmnow->tm_min,tmnow->tm_sec, tmnow->tm_isdst);

	asciitm = asctime(tmnow);
 
	printf("TIME AGAIN: %s",asciitm);

	printf("MKTIME %d\n",mktime(tmnow));

	printf("MKTIME(Local) YDay:%d   %02d-%02d-%02d %02d:%02d:%02d  isDST:%d\n",
		tmnow->tm_yday, tmnow->tm_mon,tmnow->tm_mday, tmnow->tm_year,
		tmnow->tm_hour,tmnow->tm_min,tmnow->tm_sec, tmnow->tm_isdst);

	tmnow = gmtime(&seconds);

	printf("GMTIME() YDay:%d   %02d-%02d-%02d %02d:%02d:%02d  isDST:%d\n",
		tmnow->tm_yday, tmnow->tm_mon,tmnow->tm_mday, tmnow->tm_year,
		tmnow->tm_hour,tmnow->tm_min,tmnow->tm_sec, tmnow->tm_isdst);

	asciitm = asctime(tmnow);
 
	printf("TIME AGAIN: %s",asciitm);

	printf("MKTIME %d\n",mktime(tmnow));

	printf("MKTIME(GMT) YDay:%d   %02d-%02d-%02d %02d:%02d:%02d  isDST:%d\n",
		tmnow->tm_yday, tmnow->tm_mon,tmnow->tm_mday, tmnow->tm_year,
		tmnow->tm_hour,tmnow->tm_min,tmnow->tm_sec, tmnow->tm_isdst);


	return (OK);	
} /* tod_test */

/* file: tod_test.c */
