/****************************** Copyright Notice ******************************
 *                                                                            *
 * Copyright (c) 1991 Universities Research Association, Inc.                 *
 *               All Rights Reserved.                                         *
 *                                                                            *
 ******************************************************************************/
/* @(#) vx.c 1.5 Delta: 94/01/10 14:56:27 Extraction: 94/01/11 13:25:40 @(#) */

/* ==========================================================================  
**                                                                             
** DESCRIPTION                                                                 
**	routines that extend vxworks standard functions
**                                                                             
** DEVELOPERS                                                                  
**	Margaret Votava
**                                                                             
**      Fermilab Data Acquisition Group                                        
**      Batavia, Il 60510, U.S.A.                                              
**                                                                             
** ENTRY POINTS                 	SCOPE  
**	vx_memmatch			global
**                                                                             
** MODIFICATIONS                                                               
**         Date       Initials  Description                                    
**	20dec93		das	cast printf args
**	10jan94		das	cast != expression, added header files
**
** HEADER STATEMENTS                                                           
*/                                                                             

#include <vxWorks.h>
#ifdef MIPS_R3000
#include <fioLib.h>
#else
#include <stdioLib.h>
#endif

/* ==========================================================================  
**                                                                             
** ROUTINE: vx_memmatch
**                                                                             
** DESCRIPTION                                                                 
**                                                                             
**  will search the specified memory segment for a specific byte, word,
**  or longword. 
**
** ARGUMENTS
**
** char		   type		READ_ONLY. Type of pattern to search for.
**				'b' for byte
**				'w' for word
**				'l' for longword
**
** char 	   *start_addr	READ_ONLY. Start address of memory to search.
**
** unsigned long   size		READ_ONLY. Number of addresses to search, based 
**				on search type. If search type is bytes, it is 
**				the number of bytes. If type is word, then 
**				words, etc.
**
** unsigned long   pattern	READ_ONLY. Pattern to match.
**
** VALUES RETURNED
**
**	0	if pattern is found
**	1 	if pattern is not found 
**	-1	if type is invalid
**
** ==========================================================================  
*/                                                                             

int vx_memmatch (char type, 		/* r - type of search */
		 void *start_addr,	/* r - starting address of memory to search */
		 unsigned long size, 	/* r - number of addresses to search */
		 unsigned long pattern)	/* r - pattern to search for */
{
/* local variables
   --------------- */
char		*byte_addr;	/* byte starting address */
unsigned short 	*word_addr;	/* word starting address */
unsigned long	*long_addr;	/* long starting address */	
int		i;		/* counter */
int		status = 1;	/* function status */

/* make sure this is a valid type 
   ------------------------------ */
 
if (((int)type != (int)'b') && ((int)type != (int)'w') && 
				((int)type != (int)'l')) return -1;
	
printf ("searching %x (hex) addresses for %x (hex), starting at %x (hex)\n", 
	size, pattern, (unsigned int)start_addr);

/* search through the whole array
   ------------------------------ */

for (i = 0, byte_addr = (char *)	   start_addr, 
            word_addr = (unsigned short *) start_addr, 
	    long_addr = (unsigned long *)  start_addr; i < size; i++)
   {
   switch (type) {
	case 'b':			/* looking for a byte */
	   {
	   if (*byte_addr == pattern) 
		{
		printf ("byte found at address %x \n", (unsigned int)byte_addr);
		status = 0;
		}
	   byte_addr++;
	   break;
	   }
	case 'w':			/* looking for a word */
	   {
	   if (*word_addr == pattern) 
		{
		printf ("word found at address %x \n",
				(unsigned int)word_addr);
		status = 0;
		}
	   word_addr++;
	   break;
	   }
	case 'l':			/* looking for a longword */
	   {
	   if (*long_addr == pattern) 
		{
		printf ("long found at address %x \n",
				(unsigned int)long_addr);
		status = 0;
		}
	   long_addr++;
	   break;
	   }
	}
   }

return status;

} /* end vm_memmatch */
